@@ -6,7 +6,6 @@
 #include <string.h>
 #include <inttypes.h>
 #include <sys/types.h>
-#include <linux/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -18,16 +17,14 @@
 #include <sys/ioctl.h>
 #include <sys/mount.h>
 #include <dirent.h>
+#include <libgfs2.h>
 
-#include <linux/gfs2_ondisk.h>
 #include "copyright.cf"
 
 #include "hexedit.h"
-#include "libgfs2.h"
 #include "extended.h"
 #include "gfs2hex.h"
-
-extern uint64_t block;
+#include "struct_print.h"
 
 static void print_block_details(struct iinfo *ind, int level, int cur_height,
 				int pndx, uint64_t file_offset);
@@ -61,11 +58,11 @@ static int _do_indirect_extended(char *diebuf, struct iinfo *iinf, int hgt)
 		iinf->ii[x].height = 0;
 		iinf->ii[x].block = 0;
 		iinf->ii[x].dirents = 0;
-		memset(&iinf->ii[x].dirent, 0, sizeof(struct gfs2_dirents));
+		memset(&iinf->ii[x].dirent, 0, sizeof(struct idirent));
 	}
 	headoff = sbd.gfs1 ? sizeof(struct gfs_indirect) : sizeof(struct gfs2_meta_header);
-	for (x = headoff, y = 0; x < sbd.bsize; x += sizeof(uint64_t), y++) {
-		p = be64_to_cpu(*(uint64_t *)(diebuf + x));
+	for (x = headoff, y = 0; x < sbd.sd_bsize; x += sizeof(uint64_t), y++) {
+		p = be64_to_cpu(*(__be64 *)(diebuf + x));
 		if (p) {
 			iinf->ii[i_blocks].block = p;
 			iinf->ii[i_blocks].mp.mp_list[hgt] = i_blocks;
@@ -100,18 +97,19 @@ static int dinode_valid(void)
 	return 0;
 }
 
-static uint64_t metapath_to_lblock(struct metapath *mp, int hgt)
+static uint64_t metapath_to_lblock(struct lgfs2_metapath *mp, int hgt)
 {
 	int h;
 	uint64_t lblock = 0;
 	uint64_t factor[GFS2_MAX_META_HEIGHT];
+	uint16_t height = be16_to_cpu(di->di_height);
 
-	if (di.di_height < 2)
+	if (height < 2)
 		return mp->mp_list[0];
 	/* figure out multiplication factors for each height */
 	memset(&factor, 0, sizeof(factor));
-	factor[di.di_height - 1] = 1ull;
-	for (h = di.di_height - 2; h >= 0; h--)
+	factor[height - 1] = 1ull;
+	for (h = height - 2; h >= 0; h--)
 		factor[h] = factor[h + 1] * sbd.sd_inptrs;
 	for (h = 0; h <= hgt; h++)
 		lblock += (mp->mp_list[h] * factor[h]);
@@ -129,7 +127,7 @@ static int display_indirect(struct iinfo *ind, int indblocks, int level,
 		return -1;
 	if (!level) {
 		if (gfs2_struct_type == GFS2_METATYPE_DI) {
-			if (S_ISDIR(di.di_mode))
+			if (S_ISDIR(be32_to_cpu(di->di_mode)))
 				print_gfs2("This directory contains %d indirect blocks",
 					   indblocks);
 			else
@@ -139,14 +137,14 @@ static int display_indirect(struct iinfo *ind, int indblocks, int level,
 			print_gfs2("This indirect block contains %d indirect blocks",
 				   indblocks);
 	}
-	if (dinode_valid() && !S_ISDIR(di.di_mode)) {
+	if (dinode_valid() && !S_ISDIR(be32_to_cpu(di->di_mode))) {
 		/* See if we are on an inode or have one in history. */
 		if (level)
 			cur_height = level;
 		else {
 			cur_height = get_height();
 			print_gfs2("  (at height %d of %d)",
-				   cur_height, di.di_height);
+			           cur_height, be16_to_cpu(di->di_height));
 		}
 	}
 	eol(0);
@@ -184,20 +182,20 @@ static int display_indirect(struct iinfo *ind, int indblocks, int level,
 			if (edit_row[dmode] >= 0 &&
 			    line - start_line ==
 			    edit_row[dmode] - start_row[dmode]) { 
-				sprintf(estring, "%llx",
-					(unsigned long long)ind->ii[print_entry_ndx].block);
-				strcpy(edit_fmt, "%llx");
+				sprintf(estring, "%"PRIx64,
+				        ind->ii[print_entry_ndx].block);
+				strcpy(edit_fmt, "%"PRIx64);
 				edit_size[dmode] = strlen(estring);
 				COLORS_NORMAL;
 			}
 		}
-		if (dinode_valid() && !S_ISDIR(di.di_mode)) {
+		if (dinode_valid() && !S_ISDIR(be32_to_cpu(di->di_mode))) {
 			float human_off;
 			char h;
 
 			file_offset = metapath_to_lblock(&ind->ii[pndx].mp,
 							 cur_height) *
-				sbd.bsize;
+				sbd.sd_bsize;
 			print_gfs2("     ");
 			h = 'K';
 			human_off = (file_offset / 1024.0);
@@ -213,8 +211,8 @@ static int display_indirect(struct iinfo *ind, int indblocks, int level,
 		else
 			file_offset = 0;
 		if (dinode_valid() && !termlines &&
-		    ((level + 1 < di.di_height) ||
-		     (S_ISDIR(di.di_mode) && level <= di.di_height))) {
+		    ((level + 1 < be16_to_cpu(di->di_height)) ||
+		     (S_ISDIR(be32_to_cpu(di->di_mode)) && level <= be16_to_cpu(di->di_height)))) {
 			print_block_details(ind, level, cur_height, pndx,
 					    file_offset);
 		}
@@ -231,7 +229,7 @@ static int display_indirect(struct iinfo *ind, int indblocks, int level,
 	return 0;
 }
 
-static void print_inode_type(__be16 de_type)
+static void print_inode_type(uint16_t de_type)
 {
 	if (sbd.gfs1) {
 		switch(de_type) {
@@ -296,15 +294,6 @@ static void print_inode_type(__be16 de_type)
 	}
 }
 
-#ifdef GFS2_HAS_LEAF_HINTS
-#define LEAF_HINT_FMTS "lf_inode: 0x%llx, lf_dist: %u, " \
-                       "lf_nsec: %u, lf_sec: %llu, "
-#define LEAF_HINT_FIELDS(lp) lp->lf_inode, lp->lf_dist, lp->lf_nsec, lp->lf_sec,
-#else
-#define LEAF_HINT_FMTS
-#define LEAF_HINT_FIELDS(lp)
-#endif
-
 static int display_leaf(struct iinfo *ind)
 {
 	struct gfs2_leaf *leaf = &ind->ii[0].lf;
@@ -315,11 +304,13 @@ static int display_leaf(struct iinfo *ind)
 	if (gfs2_struct_type == GFS2_METATYPE_SB)
 		print_gfs2("The superblock has 2 directories");
 	else
-		print_gfs2("Directory block: lf_depth:%d, lf_entries:%d, "
-		           LEAF_HINT_FMTS
-			   "fmt:%d next=0x%llx (%d dirents).",
-			   leaf->lf_depth, leaf->lf_entries,
-		           LEAF_HINT_FIELDS(leaf)
+		print_gfs2("Directory block: "
+		           "lf_depth:%"PRIu16", lf_entries:%"PRIu16", lf_inode: 0x%"PRIx64", "
+		           "lf_dist: %"PRIu32", lf_nsec: %"PRIu32", lf_sec: %"PRIu64", "
+			   "fmt:%"PRIu32" next=0x%"PRIx64" (%d dirents).",
+			   be16_to_cpu(leaf->lf_depth), be16_to_cpu(leaf->lf_entries),
+		           be64_to_cpu(leaf->lf_inode), be32_to_cpu(leaf->lf_dist),
+		           be32_to_cpu(leaf->lf_nsec), be64_to_cpu(leaf->lf_sec),
 			   leaf->lf_dirent_format,
 			   leaf->lf_next,
 			   ind->ii[0].dirents);
@@ -337,26 +328,21 @@ static int display_leaf(struct iinfo *ind)
 				    line - start_line - 1 ==
 				    edit_row[dmode] - start_row[dmode]) {
 					COLORS_HIGHLIGHT;
-					sprintf(estring, "%llx",
-						(unsigned long long)ind->ii[0].dirent[d].block);
+					sprintf(estring, "%"PRIx64, ind->ii[0].dirent[d].inum.in_addr);
 					strcpy(edit_fmt, "%llx");
 				}
 			}
-			print_gfs2("%d/%d [%08x] %lld/%"PRId64" (0x%llx/0x%"PRIx64") +%u: ",
+			print_gfs2("%d/%d [%08"PRIX32"] %"PRIu64"/%"PRIu64" (0x%"PRIx64"/0x%"PRIx64") +%"PRIu16": ",
 				   total_dirents, d + 1,
-				   ind->ii[0].dirent[d].dirent.de_hash,
-				   ind->ii[0].dirent[d].dirent.de_inum.no_formal_ino,
-				   ind->ii[0].dirent[d].block,
-				   ind->ii[0].dirent[d].dirent.de_inum.no_formal_ino,
-				   ind->ii[0].dirent[d].block,
-#ifdef GFS2_HAS_DE_RAHEAD
-				   (unsigned int)ind->ii[0].dirent[d].dirent.de_rahead
-#else
-				   0
-#endif
+				   ind->ii[0].dirent[d].hash,
+				   ind->ii[0].dirent[d].inum.in_formal_ino,
+				   ind->ii[0].dirent[d].inum.in_addr,
+				   ind->ii[0].dirent[d].inum.in_formal_ino,
+				   ind->ii[0].dirent[d].inum.in_addr,
+				   ind->ii[0].dirent[d].rahead
 			);
 		}
-		print_inode_type(ind->ii[0].dirent[d].dirent.de_type);
+		print_inode_type(ind->ii[0].dirent[d].type);
 		print_gfs2(" %s", ind->ii[0].dirent[d].filename);
 		if (termlines) {
 			if (edit_row[dmode] >= 0 &&
@@ -389,7 +375,7 @@ static void print_block_details(struct iinfo *ind, int level, int cur_height,
 			"display_indirect\n");
 		return;
 	}
-	tmpbuf = malloc(sbd.bsize);
+	tmpbuf = malloc(sbd.sd_bsize);
 	if (!tmpbuf) {
 		fprintf(stderr, "Out of memory in function "
 			"display_indirect\n");
@@ -398,17 +384,15 @@ static void print_block_details(struct iinfo *ind, int level, int cur_height,
 	}
 	while (thisblk) {
 		/* read in the desired block */
-		if (pread(sbd.device_fd, tmpbuf, sbd.bsize, thisblk * sbd.bsize) != sbd.bsize) {
-			fprintf(stderr, "bad read: %s from %s:%d: block %lld "
-				"(0x%llx)\n", strerror(errno), __FUNCTION__,
-				__LINE__,
-				(unsigned long long)ind->ii[pndx].block,
-				(unsigned long long)ind->ii[pndx].block);
+		if (pread(sbd.device_fd, tmpbuf, sbd.sd_bsize, thisblk * sbd.sd_bsize) != sbd.sd_bsize) {
+			fprintf(stderr, "bad read: %s from %s:%d: block %"PRIu64
+				" (0x%"PRIx64")\n", strerror(errno), __FUNCTION__,
+				__LINE__, ind->ii[pndx].block, ind->ii[pndx].block);
 			exit(-1);
 		}
 		thisblk = 0;
 		memset(more_indir, 0, sizeof(struct iinfo));
-		if (S_ISDIR(di.di_mode) && level == di.di_height) {
+		if (S_ISDIR(be32_to_cpu(di->di_mode)) && level == be16_to_cpu(di->di_height)) {
 			thisblk = do_leaf_extended(tmpbuf, more_indir);
 			display_leaf(more_indir);
 		} else {
@@ -417,7 +401,7 @@ static void print_block_details(struct iinfo *ind, int level, int cur_height,
 			for (x = 0; x < 512; x++) {
 				memcpy(&more_indir->ii[x].mp,
 				       &ind->ii[pndx].mp,
-				       sizeof(struct metapath));
+				       sizeof(struct lgfs2_metapath));
 				more_indir->ii[x].mp.mp_list[cur_height+1] = x;
 			}
 			more_ind = _do_indirect_extended(tmpbuf, more_indir,
@@ -437,29 +421,22 @@ static void print_block_details(struct iinfo *ind, int level, int cur_height,
 	free(more_indir);
 }
 
-static void gfs_jindex_print(struct gfs_jindex *ji)
-{
-        pv((unsigned long long)ji, ji_addr, "%llu", "0x%llx");
-        pv(ji, ji_nsegment, "%u", "0x%x");
-        pv(ji, ji_pad, "%u", "0x%x");
-}
-
-static int print_gfs_jindex(struct gfs2_inode *dij)
+static int print_gfs_jindex(struct lgfs2_inode *dij)
 {
 	int error, start_line;
-	struct gfs_jindex ji;
+	struct gfs_jindex *ji;
 	char jbuf[sizeof(struct gfs_jindex)];
 
 	start_line = line;
-	print_gfs2("Journal index entries found: %lld.",
-		   dij->i_di.di_size / sizeof(struct gfs_jindex));
+	print_gfs2("Journal index entries found: %"PRIu64".",
+		   dij->i_size / sizeof(struct gfs_jindex));
 	eol(0);
 	lines_per_row[dmode] = 4;
 	for (print_entry_ndx=0; ; print_entry_ndx++) {
-		error = gfs2_readi(dij, (void *)&jbuf,
+		error = lgfs2_readi(dij, (void *)&jbuf,
 				   print_entry_ndx*sizeof(struct gfs_jindex),
 				   sizeof(struct gfs_jindex));
-		gfs_jindex_in(&ji, jbuf);
+		ji = (struct gfs_jindex *)jbuf;
 		if (!error) /* end of file */
 			break;
 		if (!termlines ||
@@ -469,13 +446,13 @@ static int print_gfs_jindex(struct gfs2_inode *dij)
 			if (edit_row[dmode] == print_entry_ndx) {
 				COLORS_HIGHLIGHT;
 				strcpy(efield, "ji_addr");
-				sprintf(estring, "%llx", (unsigned long long)ji.ji_addr);
+				sprintf(estring, "%"PRIx64, be64_to_cpu(ji->ji_addr));
 			}
 			print_gfs2("Journal #%d", print_entry_ndx);
 			eol(0);
 			if (edit_row[dmode] == print_entry_ndx)
 				COLORS_NORMAL;
-			gfs_jindex_print(&ji);
+			gfs_jindex_print(ji);
 			last_entry_onscreen[dmode] = print_entry_ndx;
 		}
 	}
@@ -486,18 +463,18 @@ static int print_gfs_jindex(struct gfs2_inode *dij)
 static int print_gfs2_jindex(void)
 {
 	int d, error;
-	struct gfs2_log_header head;
-	struct gfs2_inode *ip;
+	struct lgfs2_log_header head;
+	struct lgfs2_inode *ip;
 
 	for (d = 0; d < indirect->ii[0].dirents; d++) {
 		if (strncmp(indirect->ii[0].dirent[d].filename, "journal", 7))
 			continue;
-		ip = lgfs2_inode_read(&sbd, indirect->ii[0].dirent[d].block);
-		print_gfs2("%s: 0x%-5"PRIx64" %lldMB ",
+		ip = lgfs2_inode_read(&sbd, indirect->ii[0].dirent[d].inum.in_addr);
+		print_gfs2("%s: 0x%-5"PRIx64" %"PRIu64"MB ",
 			   indirect->ii[0].dirent[d].filename,
-			   indirect->ii[0].dirent[d].block,
-			   ip->i_di.di_size / 1048576);
-		error = gfs2_find_jhead(ip, &head);
+			   indirect->ii[0].dirent[d].inum.in_addr,
+			   ip->i_size / 1048576);
+		error = lgfs2_find_jhead(ip, &head);
 		if (error) {
 			print_gfs2("corrupt.");
 		} else {
@@ -507,21 +484,21 @@ static int print_gfs2_jindex(void)
 				print_gfs2("dirty.");
 		}
 		eol(0);
-		inode_put(&ip);
+		lgfs2_inode_put(&ip);
 	}
 	return 0;
 }
 
-static int parse_rindex(struct gfs2_inode *dip, int print_rindex)
+static int parse_rindex(struct lgfs2_inode *dip, int print_rindex)
 {
 	int error, start_line;
 	struct gfs2_rindex ri;
-	char rbuf[sizeof(struct gfs2_rindex)];
 	char highlighted_addr[32];
+	struct lgfs2_rgrp_tree rg = {0};
 
 	start_line = line;
-	print_gfs2("RG index entries found: %lld.", dip->i_di.di_size /
-		   sizeof(struct gfs2_rindex));
+	print_gfs2("RG index entries found: %"PRIu64".",
+	           dip->i_size / sizeof(struct gfs2_rindex));
 	eol(0);
 	lines_per_row[dmode] = 6;
 	memset(highlighted_addr, 0, sizeof(highlighted_addr));
@@ -531,43 +508,38 @@ static int parse_rindex(struct gfs2_inode *dip, int print_rindex)
 
 		roff = print_entry_ndx * sizeof(struct gfs2_rindex);
 
-		error = gfs2_readi(dip, (void *)&rbuf, roff,
-				   sizeof(struct gfs2_rindex));
+		error = lgfs2_readi(dip, &ri, roff, sizeof(struct gfs2_rindex));
 		if (!error) /* end of file */
 			break;
-		gfs2_rindex_in(&ri, rbuf);
+		lgfs2_rindex_in(&rg, &ri);
 		if (!termlines ||
 			(print_entry_ndx >= start_row[dmode] &&
 			 ((print_entry_ndx - start_row[dmode])+1) * lines_per_row[dmode] <=
 			 termlines - start_line - 2)) {
 			if (edit_row[dmode] == print_entry_ndx) {
 				COLORS_HIGHLIGHT;
-				sprintf(highlighted_addr, "%llx", (unsigned long long)ri.ri_addr);
+				sprintf(highlighted_addr, "%"PRIx64, rg.rt_addr);
 			}
 			print_gfs2("RG #%d", print_entry_ndx);
 			if (!print_rindex)
-				print_gfs2(" located at: %llu (0x%llx)",
-					   ri.ri_addr, ri.ri_addr);
+				print_gfs2(" located at: %"PRIu64" (0x%"PRIx64")",
+					   rg.rt_addr, rg.rt_addr);
 			eol(0);
 			if (edit_row[dmode] == print_entry_ndx)
 				COLORS_NORMAL;
-			if(print_rindex)
-				gfs2_rindex_print(&ri);
+			if (print_rindex)
+				rindex_print(&ri);
 			else {
-				char buf[sizeof(struct gfs2_rgrp)] = {0};
+				struct gfs2_rgrp r = {0};
 				ssize_t ret;
 
-				ret = pread(sbd.device_fd, buf, sizeof(buf), ri.ri_addr * sbd.bsize);
-				if (ret != sizeof(buf)) {
+				ret = pread(sbd.device_fd, &r, sizeof(r), rg.rt_addr * sbd.sd_bsize);
+				if (ret != sizeof(r)) {
 					perror("Failed to read resource group");
 				} else if (sbd.gfs1) {
-					struct gfs_rgrp rg1;
-					gfs_rgrp_in(&rg1, buf);
-					gfs_rgrp_print(&rg1);
+					gfs_rgrp_print(&r);
 				} else {
-					struct gfs2_rgrp rg;
-					gfs2_rgrp_in(&rg, buf);
-					gfs2_rgrp_print(&rg);
+					rgrp_print(&r);
 				}
 			}
 			last_entry_onscreen[dmode] = print_entry_ndx;
@@ -578,12 +550,13 @@ static int parse_rindex(struct gfs2_inode *dip, int print_rindex)
 	return error;
 }
 
-static int print_inum(struct gfs2_inode *dii)
+static int print_inum(struct lgfs2_inode *dii)
 {
-	uint64_t inum, inodenum;
+	uint64_t inodenum;
+	__be64 inum;
 	int rc;
 	
-	rc = gfs2_readi(dii, (void *)&inum, 0, sizeof(inum));
+	rc = lgfs2_readi(dii, (void *)&inum, 0, sizeof(inum));
 	if (!rc) {
 		print_gfs2("The inum file is empty.");
 		eol(0);
@@ -600,73 +573,71 @@ static int print_inum(struct gfs2_inode *dii)
 	return 0;
 }
 
-static int print_statfs(struct gfs2_inode *dis)
+static int print_statfs(struct lgfs2_inode *dis)
 {
-	struct gfs2_statfs_change sfb, sfc;
+	struct gfs2_statfs_change sc;
 	int rc;
-	
-	rc = gfs2_readi(dis, (void *)&sfb, 0, sizeof(sfb));
+
+	rc = lgfs2_readi(dis, (void *)&sc, 0, sizeof(sc));
 	if (!rc) {
 		print_gfs2("The statfs file is empty.");
 		eol(0);
 		return 0;
 	}
-	if (rc != sizeof(sfb)) {
+	if (rc != sizeof(sc)) {
 		print_gfs2("Error reading statfs file.");
 		eol(0);
 		return -1;
 	}
-	gfs2_statfs_change_in(&sfc, (char *)&sfb);
 	print_gfs2("statfs file contents:");
 	eol(0);
-	gfs2_statfs_change_print(&sfc);
+	statfs_change_print(&sc);
 	return 0;
 }
 
-static int print_quota(struct gfs2_inode *diq)
+static int print_quota(struct lgfs2_inode *diq)
 {
-	struct gfs2_quota qbuf, q;
+	struct gfs2_quota q;
 	int i, error;
-	
+
 	print_gfs2("quota file contents:");
 	eol(0);
-	print_gfs2("quota entries found: %lld.", diq->i_di.di_size / sizeof(q));
+	print_gfs2("quota entries found: %"PRIu64".", diq->i_size / sizeof(q));
 	eol(0);
 	for (i=0; ; i++) {
-		error = gfs2_readi(diq, (void *)&qbuf, i * sizeof(q), sizeof(qbuf));
+		error = lgfs2_readi(diq, &q, i * sizeof(q), sizeof(q));
 		if (!error)
 			break;
-		if (error != sizeof(qbuf)) {
+		if (error != sizeof(q)) {
 			print_gfs2("Error reading quota file.");
 			eol(0);
 			return -1;
 		}
-		gfs2_quota_in(&q, (char *)&qbuf);
 		print_gfs2("Entry #%d", i + 1);
 		eol(0);
-		gfs2_quota_print(&q);
+		quota_print(&q);
 	}
 	return 0;
 }
 
 int display_extended(void)
 {
-	struct gfs2_inode *tmp_inode;
-	struct gfs2_buffer_head *tmp_bh;
+	struct lgfs2_inode *tmp_inode;
+	struct lgfs2_buffer_head *tmp_bh;
 
 	dsplines = termlines - line - 1;
 	/* Display any indirect pointers that we have. */
 	if (block_is_rindex(block)) {
-		tmp_bh = bread(&sbd, block);
+		tmp_bh = lgfs2_bread(&sbd, block);
 		tmp_inode = lgfs2_inode_get(&sbd, tmp_bh);
 		if (tmp_inode == NULL)
 			return -1;
 		parse_rindex(tmp_inode, TRUE);
-		inode_put(&tmp_inode);
-		brelse(tmp_bh);
+		lgfs2_inode_put(&tmp_inode);
+		lgfs2_brelse(tmp_bh);
 	} else if (block_is_journals(block)) {
 		if (sbd.gfs1)
-			block = sbd1->sb_jindex_di.no_addr;
+			block = sbd.sd_jindex_di.in_addr;
 		else
 			block = masterblock("jindex");
 		print_gfs2_jindex();
@@ -677,50 +648,50 @@ int display_extended(void)
 		return -1;
 	else if (block_is_rgtree(block)) {
 		if (sbd.gfs1)
-			tmp_bh = bread(&sbd, sbd1->sb_rindex_di.no_addr);
+			tmp_bh = lgfs2_bread(&sbd, sbd.sd_rindex_di.in_addr);
 		else
-			tmp_bh = bread(&sbd, masterblock("rindex"));
+			tmp_bh = lgfs2_bread(&sbd, masterblock("rindex"));
 		tmp_inode = lgfs2_inode_get(&sbd, tmp_bh);
 		if (tmp_inode == NULL)
 			return -1;
 		parse_rindex(tmp_inode, FALSE);
-		inode_put(&tmp_inode);
-		brelse(tmp_bh);
+		lgfs2_inode_put(&tmp_inode);
+		lgfs2_brelse(tmp_bh);
 	} else if (block_is_jindex(block)) {
-		tmp_bh = bread(&sbd, block);
+		tmp_bh = lgfs2_bread(&sbd, block);
 		tmp_inode = lgfs2_inode_get(&sbd, tmp_bh);
 		if (tmp_inode == NULL)
 			return -1;
 		print_gfs_jindex(tmp_inode);
-		inode_put(&tmp_inode);
-		brelse(tmp_bh);
+		lgfs2_inode_put(&tmp_inode);
+		lgfs2_brelse(tmp_bh);
 	}
 	else if (block_is_inum_file(block)) {
-		tmp_bh = bread(&sbd, block);
+		tmp_bh = lgfs2_bread(&sbd, block);
 		tmp_inode = lgfs2_inode_get(&sbd, tmp_bh);
 		if (tmp_inode == NULL)
 			return -1;
 		print_inum(tmp_inode);
-		inode_put(&tmp_inode);
-		brelse(tmp_bh);
+		lgfs2_inode_put(&tmp_inode);
+		lgfs2_brelse(tmp_bh);
 	}
 	else if (block_is_statfs_file(block)) {
-		tmp_bh = bread(&sbd, block);
+		tmp_bh = lgfs2_bread(&sbd, block);
 		tmp_inode = lgfs2_inode_get(&sbd, tmp_bh);
 		if (tmp_inode == NULL)
 			return -1;
 		print_statfs(tmp_inode);
-		inode_put(&tmp_inode);
-		brelse(tmp_bh);
+		lgfs2_inode_put(&tmp_inode);
+		lgfs2_brelse(tmp_bh);
 	}
 	else if (block_is_quota_file(block)) {
-		tmp_bh = bread(&sbd, block);
+		tmp_bh = lgfs2_bread(&sbd, block);
 		tmp_inode = lgfs2_inode_get(&sbd, tmp_bh);
 		if (tmp_inode == NULL)
 			return -1;
 		print_quota(tmp_inode);
-		inode_put(&tmp_inode);
-		brelse(tmp_bh);
+		lgfs2_inode_put(&tmp_inode);
+		lgfs2_brelse(tmp_bh);
 	}
 	return 0;
 }
