@@ -7,7 +7,6 @@
 #include <stdint.h>
 #include <inttypes.h>
 #include <sys/types.h>
-#include <linux/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -17,7 +16,6 @@
 #include <sys/ioctl.h>
 #include <limits.h>
 #include <sys/time.h>
-#include <linux/gfs2_ondisk.h>
 #include <zlib.h>
 #include <bzlib.h>
 #include <time.h>
@@ -34,17 +32,23 @@
 /* Header for the savemeta output file */
 struct savemeta_header {
 #define SAVEMETA_MAGIC (0x01171970)
-	uint32_t sh_magic;
+	__be32 sh_magic;
 #define SAVEMETA_FORMAT (1)
-	uint32_t sh_format; /* In case we want to change the layout */
-	uint64_t sh_time; /* When savemeta was run */
-	uint64_t sh_fs_bytes; /* Size of the fs */
+	__be32 sh_format; /* In case we want to change the layout */
+	__be64 sh_time; /* When savemeta was run */
+	__be64 sh_fs_bytes; /* Size of the fs */
 	uint8_t __reserved[104];
 };
 
+struct savemeta {
+	time_t sm_time;
+	unsigned sm_format;
+	size_t sm_fs_bytes;
+};
+
 struct saved_metablock {
-	uint64_t blk;
-	uint16_t siglen; /* significant data length */
+	__be64 blk;
+	__be16 siglen; /* significant data length */
 /* This needs to be packed because old versions of gfs2_edit read and write the
    individual fields separately, so the hole after siglen must be eradicated
    before the struct reflects what's on disk. */
@@ -62,9 +66,9 @@ struct metafd {
 	const char* (*strerr)(struct metafd *mfd);
 };
 
-char *restore_buf;
-ssize_t restore_left;
-off_t restore_off;
+static char *restore_buf;
+static ssize_t restore_left;
+static off_t restore_off;
 #define RESTORE_BUF_SIZE (2 * 1024 * 1024)
 
 static char *restore_buf_next(struct metafd *mfd, size_t required_len)
@@ -186,7 +190,6 @@ static uint64_t journal_blocks[MAX_JOURNALS_SAVED];
 static uint64_t gfs1_journal_size = 0; /* in blocks */
 static int journals_found = 0;
 int print_level = MSG_NOTICE;
-extern char *device;
 
 static int block_is_a_journal(uint64_t blk)
 {
@@ -198,7 +201,7 @@ static int block_is_a_journal(uint64_t blk)
 	return FALSE;
 }
 
-struct osi_root per_node_tree;
+static struct osi_root per_node_tree;
 struct per_node_node {
 	struct osi_node node;
 	uint64_t block;
@@ -263,7 +266,7 @@ static int insert_per_node_lookup(uint64_t blk)
 static int init_per_node_lookup(void)
 {
 	int i;
-	struct gfs2_inode *per_node_di;
+	struct lgfs2_inode *per_node_di;
 
 	if (sbd.gfs1)
 		return FALSE;
@@ -274,13 +277,13 @@ static int init_per_node_lookup(void)
 		return 1;
 	}
 
-	do_dinode_extended(&per_node_di->i_di, per_node_di->i_bh->b_data);
-	inode_put(&per_node_di);
+	do_dinode_extended(per_node_di->i_bh->b_data);
+	lgfs2_inode_put(&per_node_di);
 
 	for (i = 0; i < indirect_blocks; i++) {
 		int d;
 		for (d = 0; d < indirect->ii[i].dirents; d++) {
-			int ret = insert_per_node_lookup(indirect->ii[i].dirent[d].block);
+			int ret = insert_per_node_lookup(indirect->ii[i].dirent[d].inum.in_addr);
 			if (ret != 0)
 				return ret;
 		}
@@ -299,21 +302,23 @@ static int block_is_systemfile(uint64_t blk)
 static size_t di_save_len(const char *buf, uint64_t owner)
 {
 	const struct gfs2_dinode *dn;
+	const struct gfs_dinode *d1;
 	uint16_t di_height;
 	uint32_t di_mode;
 	int gfs1dir;
 
 	dn = (void *)buf;
+	d1 = (void *)buf;
 	di_mode = be32_to_cpu(dn->di_mode);
 	di_height = be16_to_cpu(dn->di_height);
 	/* __pad1 is di_type in gfs1 */
-	gfs1dir = sbd.gfs1 && (be16_to_cpu(dn->__pad1) == GFS_FILE_DIR);
+	gfs1dir = sbd.gfs1 && (be16_to_cpu(d1->di_type) == GFS_FILE_DIR);
 
 	/* Do not save (user) data from the inode block unless they are
 	   indirect pointers, dirents, symlinks or fs internal data */
 	if (di_height > 0 || S_ISDIR(di_mode) || S_ISLNK(di_mode) || gfs1dir
 	    || block_is_systemfile(owner))
-		return sbd.bsize;
+		return sbd.sd_bsize;
 	return sizeof(struct gfs2_dinode);
 }
 
@@ -331,25 +336,24 @@ static size_t di_save_len(const char *buf, uint64_t owner)
 static int get_gfs_struct_info(const char *buf, uint64_t owner, unsigned *block_type,
                                unsigned *gstruct_len)
 {
-	struct gfs2_meta_header mh;
+	struct gfs2_meta_header *mh = (struct gfs2_meta_header *)buf;
 
 	if (block_type != NULL)
 		*block_type = 0;
 
 	if (gstruct_len != NULL)
-		*gstruct_len = sbd.bsize;
+		*gstruct_len = sbd.sd_bsize;
 
-	gfs2_meta_header_in(&mh, buf);
-	if (mh.mh_magic != GFS2_MAGIC)
+	if (be32_to_cpu(mh->mh_magic) != GFS2_MAGIC)
 		return -1;
 
 	if (block_type != NULL)
-		*block_type = mh.mh_type;
+		*block_type = be32_to_cpu(mh->mh_type);
 
 	if (gstruct_len == NULL)
 		return 0;
 
-	switch (mh.mh_type) {
+	switch (be32_to_cpu(mh->mh_type)) {
 	case GFS2_METATYPE_SB:   /* 1 (superblock) */
 		if (sbd.gfs1)
 			*gstruct_len = sizeof(struct gfs_sb);
@@ -357,22 +361,22 @@ static int get_gfs_struct_info(const char *buf, uint64_t owner, unsigned *block_
 			*gstruct_len = sizeof(struct gfs2_sb);
 		break;
 	case GFS2_METATYPE_RG:   /* 2 (rsrc grp hdr) */
-		*gstruct_len = sbd.bsize; /*sizeof(struct gfs_rgrp);*/
+		*gstruct_len = sbd.sd_bsize; /*sizeof(struct gfs_rgrp);*/
 		break;
 	case GFS2_METATYPE_RB:   /* 3 (rsrc grp bitblk) */
-		*gstruct_len = sbd.bsize;
+		*gstruct_len = sbd.sd_bsize;
 		break;
 	case GFS2_METATYPE_DI:   /* 4 (disk inode) */
 		*gstruct_len = di_save_len(buf, owner);
 		break;
 	case GFS2_METATYPE_IN:   /* 5 (indir inode blklst) */
-		*gstruct_len = sbd.bsize; /*sizeof(struct gfs_indirect);*/
+		*gstruct_len = sbd.sd_bsize; /*sizeof(struct gfs_indirect);*/
 		break;
 	case GFS2_METATYPE_LF:   /* 6 (leaf dinode blklst) */
-		*gstruct_len = sbd.bsize; /*sizeof(struct gfs_leaf);*/
+		*gstruct_len = sbd.sd_bsize; /*sizeof(struct gfs_leaf);*/
 		break;
 	case GFS2_METATYPE_JD:   /* 7 (journal data) */
-		*gstruct_len = sbd.bsize;
+		*gstruct_len = sbd.sd_bsize;
 		break;
 	case GFS2_METATYPE_LH:   /* 8 (log header) */
 		if (sbd.gfs1)
@@ -384,16 +388,16 @@ static int get_gfs_struct_info(const char *buf, uint64_t owner, unsigned *block_
 			*gstruct_len = sizeof(struct gfs2_log_header);
 		break;
 	case GFS2_METATYPE_LD:   /* 9 (log descriptor) */
-		*gstruct_len = sbd.bsize;
+		*gstruct_len = sbd.sd_bsize;
 		break;
 	case GFS2_METATYPE_EA:   /* 10 (extended attr hdr) */
-		*gstruct_len = sbd.bsize;
+		*gstruct_len = sbd.sd_bsize;
 		break;
 	case GFS2_METATYPE_ED:   /* 11 (extended attr data) */
-		*gstruct_len = sbd.bsize;
+		*gstruct_len = sbd.sd_bsize;
 		break;
 	default:
-		*gstruct_len = sbd.bsize;
+		*gstruct_len = sbd.sd_bsize;
 		break;
 	}
 	return 0;
@@ -573,12 +577,12 @@ struct block_range {
 
 static int block_range_prepare(struct block_range *br)
 {
-	br->buf = calloc(br->len, sbd.bsize + sizeof(*br->blktype) + sizeof(*br->blklen));
+	br->buf = calloc(br->len, sbd.sd_bsize + sizeof(*br->blktype) + sizeof(*br->blklen));
 	if (br->buf == NULL) {
 		perror("Failed to allocate block range buffer");
 		return 1;
 	}
-	br->blktype = (unsigned *)(br->buf + (br->len * sbd.bsize));
+	br->blktype = (unsigned *)(br->buf + (br->len * sbd.sd_bsize));
 	br->blklen = br->blktype + br->len;
 	return 0;
 }
@@ -598,7 +602,7 @@ static int block_range_check(struct block_range *br)
 static void block_range_setinfo(struct block_range *br, uint64_t owner)
 {
 	for (unsigned i = 0; i < br->len; i++) {
-		char *buf = br->buf + (i * sbd.bsize);
+		char *buf = br->buf + (i * sbd.sd_bsize);
 		uint64_t addr = br->start + i;
 		uint64_t _owner = (owner == 0) ? addr : owner;
 
@@ -646,7 +650,7 @@ static int save_range(struct metafd *mfd, struct block_range *br)
 	for (unsigned i = 0; i < br->len; i++) {
 		int err;
 
-		err = save_buf(mfd, br->buf + (i * sbd.bsize), br->start + i, br->blklen[i]);
+		err = save_buf(mfd, br->buf + (i * sbd.sd_bsize), br->start + i, br->blklen[i]);
 		if (err != 0)
 			return err;
 	}
@@ -663,8 +667,8 @@ static int check_read_range(int fd, struct block_range *br, uint64_t owner)
 	if (block_range_check(br) != 0)
 		return 1;
 
-	size = br->len * sbd.bsize;
-	if (pread(sbd.device_fd, br->buf, size, sbd.bsize * br->start) != size) {
+	size = br->len * sbd.sd_bsize;
+	if (pread(sbd.device_fd, br->buf, size, sbd.sd_bsize * br->start) != size) {
 		fprintf(stderr, "Failed to read block range 0x%"PRIx64" (%u blocks): %s\n",
 		        br->start, br->len, strerror(errno));
 		free(br->buf);
@@ -696,31 +700,35 @@ static char *check_read_block(int fd, uint64_t blk, uint64_t owner, int *blktype
  */
 static void save_ea_block(struct metafd *mfd, char *buf, uint64_t owner)
 {
+	struct gfs2_ea_header *ea;
+	uint32_t rec_len = 0;
 	int e;
-	struct gfs2_ea_header ea;
 
-	for (e = sizeof(struct gfs2_meta_header); e < sbd.bsize; e += ea.ea_rec_len) {
-		uint64_t blk, *b;
+	for (e = sizeof(struct gfs2_meta_header); e < sbd.sd_bsize; e += rec_len) {
+		uint64_t blk;
 		int charoff, i;
+		__be64 *b;
 
-		gfs2_ea_header_in(&ea, buf + e);
-		for (i = 0; i < ea.ea_num_ptrs; i++) {
+		ea = (void *)(buf + e);
+		/* ea_num_ptrs and ea_name_len are u8 so no endianness worries */
+		for (i = 0; i < ea->ea_num_ptrs; i++) {
 			char *_buf;
 
-			charoff = e + ea.ea_name_len +
+			charoff = e + ea->ea_name_len +
 				sizeof(struct gfs2_ea_header) +
 				sizeof(uint64_t) - 1;
 			charoff /= sizeof(uint64_t);
-			b = (uint64_t *)buf;
+			b = (__be64 *)buf;
 			b += charoff + i;
 			blk = be64_to_cpu(*b);
 			_buf = check_read_block(sbd.device_fd, blk, owner, NULL, NULL);
 			if (_buf != NULL) {
-				save_buf(mfd, _buf, blk, sbd.bsize);
+				save_buf(mfd, _buf, blk, sbd.sd_bsize);
 				free(_buf);
 			}
 		}
-		if (!ea.ea_rec_len)
+		rec_len = be32_to_cpu(ea->ea_rec_len);
+		if (rec_len == 0)
 			break;
 	}
 }
@@ -736,7 +744,7 @@ static void save_indirect_range(struct metafd *mfd, struct block_range **brp, ui
 	save_range(mfd, br);
 	for (unsigned i = 0; i < br->len; i++) {
 		if (br->blktype[i] == GFS2_METATYPE_EA)
-			save_ea_block(mfd, br->buf + (i * sbd.bsize), owner);
+			save_ea_block(mfd, br->buf + (i * sbd.sd_bsize), owner);
 	}
 	if (q) {
 		block_range_queue_insert(q, br);
@@ -752,10 +760,10 @@ static void save_indirect_blocks(struct metafd *mfd, char *buf, uint64_t owner,
 {
 	uint64_t old_block = 0, indir_block;
 	struct block_range *br = NULL;
-	uint64_t *ptr;
+	__be64 *ptr;
 
-	for (ptr = (uint64_t *)(buf + headsize);
-	     (char *)ptr < (buf + sbd.bsize); ptr++) {
+	for (ptr = (__be64 *)(buf + headsize);
+	     (char *)ptr < (buf + sbd.sd_bsize); ptr++) {
 		if (!*ptr)
 			continue;
 
@@ -788,35 +796,30 @@ new_range:
 	free(br);
 }
 
-static int save_leaf_chain(struct metafd *mfd, struct gfs2_sbd *sdp, char *buf)
+static int save_leaf_chain(struct metafd *mfd, struct lgfs2_sbd *sdp, char *buf)
 {
-	struct gfs2_leaf leaf;
+	struct gfs2_leaf *leaf = (struct gfs2_leaf *)buf;
 
-	gfs2_leaf_in(&leaf, buf);
-
-	while (leaf.lf_next != 0) {
-		uint64_t blk = leaf.lf_next;
+	while (leaf->lf_next != 0) {
+		uint64_t blk = be64_to_cpu(leaf->lf_next);
 		ssize_t r;
 
-		if (gfs2_check_range(sdp, blk) != 0)
+		if (lgfs2_check_range(sdp, blk) != 0)
 			return 0;
 
-		r = pread(sdp->device_fd, buf, sdp->bsize, sdp->bsize * blk);
-		if (r != sdp->bsize) {
+		r = pread(sdp->device_fd, buf, sdp->sd_bsize, sdp->sd_bsize * blk);
+		if (r != sdp->sd_bsize) {
 			fprintf(stderr, "Failed to read leaf block %"PRIx64": %s\n",
 			        blk, strerror(errno));
-			free(buf);
 			return 1;
 		}
 		report_progress(blk, 0);
-		if (gfs2_check_meta(buf, GFS2_METATYPE_LF) == 0) {
-			int ret = save_buf(mfd, buf, blk, sdp->bsize);
-			if (ret != 0) {
-				free(buf);
+		if (lgfs2_check_meta(buf, GFS2_METATYPE_LF) == 0) {
+			int ret = save_buf(mfd, buf, blk, sdp->sd_bsize);
+			if (ret != 0)
 				return ret;
-			}
 		}
-		gfs2_leaf_in(&leaf, buf);
+		leaf = (struct gfs2_leaf *)buf;
 	}
 	return 0;
 }
@@ -827,13 +830,12 @@ static void save_leaf_blocks(struct metafd *mfd, struct block_range_queue *q)
 		struct block_range *br = q->tail;
 
 		for (unsigned i = 0; i < br->len; i++) {
-			char *buf = br->buf + (i * sbd.bsize);
+			char *buf = br->buf + (i * sbd.sd_bsize);
 
 			save_leaf_chain(mfd, &sbd, buf);
 		}
 		q->tail = br->next;
-		free(br->buf);
-		free(br);
+		block_range_free(&br);
 	}
 }
 
@@ -856,15 +858,15 @@ static void save_leaf_blocks(struct metafd *mfd, struct block_range_queue *q)
 static void save_inode_data(struct metafd *mfd, char *ibuf, uint64_t iblk)
 {
 	struct block_range_queue indq[GFS2_MAX_META_HEIGHT] = {{NULL}};
-	uint32_t height;
-	struct gfs2_dinode _di = {0};
+	struct gfs2_dinode *dip = (struct gfs2_dinode *)ibuf;
+	struct gfs_dinode *dip1 = (struct gfs_dinode *)ibuf;
+	uint16_t height;
 	int is_exhash;
 
 	for (unsigned i = 0; i < GFS2_MAX_META_HEIGHT; i++)
 		block_range_queue_init(&indq[i]);
 
-	gfs2_dinode_in(&_di, ibuf);
-	height = _di.di_height;
+	height = be16_to_cpu(dip->di_height);
 
 	/* If this is a user inode, we don't follow to the file height.
 	   We stop one level less.  That way we save off the indirect
@@ -872,29 +874,35 @@ static void save_inode_data(struct metafd *mfd, char *ibuf, uint64_t iblk)
 	   is directories, where the height represents the level at which
 	   the hash table exists, and we have to save the directory data. */
 
-	is_exhash = (S_ISDIR(_di.di_mode) || (sbd.gfs1 && _di.__pad1 == GFS_FILE_DIR)) &&
-	             _di.di_flags & GFS2_DIF_EXHASH;
+	is_exhash = (S_ISDIR(be32_to_cpu(dip->di_mode)) ||
+	             (sbd.gfs1 && be16_to_cpu(dip1->di_type) == GFS_FILE_DIR)) &&
+	             be32_to_cpu(dip->di_flags) & GFS2_DIF_EXHASH;
 	if (is_exhash)
 		height++;
-	else if (height > 0 && !(_di.di_flags & GFS2_DIF_SYSTEM) &&
-		 !block_is_systemfile(iblk) && !S_ISDIR(_di.di_mode))
+	else if (height > 0 && !(be32_to_cpu(dip->di_flags) & GFS2_DIF_SYSTEM) &&
+		 !block_is_systemfile(iblk) && !S_ISDIR(be32_to_cpu(dip->di_mode)))
 		height--;
 
-	if (height > 0)
-		save_indirect_blocks(mfd, ibuf, iblk, height == 1 ? NULL : &indq[0], sizeof(_di));
+	if (height == 1)
+		save_indirect_blocks(mfd, ibuf, iblk, NULL, sizeof(*dip));
+	else if (height > 1)
+		save_indirect_blocks(mfd, ibuf, iblk, &indq[0], sizeof(*dip));
+
 	for (unsigned i = 1; i < height; i++) {
 		struct block_range_queue *nextq = &indq[i];
 
 		if (!is_exhash && i == height - 1)
 			nextq = NULL;
 
+		/* Coverity can't figure out that the tail becomes NULL eventually. */
+		/* coverity[loop_top:SUPPRESS] */
 		while (indq[i - 1].tail != NULL) {
 			struct block_range *q = block_range_queue_pop(&indq[i - 1]);
 
 			for (unsigned j = 0; j < q->len; j++) {
-				char *_buf = q->buf + (j * sbd.bsize);
+				char *_buf = q->buf + (j * sbd.sd_bsize);
 
-				save_indirect_blocks(mfd, _buf, iblk, nextq, sizeof(_di.di_header));
+				save_indirect_blocks(mfd, _buf, iblk, nextq, sizeof(dip->di_header));
 			}
 			report_progress(q->start + q->len, 0);
 			block_range_free(&q);
@@ -902,20 +910,20 @@ static void save_inode_data(struct metafd *mfd, char *ibuf, uint64_t iblk)
 	}
 	if (is_exhash)
 		save_leaf_blocks(mfd, &indq[height - 1]);
-	if (_di.di_eattr) { /* if this inode has extended attributes */
+	if (dip->di_eattr) { /* if this inode has extended attributes */
 		size_t blklen;
 		uint64_t blk;
 		int mhtype;
 		char *buf;
 
-		blk = _di.di_eattr;
+		blk = be64_to_cpu(dip->di_eattr);
 		buf = check_read_block(sbd.device_fd, blk, iblk, &mhtype, &blklen);
 		if (buf != NULL) {
 			save_buf(mfd, buf, blk, blklen);
 			if (mhtype == GFS2_METATYPE_EA)
 				save_ea_block(mfd, buf, iblk);
 			else if (mhtype == GFS2_METATYPE_IN)
-				save_indirect_blocks(mfd, buf, iblk, NULL, sizeof(_di.di_header));
+				save_indirect_blocks(mfd, buf, iblk, NULL, sizeof(dip->di_header));
 			free(buf);
 		}
 	}
@@ -939,31 +947,30 @@ static void get_journal_inode_blocks(void)
 	for (journal = 0; ; journal++) { /* while journals exist */
 		uint64_t jblock;
 		int amt;
-		struct gfs2_inode *j_inode = NULL;
+		struct lgfs2_inode *j_inode = NULL;
 
 		if (sbd.gfs1) {
-			struct gfs_jindex ji;
+			struct gfs_jindex *ji;
 			char jbuf[sizeof(struct gfs_jindex)];
 
-			j_inode = lgfs2_gfs_inode_read(&sbd,
-						 sbd1->sb_jindex_di.no_addr);
+			j_inode = lgfs2_gfs_inode_read(&sbd, sbd.sd_jindex_di.in_addr);
 			if (j_inode == NULL) {
 				fprintf(stderr, "Error reading journal inode: %s\n", strerror(errno));
 				return;
 			}
-			amt = gfs2_readi(j_inode, (void *)&jbuf,
+			amt = lgfs2_readi(j_inode, (void *)&jbuf,
 					 journal * sizeof(struct gfs_jindex),
 					 sizeof(struct gfs_jindex));
-			inode_put(&j_inode);
+			lgfs2_inode_put(&j_inode);
 			if (!amt)
 				break;
-			gfs_jindex_in(&ji, jbuf);
-			jblock = ji.ji_addr;
-			gfs1_journal_size = (uint64_t)ji.ji_nsegment * 16;
+			ji = (struct gfs_jindex *)jbuf;
+			jblock = be64_to_cpu(ji->ji_addr);
+			gfs1_journal_size = (uint64_t)be32_to_cpu(ji->ji_nsegment) * 16;
 		} else {
 			if (journal + 3 > indirect->ii[0].dirents)
 				break;
-			jblock = indirect->ii[0].dirent[journal + 2].block;
+			jblock = indirect->ii[0].dirent[journal + 2].inum.in_addr;
 		}
 		journal_blocks[journals_found++] = jblock;
 	}
@@ -976,7 +983,7 @@ static void save_allocated_range(struct metafd *mfd, struct block_range *br)
 
 	save_range(mfd, br);
 	for (unsigned i = 0; i < br->len; i++) {
-		char *buf = br->buf + (i * sbd.bsize);
+		char *buf = br->buf + (i * sbd.sd_bsize);
 
 		if (br->blktype[i] == GFS2_METATYPE_DI)
 			save_inode_data(mfd, buf, br->start + i);
@@ -984,13 +991,13 @@ static void save_allocated_range(struct metafd *mfd, struct block_range *br)
 	free(br->buf);
 }
 
-static void save_allocated(struct rgrp_tree *rgd, struct metafd *mfd)
+static void save_allocated(struct lgfs2_rgrp_tree *rgd, struct metafd *mfd)
 {
 	uint64_t blk = 0;
 	unsigned i, j, m;
-	uint64_t *ibuf = malloc(sbd.bsize * GFS2_NBBY * sizeof(uint64_t));
+	uint64_t *ibuf = malloc(sbd.sd_bsize * GFS2_NBBY * sizeof(uint64_t));
 
-	for (i = 0; i < rgd->ri.ri_length; i++) {
+	for (i = 0; i < rgd->rt_length; i++) {
 		struct block_range br = {0};
 
 		m = lgfs2_bm_scan(rgd, i, ibuf, GFS2_BLKST_DINODE);
@@ -1030,13 +1037,13 @@ static void save_allocated(struct rgrp_tree *rgd, struct metafd *mfd)
 	free(ibuf);
 }
 
-static char *rgrp_read(struct gfs2_sbd *sdp, uint64_t addr, unsigned blocks)
+static char *rgrp_read(struct lgfs2_sbd *sdp, uint64_t addr, unsigned blocks)
 {
-	size_t len = blocks * sdp->bsize;
-	off_t off = addr * sdp->bsize;
+	size_t len = blocks * sdp->sd_bsize;
+	off_t off = addr * sdp->sd_bsize;
 	char *buf;
 
-	if (blocks == 0 || gfs2_check_range(sdp, addr))
+	if (blocks == 0 || lgfs2_check_range(sdp, addr))
 		return NULL;
 
 	buf = calloc(1, len);
@@ -1050,36 +1057,31 @@ static char *rgrp_read(struct gfs2_sbd *sdp, uint64_t addr, unsigned blocks)
 	return buf;
 }
 
-static void save_rgrp(struct gfs2_sbd *sdp, struct metafd *mfd, struct rgrp_tree *rgd, int withcontents)
+static void save_rgrp(struct lgfs2_sbd *sdp, struct metafd *mfd, struct lgfs2_rgrp_tree *rgd, int withcontents)
 {
-	uint64_t addr = rgd->ri.ri_addr;
+	uint64_t addr = rgd->rt_addr;
 	char *buf;
 
-	buf = rgrp_read(sdp, rgd->ri.ri_addr, rgd->ri.ri_length);
+	buf = rgrp_read(sdp, rgd->rt_addr, rgd->rt_length);
 	if (buf == NULL)
 		return;
 
-	if (sdp->gfs1)
-		gfs_rgrp_in((struct gfs_rgrp *)&rgd->rg, buf);
-	else
-		gfs2_rgrp_in(&rgd->rg, buf);
+	for (unsigned i = 0; i < rgd->rt_length; i++)
+		rgd->rt_bits[i].bi_data = buf + (i * sdp->sd_bsize);
 
-	for (unsigned i = 0; i < rgd->ri.ri_length; i++)
-		rgd->bits[i].bi_data = buf + (i * sdp->bsize);
-
-	log_debug("RG at %"PRIu64" is %"PRIu32" long\n", addr, (uint32_t)rgd->ri.ri_length);
+	log_debug("RG at %"PRIu64" is %"PRIu32" long\n", addr, rgd->rt_length);
 	/* Save the rg and bitmaps */
-	for (unsigned i = 0; i < rgd->ri.ri_length; i++) {
-		report_progress(rgd->ri.ri_addr + i, 0);
-		save_buf(mfd, buf + (i * sdp->bsize), rgd->ri.ri_addr + i, sdp->bsize);
+	for (unsigned i = 0; i < rgd->rt_length; i++) {
+		report_progress(rgd->rt_addr + i, 0);
+		save_buf(mfd, buf + (i * sdp->sd_bsize), rgd->rt_addr + i, sdp->sd_bsize);
 	}
 	/* Save the other metadata: inodes, etc. if mode is not 'savergs' */
 	if (withcontents)
 		save_allocated(rgd, mfd);
 
 	free(buf);
-	for (unsigned i = 0; i < rgd->ri.ri_length; i++)
-		rgd->bits[i].bi_data = NULL;
+	for (unsigned i = 0; i < rgd->rt_length; i++)
+		rgd->rt_bits[i].bi_data = NULL;
 }
 
 static int save_header(struct metafd *mfd, uint64_t fsbytes)
@@ -1096,24 +1098,23 @@ static int save_header(struct metafd *mfd, uint64_t fsbytes)
 	return 0;
 }
 
-static int parse_header(char *buf, struct savemeta_header *smh)
+static int parse_header(char *buf, struct savemeta *sm)
 {
-	struct savemeta_header *smh_be = (void *)buf;
+	struct savemeta_header *smh = (void *)buf;
 
-	if (be32_to_cpu(smh_be->sh_magic) != SAVEMETA_MAGIC) {
+	if (be32_to_cpu(smh->sh_magic) != SAVEMETA_MAGIC) {
 		printf("No valid file header found. Falling back to old format...\n");
 		return 1;
 	}
-	if (be32_to_cpu(smh_be->sh_format) > SAVEMETA_FORMAT) {
+	if (be32_to_cpu(smh->sh_format) > SAVEMETA_FORMAT) {
 		printf("This version of gfs2_edit is too old to restore this metadata format.\n");
 		return -1;
 	}
-	smh->sh_magic = be32_to_cpu(smh_be->sh_magic);
-	smh->sh_format = be32_to_cpu(smh_be->sh_format);
-	smh->sh_time = be64_to_cpu(smh_be->sh_time);
-	smh->sh_fs_bytes = be64_to_cpu(smh_be->sh_fs_bytes);
-	printf("Metadata saved at %s", ctime((time_t *)&smh->sh_time)); /* ctime() adds \n */
-	printf("File system size %.2fGB\n", smh->sh_fs_bytes / ((float)(1 << 30)));
+	sm->sm_format = be32_to_cpu(smh->sh_format);
+	sm->sm_time = be64_to_cpu(smh->sh_time);
+	sm->sm_fs_bytes = be64_to_cpu(smh->sh_fs_bytes);
+	printf("Metadata saved at %s", ctime(&sm->sm_time)); /* ctime() adds \n */
+	printf("File system size %.2fGB\n", sm->sm_fs_bytes / ((float)(1 << 30)));
 	return 0;
 }
 
@@ -1130,12 +1131,10 @@ void savemeta(char *out_fn, int saveoption, int gziplevel)
 	mfd = savemetaopen(out_fn, gziplevel);
 
 	blks_saved = 0;
-	if (sbd.gfs1)
-		sbd.bsize = sbd.sd_sb.sb_bsize;
-	printf("There are %llu blocks of %u bytes in the filesystem.\n",
-	                     (unsigned long long)sbd.fssize, sbd.bsize);
+	printf("There are %"PRIu64" blocks of %u bytes in the filesystem.\n",
+	       sbd.fssize, sbd.sd_bsize);
 
-	printf("Filesystem size: %.2fGB\n", (sbd.fssize * sbd.bsize) / ((float)(1 << 30)));
+	printf("Filesystem size: %.2fGB\n", (sbd.fssize * sbd.sd_bsize) / ((float)(1 << 30)));
 	get_journal_inode_blocks();
 
 	err = init_per_node_lookup();
@@ -1143,13 +1142,13 @@ void savemeta(char *out_fn, int saveoption, int gziplevel)
 		exit(1);
 
 	/* Write the savemeta file header */
-	err = save_header(&mfd, sbd.fssize * sbd.bsize);
+	err = save_header(&mfd, sbd.fssize * sbd.sd_bsize);
 	if (err) {
 		perror("Failed to write metadata file header");
 		exit(1);
 	}
 	/* Save off the superblock */
-	sb_addr = GFS2_SB_ADDR * GFS2_BASIC_BLOCK / sbd.bsize;
+	sb_addr = GFS2_SB_ADDR * GFS2_BASIC_BLOCK / sbd.sd_bsize;
 	buf = check_read_block(sbd.device_fd, sb_addr, 0, NULL, NULL);
 	if (buf != NULL) {
 		if (sbd.gfs1)
@@ -1164,10 +1163,10 @@ void savemeta(char *out_fn, int saveoption, int gziplevel)
 		uint64_t blk;
 		int j;
 
-		blk = sbd1->sb_rindex_di.no_addr;
+		blk = sbd.sd_rindex_di.in_addr;
 		buf = check_read_block(sbd.device_fd, blk, blk, NULL, NULL);
 		if (buf != NULL) {
-			save_buf(&mfd, buf, blk, sbd.bsize);
+			save_buf(&mfd, buf, blk, sbd.sd_bsize);
 			save_inode_data(&mfd, buf, blk);
 			free(buf);
 		}
@@ -1189,9 +1188,9 @@ void savemeta(char *out_fn, int saveoption, int gziplevel)
 	}
 	/* Walk through the resource groups saving everything within */
 	for (n = osi_first(&sbd.rgtree); n; n = osi_next(n)) {
-		struct rgrp_tree *rgd;
+		struct lgfs2_rgrp_tree *rgd;
 
-		rgd = (struct rgrp_tree *)n;
+		rgd = (struct lgfs2_rgrp_tree *)n;
 		save_rgrp(&sbd, &mfd, rgd, (saveoption != 2));
 	}
 	/* Clean up */
@@ -1208,38 +1207,39 @@ void savemeta(char *out_fn, int saveoption, int gziplevel)
 	close(sbd.device_fd);
 	destroy_per_node_lookup();
 	free(indirect);
-	gfs2_rgrp_free(&sbd, &sbd.rgtree);
+	lgfs2_rgrp_free(&sbd, &sbd.rgtree);
 	exit(0);
 }
 
-static char *restore_block(struct metafd *mfd, struct saved_metablock *svb)
+static char *restore_block(struct metafd *mfd, uint64_t *blk, uint16_t *siglen)
 {
-	struct saved_metablock *svb_be;
+	struct saved_metablock *svb;
 	const char *errstr;
 	char *buf = NULL;
 
-	svb_be = (struct saved_metablock *)(restore_buf_next(mfd, sizeof(*svb)));
-	if (svb_be == NULL)
+	svb = (struct saved_metablock *)(restore_buf_next(mfd, sizeof(*svb)));
+	if (svb == NULL)
 		goto nobuffer;
-	svb->blk = be64_to_cpu(svb_be->blk);
-	svb->siglen = be16_to_cpu(svb_be->siglen);
+	*blk = be64_to_cpu(svb->blk);
+	*siglen = be16_to_cpu(svb->siglen);
 
-	if (sbd.fssize && svb->blk >= sbd.fssize) {
+	if (sbd.fssize && *blk >= sbd.fssize) {
 		fprintf(stderr, "Error: File system is too small to restore this metadata.\n");
-		fprintf(stderr, "File system is %llu blocks. Restore block = %llu\n",
-		        (unsigned long long)sbd.fssize, (unsigned long long)svb->blk);
+		fprintf(stderr, "File system is %"PRIu64" blocks. Restore block = %"PRIu64"\n",
+		        sbd.fssize, *blk);
 		return NULL;
 	}
 
-	if (svb->siglen > sbd.bsize) {
+	if (*siglen > sbd.sd_bsize) {
 		fprintf(stderr, "Bad record length: %u for block %"PRIu64" (0x%"PRIx64").\n",
-			svb->siglen, svb->blk, svb->blk);
+			*siglen, *blk, *blk);
 		return NULL;
 	}
 
-	buf = restore_buf_next(mfd, svb->siglen);
-	if (buf != NULL)
+	buf = restore_buf_next(mfd, *siglen);
+	if (buf != NULL) {
 		return buf;
+	}
 nobuffer:
 	if (mfd->eof)
 		return NULL;
@@ -1253,17 +1253,15 @@ static int restore_super(struct metafd *mfd, void *buf, int printonly)
 {
 	int ret;
 
-	gfs2_sb_in(&sbd.sd_sb, buf);
-	sbd1 = (struct gfs_sb *)&sbd.sd_sb;
-	ret = check_sb(&sbd.sd_sb);
+	lgfs2_sb_in(&sbd, buf);
+	ret = lgfs2_check_sb(buf);
 	if (ret < 0) {
 		fprintf(stderr, "Error: Invalid superblock in metadata file.\n");
 		return -1;
 	}
 	if (ret == 1)
 		sbd.gfs1 = 1;
-	sbd.bsize = sbd.sd_sb.sb_bsize;
-	if ((!printonly) && lgfs2_sb_write(&sbd.sd_sb, sbd.device_fd, sbd.bsize)) {
+	if ((!printonly) && lgfs2_sb_write(&sbd, sbd.device_fd)) {
 		fprintf(stderr, "Failed to write superblock\n");
 		return -1;
 	}
@@ -1273,20 +1271,21 @@ static int restore_super(struct metafd *mfd, void *buf, int printonly)
 
 static int restore_data(int fd, struct metafd *mfd, int printonly)
 {
-	struct saved_metablock savedata = {0};
 	uint64_t writes = 0;
 	char *buf;
 
-	buf = calloc(1, sbd.bsize);
+	buf = calloc(1, sbd.sd_bsize);
 	if (buf == NULL) {
 		perror("Failed to restore data");
 		exit(1);
 	}
 
 	while (TRUE) {
+		uint16_t siglen = 0;
+		uint64_t blk = 0;
 		char *bp;
 
-		bp = restore_block(mfd, &savedata);
+		bp = restore_block(mfd, &blk, &siglen);
 		if (bp == NULL && mfd->eof)
 			break;
 		if (bp == NULL) {
@@ -1294,23 +1293,21 @@ static int restore_data(int fd, struct metafd *mfd, int printonly)
 			return -1;
 		}
 		if (printonly) {
-			if (printonly > 1 && printonly == savedata.blk) {
-				display_block_type(bp, savedata.blk, TRUE);
+			if (printonly > 1 && printonly == blk) {
+				display_block_type(bp, blk, TRUE);
 				display_gfs2(bp);
 				break;
 			} else if (printonly == 1) {
-				print_gfs2("%"PRId64" (l=0x%x): ", blks_saved, savedata.siglen);
-				display_block_type(bp, savedata.blk, TRUE);
+				print_gfs2("%"PRId64" (l=0x%x): ", blks_saved, siglen);
+				display_block_type(bp, blk, TRUE);
 			}
 		} else {
-			report_progress(savedata.blk, 0);
-			memcpy(buf, bp, savedata.siglen);
-			memset(buf + savedata.siglen, 0, sbd.bsize - savedata.siglen);
-			if (pwrite(fd, buf, sbd.bsize, savedata.blk * sbd.bsize) != sbd.bsize) {
-				fprintf(stderr, "write error: %s from %s:%d: block %lld (0x%llx)\n",
-					strerror(errno), __FUNCTION__, __LINE__,
-					(unsigned long long)savedata.blk,
-					(unsigned long long)savedata.blk);
+			report_progress(blk, 0);
+			memcpy(buf, bp, siglen);
+			memset(buf + siglen, 0, sbd.sd_bsize - siglen);
+			if (pwrite(fd, buf, sbd.sd_bsize, blk * sbd.sd_bsize) != sbd.sd_bsize) {
+				fprintf(stderr, "write error: %s from %s:%d: block %"PRIu64" (0x%"PRIx64")\n",
+					strerror(errno), __FUNCTION__, __LINE__, blk, blk);
 				free(buf);
 				return -1;
 			}
@@ -1326,15 +1323,15 @@ static int restore_data(int fd, struct metafd *mfd, int printonly)
 	return 0;
 }
 
-static void complain(const char *complaint)
+static void restoremeta_usage(void)
 {
-	fprintf(stderr, "%s\n", complaint);
-	die("Format is: \ngfs2_edit restoremeta <file to restore> "
-	    "<dest file system>\n");
+	fprintf(stderr, "Usage:\n");
+	fprintf(stderr, "gfs2_edit restoremeta <metadata_file> <block_device>\n");
 }
 
-static int restore_init(const char *path, struct metafd *mfd, struct savemeta_header *smh, int printonly)
+static int restore_init(const char *path, struct metafd *mfd, int printonly)
 {
+	struct savemeta sm = {0};
 	struct gfs2_sb rsb;
 	uint16_t sb_siglen;
 	char *end;
@@ -1362,10 +1359,10 @@ static int restore_init(const char *path, struct metafd *mfd, struct savemeta_he
 		return -1;
 	}
 	bp = restore_buf;
-	ret = parse_header(bp, smh);
+	ret = parse_header(bp, &sm);
 	if (ret == 0) {
-		bp = restore_buf + sizeof(*smh);
-		restore_off = sizeof(*smh);
+		bp = restore_buf + sizeof(struct savemeta_header);
+		restore_off = sizeof(struct savemeta_header);
 	} else if (ret == -1) {
 		return -1;
 	}
@@ -1388,12 +1385,12 @@ static int restore_init(const char *path, struct metafd *mfd, struct savemeta_he
 	if (ret != 0)
 		return ret;
 
-	if (smh->sh_fs_bytes > 0) {
-		sbd.fssize = smh->sh_fs_bytes / sbd.bsize;
+	if (sm.sm_fs_bytes > 0) {
+		sbd.fssize = sm.sm_fs_bytes / sbd.sd_bsize;
 		printf("Saved file system size is %"PRIu64" blocks, %.2fGB\n",
-		       sbd.fssize, smh->sh_fs_bytes / ((float)(1 << 30)));
+		       sbd.fssize, sm.sm_fs_bytes / ((float)(1 << 30)));
 	}
-	printf("Block size is %uB\n", sbd.bsize);
+	printf("Block size is %uB\n", sbd.sd_bsize);
 	printf("This is gfs%c metadata.\n", sbd.gfs1 ? '1': '2');
 	if (printonly > 1 && printonly == LGFS2_SB_ADDR(&sbd)) {
 		display_block_type(bp, LGFS2_SB_ADDR(&sbd), TRUE);
@@ -1410,31 +1407,37 @@ static int restore_init(const char *path, struct metafd *mfd, struct savemeta_he
 
 void restoremeta(const char *in_fn, const char *out_device, uint64_t printonly)
 {
-	struct savemeta_header smh = {0};
 	struct metafd mfd = {0};
 	int error;
 
 	termlines = 0;
-	if (!in_fn)
-		complain("No source file specified.");
-	if (!printonly && !out_device)
-		complain("No destination file system specified.");
-
+	if (in_fn == NULL || in_fn[0] == '\0') {
+		fprintf(stderr, "No source file specified.");
+		restoremeta_usage();
+		exit(1);
+	}
+	if (!printonly && (out_device == NULL || out_device[0] == '\0')) {
+		fprintf(stderr, "No destination file system specified.");
+		restoremeta_usage();
+		exit(1);
+	}
 	if (!printonly) {
 		sbd.device_fd = open(out_device, O_RDWR);
-		if (sbd.device_fd < 0)
-			die("Can't open destination file system %s: %s\n",
+		if (sbd.device_fd < 0) {
+			fprintf(stderr, "Failed to open target '%s': %s\n",
 			    out_device, strerror(errno));
+			exit(1);
+		}
 	} else if (out_device) /* for printsavedmeta, the out_device is an
 				  optional block no */
 		printonly = check_keywords(out_device);
 
-	error = restore_init(in_fn, &mfd, &smh, printonly);
+	error = restore_init(in_fn, &mfd, printonly);
 	if (error != 0)
 		exit(error);
 
 	if (!printonly) {
-		uint64_t space = lseek(sbd.device_fd, 0, SEEK_END) / sbd.bsize;
+		uint64_t space = lseek(sbd.device_fd, 0, SEEK_END) / sbd.sd_bsize;
 		printf("There are %"PRIu64" free blocks on the destination device.\n", space);
 	}
 
