@@ -19,26 +19,28 @@
 #define JOURNAL_NAME_SIZE 18
 #define JOURNAL_SEQ_TOLERANCE 10
 
-unsigned int sd_found_jblocks = 0, sd_replayed_jblocks = 0;
-unsigned int sd_found_metablocks = 0, sd_replayed_metablocks = 0;
-unsigned int sd_found_revokes = 0;
-osi_list_t sd_revoke_list;
-unsigned int sd_replay_tail;
+static unsigned int sd_found_jblocks = 0;
+static unsigned int sd_replayed_jblocks = 0;
+static unsigned int sd_found_metablocks = 0;
+static unsigned int sd_replayed_metablocks = 0;
+static unsigned int sd_found_revokes = 0;
+static osi_list_t sd_revoke_list;
+static unsigned int sd_replay_tail;
 
-struct gfs2_revoke_replay {
+struct revoke_replay {
 	osi_list_t rr_list;
 	uint64_t rr_blkno;
 	unsigned int rr_where;
 };
 
-int gfs2_revoke_add(struct gfs2_sbd *sdp, uint64_t blkno, unsigned int where)
+static int revoke_add(struct lgfs2_sbd *sdp, uint64_t blkno, unsigned int where)
 {
 	osi_list_t *tmp, *head = &sd_revoke_list;
-	struct gfs2_revoke_replay *rr;
+	struct revoke_replay *rr;
 	int found = 0;
 
 	osi_list_foreach(tmp, head) {
-		rr = osi_list_entry(tmp, struct gfs2_revoke_replay, rr_list);
+		rr = osi_list_entry(tmp, struct revoke_replay, rr_list);
 		if (rr->rr_blkno == blkno) {
 			found = 1;
 			break;
@@ -50,7 +52,7 @@ int gfs2_revoke_add(struct gfs2_sbd *sdp, uint64_t blkno, unsigned int where)
 		return 0;
 	}
 
-	rr = malloc(sizeof(struct gfs2_revoke_replay));
+	rr = malloc(sizeof(struct revoke_replay));
 	if (!rr)
 		return -ENOMEM;
 
@@ -60,15 +62,15 @@ int gfs2_revoke_add(struct gfs2_sbd *sdp, uint64_t blkno, unsigned int where)
 	return 1;
 }
 
-int gfs2_revoke_check(struct gfs2_sbd *sdp, uint64_t blkno, unsigned int where)
+static int revoke_check(struct lgfs2_sbd *sdp, uint64_t blkno, unsigned int where)
 {
 	osi_list_t *tmp;
-	struct gfs2_revoke_replay *rr;
+	struct revoke_replay *rr;
 	int wrap, a, b;
 	int found = 0;
 
 	osi_list_foreach(tmp, &sd_revoke_list) {
-		rr = osi_list_entry(tmp, struct gfs2_revoke_replay, rr_list);
+		rr = osi_list_entry(tmp, struct revoke_replay, rr_list);
 		if (rr->rr_blkno == blkno) {
 			found = 1;
 			break;
@@ -84,100 +86,96 @@ int gfs2_revoke_check(struct gfs2_sbd *sdp, uint64_t blkno, unsigned int where)
 	return (wrap) ? (a || b) : (a && b);
 }
 
-void gfs2_revoke_clean(struct gfs2_sbd *sdp)
+static void revoke_clean(struct lgfs2_sbd *sdp)
 {
 	osi_list_t *head = &sd_revoke_list;
-	struct gfs2_revoke_replay *rr;
+	struct revoke_replay *rr;
 
 	while (!osi_list_empty(head)) {
-		rr = osi_list_entry(head->next, struct gfs2_revoke_replay, rr_list);
+		rr = osi_list_entry(head->next, struct revoke_replay, rr_list);
 		osi_list_del(&rr->rr_list);
 		free(rr);
 	}
 }
 
-static void refresh_rgrp(struct gfs2_sbd *sdp, struct rgrp_tree *rgd,
-			 struct gfs2_buffer_head *bh, uint64_t blkno)
+static void refresh_rgrp(struct lgfs2_sbd *sdp, struct lgfs2_rgrp_tree *rgd,
+			 struct lgfs2_buffer_head *bh, uint64_t blkno)
 {
 	int i;
 
-	log_debug(_("Block is part of rgrp 0x%llx; refreshing the rgrp.\n"),
-		  (unsigned long long)rgd->ri.ri_addr);
-	for (i = 0; i < rgd->ri.ri_length; i++) {
-		if (rgd->ri.ri_addr + i != blkno)
+	log_debug(_("Block is part of rgrp 0x%"PRIx64"; refreshing the rgrp.\n"),
+	          rgd->rt_addr);
+	for (i = 0; i < rgd->rt_length; i++) {
+		if (rgd->rt_addr + i != blkno)
 			continue;
 
-		memcpy(rgd->bits[i].bi_data, bh->b_data, sdp->bsize);
-		rgd->bits[i].bi_modified = 1;
+		memcpy(rgd->rt_bits[i].bi_data, bh->b_data, sdp->sd_bsize);
+		rgd->rt_bits[i].bi_modified = 1;
 		if (i == 0) { /* this is the rgrp itself */
 			if (sdp->gfs1)
-				gfs_rgrp_in((struct gfs_rgrp *)&rgd->rg, rgd->bits[0].bi_data);
+				lgfs2_gfs_rgrp_in(rgd,
+						  rgd->rt_bits[0].bi_data);
 			else
-				gfs2_rgrp_in(&rgd->rg, rgd->bits[0].bi_data);
+				lgfs2_rgrp_in(rgd, rgd->rt_bits[0].bi_data);
 		}
 		break;
 	}
 }
 
-static int buf_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
+static int buf_lo_scan_elements(struct lgfs2_inode *ip, unsigned int start,
 				struct gfs2_log_descriptor *ld, __be64 *ptr,
 				int pass)
 {
-	struct gfs2_sbd *sdp = ip->i_sbd;
+	struct lgfs2_sbd *sdp = ip->i_sbd;
 	unsigned int blks = be32_to_cpu(ld->ld_data1);
-	struct gfs2_buffer_head *bh_log, *bh_ip;
 	uint64_t blkno;
 	int error = 0;
-	struct rgrp_tree *rgd;
+	struct lgfs2_rgrp_tree *rgd;
 
 	if (pass != 1 || be32_to_cpu(ld->ld_type) != GFS2_LOG_DESC_METADATA)
 		return 0;
 
-	gfs2_replay_incr_blk(ip, &start);
+	lgfs2_replay_incr_blk(ip, &start);
 
-	for (; blks; gfs2_replay_incr_blk(ip, &start), blks--) {
-		uint32_t check_magic;
+	for (; blks; lgfs2_replay_incr_blk(ip, &start), blks--) {
+		struct lgfs2_buffer_head *bh_log, *bh_ip;
+		struct gfs2_meta_header *mhp;
 
 		sd_found_metablocks++;
 
 		blkno = be64_to_cpu(*ptr);
 		ptr++;
-		if (gfs2_revoke_check(sdp, blkno, start))
+		if (revoke_check(sdp, blkno, start))
 			continue;
 
-		error = gfs2_replay_read_block(ip, start, &bh_log);
+		error = lgfs2_replay_read_block(ip, start, &bh_log);
 		if (error)
 			return error;
 
-		log_info( _("Journal replay writing metadata block #"
-			    "%lld (0x%llx) for journal+0x%x\n"),
-			  (unsigned long long)blkno, (unsigned long long)blkno,
-			  start);
-		bh_ip = bget(sdp, blkno);
+		log_info(_("Journal replay writing metadata block #%"PRIu64" (0x%"PRIx64") for journal+0x%x\n"),
+		         blkno, blkno, start);
+		bh_ip = lgfs2_bget(sdp, blkno);
 		if (!bh_ip) {
 			log_err(_("Out of memory when replaying journals.\n"));
+			lgfs2_bfree(&bh_log);
 			return FSCK_ERROR;
 		}
-		memcpy(bh_ip->b_data, bh_log->b_data, sdp->bsize);
+		memcpy(bh_ip->b_data, bh_log->b_data, sdp->sd_bsize);
 
-		check_magic = ((struct gfs2_meta_header *)
-			       (bh_ip->b_data))->mh_magic;
-		check_magic = be32_to_cpu(check_magic);
-		if (check_magic != GFS2_MAGIC) {
-			log_err(_("Journal corruption detected at block #"
-				  "%lld (0x%llx) for journal+0x%x.\n"),
-				(unsigned long long)blkno, (unsigned long long)blkno,
-				start);
+		mhp = (struct gfs2_meta_header *)bh_ip->b_data;
+		if (be32_to_cpu(mhp->mh_magic) != GFS2_MAGIC) {
+			log_err(_("Journal corruption detected at block #%"PRIu64" (0x%"PRIx64") for journal+0x%x.\n"),
+			        blkno, blkno, start);
 			error = -EIO;
 		} else {
-			bmodified(bh_ip);
-			rgd = gfs2_blk2rgrpd(sdp, blkno);
-			if (rgd && blkno < rgd->ri.ri_data0)
+			lgfs2_bmodified(bh_ip);
+			rgd = lgfs2_blk2rgrpd(sdp, blkno);
+			if (rgd && blkno < rgd->rt_data0)
 				refresh_rgrp(sdp, rgd, bh_ip, blkno);
 		}
 
-		brelse(bh_log);
-		brelse(bh_ip);
+		lgfs2_brelse(bh_log);
+		lgfs2_brelse(bh_ip);
 		if (error)
 			break;
 
@@ -186,14 +184,13 @@ static int buf_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
 	return error;
 }
 
-static int revoke_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
+static int revoke_lo_scan_elements(struct lgfs2_inode *ip, unsigned int start,
 				   struct gfs2_log_descriptor *ld, __be64 *ptr,
 				   int pass)
 {
-	struct gfs2_sbd *sdp = ip->i_sbd;
+	struct lgfs2_sbd *sdp = ip->i_sbd;
 	unsigned int blks = be32_to_cpu(ld->ld_length);
 	unsigned int revokes = be32_to_cpu(ld->ld_data1);
-	struct gfs2_buffer_head *bh;
 	unsigned int offset;
 	uint64_t blkno;
 	int first = 1;
@@ -204,25 +201,26 @@ static int revoke_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
 
 	offset = sizeof(struct gfs2_log_descriptor);
 
-	for (; blks; gfs2_replay_incr_blk(ip, &start), blks--) {
-		error = gfs2_replay_read_block(ip, start, &bh);
+	for (; blks; lgfs2_replay_incr_blk(ip, &start), blks--) {
+		struct lgfs2_buffer_head *bh;
+
+		error = lgfs2_replay_read_block(ip, start, &bh);
 		if (error)
 			return error;
 
 		if (!first) {
-			if (gfs2_check_meta(bh->b_data, GFS2_METATYPE_LB))
+			if (lgfs2_check_meta(bh->b_data, GFS2_METATYPE_LB))
 				continue;
 		}
-		while (offset + sizeof(uint64_t) <= sdp->sd_sb.sb_bsize) {
+		while (offset + sizeof(uint64_t) <= sdp->sd_bsize) {
 			blkno = be64_to_cpu(*(__be64 *)(bh->b_data + offset));
-			log_info( _("Journal replay processing revoke for "
-				    "block #%lld (0x%llx) for journal+0x%x\n"),
-				  (unsigned long long)blkno,
-				  (unsigned long long)blkno,
-				  start);
-			error = gfs2_revoke_add(sdp, blkno, start);
-			if (error < 0)
+			log_info(_("Journal replay processing revoke for block #%"PRIu64" (0x%"PRIx64") for journal+0x%x\n"),
+			         blkno, blkno, start);
+			error = revoke_add(sdp, blkno, start);
+			if (error < 0) {
+				lgfs2_bfree(&bh);
 				return error;
+			}
 			else if (error)
 				sd_found_revokes++;
 
@@ -231,21 +229,20 @@ static int revoke_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
 			offset += sizeof(uint64_t);
 		}
 
-		bmodified(bh);
-		brelse(bh);
+		lgfs2_bmodified(bh);
+		lgfs2_brelse(bh);
 		offset = sizeof(struct gfs2_meta_header);
 		first = 0;
 	}
 	return 0;
 }
 
-static int databuf_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
+static int databuf_lo_scan_elements(struct lgfs2_inode *ip, unsigned int start,
 				    struct gfs2_log_descriptor *ld,
 				    __be64 *ptr, int pass)
 {
-	struct gfs2_sbd *sdp = ip->i_sbd;
+	struct lgfs2_sbd *sdp = ip->i_sbd;
 	unsigned int blks = be32_to_cpu(ld->ld_data1);
-	struct gfs2_buffer_head *bh_log, *bh_ip;
 	uint64_t blkno;
 	uint64_t esc;
 	int error = 0;
@@ -253,8 +250,10 @@ static int databuf_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
 	if (pass != 1 || be32_to_cpu(ld->ld_type) != GFS2_LOG_DESC_JDATA)
 		return 0;
 
-	gfs2_replay_incr_blk(ip, &start);
-	for (; blks; gfs2_replay_incr_blk(ip, &start), blks--) {
+	lgfs2_replay_incr_blk(ip, &start);
+	for (; blks; lgfs2_replay_incr_blk(ip, &start), blks--) {
+		struct lgfs2_buffer_head *bh_log, *bh_ip;
+
 		blkno = be64_to_cpu(*ptr);
 		ptr++;
 		esc = be64_to_cpu(*ptr);
@@ -262,23 +261,22 @@ static int databuf_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
 
 		sd_found_jblocks++;
 
-		if (gfs2_revoke_check(sdp, blkno, start))
+		if (revoke_check(sdp, blkno, start))
 			continue;
 
-		error = gfs2_replay_read_block(ip, start, &bh_log);
+		error = lgfs2_replay_read_block(ip, start, &bh_log);
 		if (error)
 			return error;
 
-		log_info( _("Journal replay writing data block #%lld (0x%llx)"
-			    " for journal+0x%x\n"),
-			  (unsigned long long)blkno, (unsigned long long)blkno,
-			  start);
-		bh_ip = bget(sdp, blkno);
+		log_info(_("Journal replay writing data block #%"PRIu64" (0x%"PRIx64") for journal+0x%x\n"),
+		         blkno, blkno, start);
+		bh_ip = lgfs2_bget(sdp, blkno);
 		if (!bh_ip) {
 			log_err(_("Out of memory when replaying journals.\n"));
+			lgfs2_bfree(&bh_log);
 			return FSCK_ERROR;
 		}
-		memcpy(bh_ip->b_data, bh_log->b_data, sdp->bsize);
+		memcpy(bh_ip->b_data, bh_log->b_data, sdp->sd_bsize);
 
 		/* Unescape */
 		if (esc) {
@@ -286,9 +284,9 @@ static int databuf_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
 			*eptr = cpu_to_be32(GFS2_MAGIC);
 		}
 
-		brelse(bh_log);
-		bmodified(bh_ip);
-		brelse(bh_ip);
+		lgfs2_brelse(bh_log);
+		lgfs2_bmodified(bh_ip);
+		lgfs2_brelse(bh_ip);
 
 		sd_replayed_jblocks++;
 	}
@@ -307,10 +305,10 @@ static int databuf_lo_scan_elements(struct gfs2_inode *ip, unsigned int start,
  * Returns: errno
  */
 
-static int foreach_descriptor(struct gfs2_inode *ip, unsigned int start,
+static int foreach_descriptor(struct lgfs2_inode *ip, unsigned int start,
 		       unsigned int end, int pass)
 {
-	struct gfs2_buffer_head *bh;
+	struct lgfs2_buffer_head *bh;
 	struct gfs2_log_descriptor *ld;
 	int error = 0;
 	uint32_t length;
@@ -320,30 +318,28 @@ static int foreach_descriptor(struct gfs2_inode *ip, unsigned int start,
 	offset &= ~(sizeof(__be64) - 1);
 
 	while (start != end) {
-		uint32_t check_magic;
+		struct gfs2_meta_header *mhp;
 
-		error = gfs2_replay_read_block(ip, start, &bh);
+		error = lgfs2_replay_read_block(ip, start, &bh);
 		if (error)
 			return error;
-		check_magic = ((struct gfs2_meta_header *)
-			       (bh->b_data))->mh_magic;
-		check_magic = be32_to_cpu(check_magic);
-		if (check_magic != GFS2_MAGIC) {
-			bmodified(bh);
-			brelse(bh);
+		mhp = (struct gfs2_meta_header *)bh->b_data;
+		if (be32_to_cpu(mhp->mh_magic) != GFS2_MAGIC) {
+			lgfs2_bmodified(bh);
+			lgfs2_brelse(bh);
 			return -EIO;
 		}
 		ld = (struct gfs2_log_descriptor *)bh->b_data;
 		length = be32_to_cpu(ld->ld_length);
 
 		if (be32_to_cpu(ld->ld_header.mh_type) == GFS2_METATYPE_LH) {
-			struct gfs2_log_header lh;
+			struct lgfs2_log_header lh;
 
-			error = get_log_header(ip, start, &lh);
+			error = lgfs2_get_log_header(ip, start, &lh);
 			if (!error) {
-				gfs2_replay_incr_blk(ip, &start);
-				bmodified(bh);
-				brelse(bh);
+				lgfs2_replay_incr_blk(ip, &start);
+				lgfs2_bmodified(bh);
+				lgfs2_brelse(bh);
 				continue;
 			}
 			if (error == 1) {
@@ -351,39 +347,39 @@ static int foreach_descriptor(struct gfs2_inode *ip, unsigned int start,
 					  "journal+0x%x.\n"), start);
 				error = -EIO;
 			}
-			bmodified(bh);
-			brelse(bh);
+			lgfs2_bmodified(bh);
+			lgfs2_brelse(bh);
 			return error;
-		} else if (gfs2_check_meta(bh->b_data, GFS2_METATYPE_LD)) {
-			bmodified(bh);
-			brelse(bh);
+		} else if (lgfs2_check_meta(bh->b_data, GFS2_METATYPE_LD)) {
+			lgfs2_bmodified(bh);
+			lgfs2_brelse(bh);
 			return -EIO;
 		}
 		ptr = (__be64 *)(bh->b_data + offset);
 		error = databuf_lo_scan_elements(ip, start, ld, ptr, pass);
 		if (error) {
-			bmodified(bh);
-			brelse(bh);
+			lgfs2_bmodified(bh);
+			lgfs2_brelse(bh);
 			return error;
 		}
 		error = buf_lo_scan_elements(ip, start, ld, ptr, pass);
 		if (error) {
-			bmodified(bh);
-			brelse(bh);
+			lgfs2_bmodified(bh);
+			lgfs2_brelse(bh);
 			return error;
 		}
 		error = revoke_lo_scan_elements(ip, start, ld, ptr, pass);
 		if (error) {
-			bmodified(bh);
-			brelse(bh);
+			lgfs2_bmodified(bh);
+			lgfs2_brelse(bh);
 			return error;
 		}
 
 		while (length--)
-			gfs2_replay_incr_blk(ip, &start);
+			lgfs2_replay_incr_blk(ip, &start);
 
-		bmodified(bh);
-		brelse(bh);
+		lgfs2_bmodified(bh);
+		lgfs2_brelse(bh);
 	}
 
 	return 0;
@@ -396,21 +392,21 @@ static int foreach_descriptor(struct gfs2_inode *ip, unsigned int start,
  *
  * Returns: The number of sequencing errors (hopefully none).
  */
-static int check_journal_seq_no(struct gfs2_inode *ip, int fix)
+static int check_journal_seq_no(struct lgfs2_inode *ip, int fix)
 {
 	int error = 0, wrapped = 0;
-	uint32_t jd_blocks = ip->i_di.di_size / ip->i_sbd->sd_sb.sb_bsize;
+	uint32_t jd_blocks = ip->i_size / ip->i_sbd->sd_bsize;
 	uint32_t blk;
-	struct gfs2_log_header lh;
+	struct lgfs2_log_header lh;
 	uint64_t highest_seq = 0, lowest_seq = 0, prev_seq = 0;
 	int new = 0;
 	uint64_t dblock;
-	struct gfs2_buffer_head *bh;
+	struct lgfs2_buffer_head *bh;
 	int seq_errors = 0;
 
 	memset(&lh, 0, sizeof(lh));
 	for (blk = 0; blk < jd_blocks; blk++) {
-		error = get_log_header(ip, blk, &lh);
+		error = lgfs2_get_log_header(ip, blk, &lh);
 		if (error == 1) /* if not a log header */
 			continue; /* just journal data--ignore it */
 		if (!lowest_seq || lh.lh_sequence < lowest_seq)
@@ -428,24 +424,25 @@ static int check_journal_seq_no(struct gfs2_inode *ip, int fix)
 			prev_seq = lh.lh_sequence;
 			continue;
 		}
-		log_err( _("Journal block %u (0x%x): sequence no. 0x%llx "
+		log_err(_("Journal block %"PRIu32" (0x%"PRIx32"): sequence no. 0x%"PRIx64" "
 			   "out of order.\n"), blk, blk, lh.lh_sequence);
-		log_info( _("Low: 0x%llx, High: 0x%llx, Prev: 0x%llx\n"),
-			  (unsigned long long)lowest_seq,
-			  (unsigned long long)highest_seq,
-			  (unsigned long long)prev_seq);
+		log_info(_("Low: 0x%"PRIx64", High: 0x%"PRIx64", Prev: 0x%"PRIx64"\n"),
+		         lowest_seq, highest_seq, prev_seq);
 		seq_errors++;
 		if (!fix)
 			continue;
 		highest_seq++;
-		lh.lh_sequence = highest_seq;
-		prev_seq = lh.lh_sequence;
-		log_warn( _("Renumbering it as 0x%llx\n"), lh.lh_sequence);
-		block_map(ip, blk, &new, &dblock, NULL, 0);
-		bh = bread(ip->i_sbd, dblock);
-		gfs2_log_header_out(&lh, bh->b_data);
-		bmodified(bh);
-		brelse(bh);
+		prev_seq = highest_seq;
+		log_warn(_("Renumbering it as 0x%"PRIx64"\n"), highest_seq);
+		if (lgfs2_block_map(ip, blk, &new, &dblock, NULL, 0)) {
+			log_crit(_("Failed to map block 0x%"PRIx32" in journal at 0x%"PRIx64": %s\n"),
+			         blk, ip->i_num.in_addr, strerror(errno));
+			exit(1);
+		}
+		bh = lgfs2_bread(ip->i_sbd, dblock);
+		((struct gfs2_log_header *)bh->b_data)->lh_sequence = cpu_to_be64(highest_seq);
+		lgfs2_bmodified(bh);
+		lgfs2_brelse(bh);
 	}
 	if (seq_errors && fix) {
 		log_err(_("%d sequence errors fixed.\n"), seq_errors);
@@ -464,13 +461,13 @@ static int check_journal_seq_no(struct gfs2_inode *ip, int fix)
  * mounted by other nodes in the cluster, which is dangerous and therefore,
  * we should warn the user to run fsck.gfs2 manually when it's safe.
  */
-int preen_is_safe(struct gfs2_sbd *sdp, int preen, int force_check)
+int preen_is_safe(struct lgfs2_sbd *sdp, const struct fsck_options * const opts)
 {
-	if (!preen)       /* If preen was not specified */
+	if (!opts->preen)
 		return 1; /* not called by rc.sysinit--we're okay to preen */
-	if (force_check)  /* If check was forced by the user? */
+	if (opts->force)
 		return 1; /* user's responsibility--we're okay to preen */
-	if (!memcmp(sdp->sd_sb.sb_lockproto + 5, "nolock", 6))
+	if (!memcmp(sdp->sd_lockproto + 5, "nolock", 6))
 		return 1; /* local file system--preen is okay */
 	return 0; /* might be mounted on another node--not guaranteed safe */
 }
@@ -490,11 +487,10 @@ int preen_is_safe(struct gfs2_sbd *sdp, int preen, int force_check)
  * Returns: errno
  */
 
-static int gfs2_recover_journal(struct gfs2_inode *ip, int j, int preen,
-				int force_check, int *was_clean)
+static int recover_journal(struct lgfs2_inode *ip, int j, struct fsck_cx *cx, int *was_clean)
 {
-	struct gfs2_sbd *sdp = ip->i_sbd;
-	struct gfs2_log_header head;
+	struct lgfs2_sbd *sdp = ip->i_sbd;
+	struct lgfs2_log_header head;
 	unsigned int pass;
 	int error;
 
@@ -502,7 +498,7 @@ static int gfs2_recover_journal(struct gfs2_inode *ip, int j, int preen,
 	log_info( _("jid=%u: Looking at journal...\n"), j);
 
 	osi_list_init(&sd_revoke_list);
-	error = gfs2_find_jhead(ip, &head);
+	error = lgfs2_find_jhead(ip, &head);
 	if (!error) {
 		error = check_journal_seq_no(ip, 0);
 		if (error > JOURNAL_SEQ_TOLERANCE) {
@@ -513,12 +509,12 @@ static int gfs2_recover_journal(struct gfs2_inode *ip, int j, int preen,
 		}
 	}
 	if (error) {
-		if (opts.no) {
+		if (cx->opts->no) {
 			log_err( _("Journal #%d (\"journal%d\") is corrupt\n"),j+1, j);
 			log_err( _("Not fixing it due to the -n option.\n"));
 			goto out;
 		}
-		if (!preen_is_safe(sdp, preen, force_check)) {
+		if (!preen_is_safe(sdp, cx->opts)) {
 			log_err(_("Journal #%d (\"journal%d\") is corrupt.\n"),
 				j+1, j);
 			log_err(_("I'm not fixing it because it may be unsafe:\n"
@@ -529,7 +525,7 @@ static int gfs2_recover_journal(struct gfs2_inode *ip, int j, int preen,
 				 "without -a or -p.\n"));
 			goto out;
 		}
-		if (!query( _("\nJournal #%d (\"journal%d\") is "
+		if (!query(cx, _("\nJournal #%d (\"journal%d\") is "
 			      "corrupt.  Okay to repair it? (y/n)"),
 			    j+1, j)) {
 			log_err( _("jid=%u: The journal was not repaired.\n"),
@@ -543,7 +539,7 @@ static int gfs2_recover_journal(struct gfs2_inode *ip, int j, int preen,
 				 j);
 			goto out;
 		}
-		error = gfs2_find_jhead(ip, &head);
+		error = lgfs2_find_jhead(ip, &head);
 		if (error) {
 			log_err( _("jid=%u: Unable to fix the bad journal.\n"),
 				 j);
@@ -557,12 +553,12 @@ static int gfs2_recover_journal(struct gfs2_inode *ip, int j, int preen,
 		*was_clean = 1;
 		return 0;
 	}
-	if (opts.no) {
+	if (cx->opts->no) {
 		log_err(_("Journal #%d (\"journal%d\") is dirty\n"),j+1, j);
 		log_err(_("not replaying due to the -n option.\n"));
 		goto out;
 	}
-	if (!preen_is_safe(sdp, preen, force_check)) {
+	if (!preen_is_safe(sdp, cx->opts)) {
 		log_err( _("Journal #%d (\"journal%d\") is dirty\n"), j+1, j);
 		log_err( _("I'm not replaying it because it may be unsafe:\n"
 			   "Locking protocol is not lock_nolock and "
@@ -573,7 +569,7 @@ static int gfs2_recover_journal(struct gfs2_inode *ip, int j, int preen,
 		error = FSCK_ERROR;
 		goto out;
 	}
-	if (!query( _("\nJournal #%d (\"journal%d\") is dirty.  Okay to "
+	if (!query(cx, _("\nJournal #%d (\"journal%d\") is dirty.  Okay to "
 		      "replay it? (y/n)"), j+1, j))
 		goto reinit;
 
@@ -592,8 +588,8 @@ static int gfs2_recover_journal(struct gfs2_inode *ip, int j, int preen,
 		}
 	}
 	log_info( _("jid=%u: Found %u revoke tags\n"), j, sd_found_revokes);
-	gfs2_revoke_clean(sdp);
-	error = clean_journal(ip, &head);
+	revoke_clean(sdp);
+	error = lgfs2_clean_journal(ip, &head);
 	if (error)
 		goto out;
 	log_err( _("jid=%u: Replayed %u of %u journaled data blocks\n"),
@@ -610,10 +606,10 @@ out:
 	}
 	log_err( _("jid=%u: Failed\n"), j);
 reinit:
-	if (query( _("Do you want to clear the journal instead? (y/n)"))) {
-		error = write_journal(sdp->md.journal[j], sdp->bsize,
-				      sdp->md.journal[j]->i_di.di_size /
-				      sdp->sd_sb.sb_bsize);
+	if (query(cx, _("Do you want to clear the journal instead? (y/n)"))) {
+		error = lgfs2_write_journal(sdp->md.journal[j], sdp->sd_bsize,
+				      sdp->md.journal[j]->i_size /
+				      sdp->sd_bsize);
 		log_err(_("jid=%u: journal was cleared.\n"), j);
 	} else {
 		log_err( _("jid=%u: journal not cleared.\n"), j);
@@ -623,22 +619,21 @@ reinit:
 
 /* We can't use the rangecheck function from pass1 because we haven't gone
  * through initialization properly yet. */
-static int rangecheck_jblock(struct gfs2_inode *ip, uint64_t block)
+static int rangecheck_jblock(struct lgfs2_inode *ip, uint64_t block)
 {
 	if((block > ip->i_sbd->fssize) || (block <= LGFS2_SB_ADDR(ip->i_sbd))) {
 		log_info( _("Bad block pointer (out of range) found in "
-			    "journal inode %lld (0x%llx).\n"),
-			  (unsigned long long)ip->i_di.di_num.no_addr,
-			  (unsigned long long)ip->i_di.di_num.no_addr);
-		return meta_error; /* Exits check_metatree quicker */
+			    "journal inode %"PRIu64" (0x%"PRIx64").\n"),
+			  ip->i_num.in_addr, ip->i_num.in_addr);
+		return META_ERROR; /* Exits check_metatree quicker */
 	}
-	return meta_is_good;
+	return META_IS_GOOD;
 }
 
-static int rangecheck_jmeta(struct iptr iptr, struct gfs2_buffer_head **bh, int h,
+static int rangecheck_jmeta(struct fsck_cx *cx, struct iptr iptr, struct lgfs2_buffer_head **bh, int h,
                             int *is_valid, int *was_duplicate, void *private)
 {
-	struct gfs2_inode *ip = iptr.ipt_ip;
+	struct lgfs2_inode *ip = iptr.ipt_ip;
 	uint64_t block = iptr_block(iptr);
 	int rc;
 
@@ -646,34 +641,31 @@ static int rangecheck_jmeta(struct iptr iptr, struct gfs2_buffer_head **bh, int
 	*was_duplicate = 0;
 	*is_valid = 0;
 	rc = rangecheck_jblock(ip, block);
-	if (rc == meta_is_good) {
-		*bh = bread(ip->i_sbd, block);
-		*is_valid = (gfs2_check_meta((*bh)->b_data, GFS2_METATYPE_IN) == 0);
+	if (rc == META_IS_GOOD) {
+		*bh = lgfs2_bread(ip->i_sbd, block);
+		*is_valid = (lgfs2_check_meta((*bh)->b_data, GFS2_METATYPE_IN) == 0);
 		if (!(*is_valid)) {
-			log_err( _("Journal at block %lld (0x%llx) has a bad "
-				   "indirect block pointer %lld (0x%llx) "
+			log_err( _("Journal at block %"PRIu64" (0x%"PRIx64") has a bad "
+				   "indirect block pointer %"PRIu64" (0x%"PRIx64") "
 				   "(points to something that is not an "
 				   "indirect block).\n"),
-				 (unsigned long long)ip->i_di.di_num.no_addr,
-				 (unsigned long long)ip->i_di.di_num.no_addr,
-				 (unsigned long long)block,
-				 (unsigned long long)block);
-			brelse(*bh);
+			        ip->i_num.in_addr, ip->i_num.in_addr, block, block);
+			lgfs2_brelse(*bh);
 			*bh = NULL;
-			return meta_skip_further;
+			return META_SKIP_FURTHER;
 		}
 	}
 	return rc;
 }
 
-static int rangecheck_jdata(struct gfs2_inode *ip, uint64_t metablock,
+static int rangecheck_jdata(struct fsck_cx *cx, struct lgfs2_inode *ip, uint64_t metablock,
 			    uint64_t block, void *private,
-			    struct gfs2_buffer_head *bh, uint64_t *ptr)
+			    struct lgfs2_buffer_head *bh, __be64 *ptr)
 {
 	return rangecheck_jblock(ip, block);
 }
 
-struct metawalk_fxns rangecheck_journal = {
+static struct metawalk_fxns rangecheck_journal = {
 	.private = NULL,
 	.invalid_meta_is_fatal = 1,
 	.check_metalist = rangecheck_jmeta,
@@ -693,24 +685,27 @@ struct metawalk_fxns rangecheck_journal = {
  *
  * Returns: 0 on success, -1 on failure
  */
-int replay_journals(struct gfs2_sbd *sdp, int preen, int force_check,
-		    int *clean_journals)
+int replay_journals(struct fsck_cx *cx, int *clean_journals)
 {
+	struct lgfs2_sbd *sdp = cx->sdp;
+	int dirty_journals = 0;
+	int gave_msg = 0;
+	int error = 0;
+	int clean = 0;
 	int i;
-	int clean = 0, dirty_journals = 0, error = 0, gave_msg = 0;
 
 	*clean_journals = 0;
 
-	sdp->jsize = GFS2_DEFAULT_JSIZE;
+	sdp->jsize = LGFS2_DEFAULT_JSIZE;
 
 	for(i = 0; i < sdp->md.journals; i++) {
 		if (sdp->md.journal[i]) {
-			error = check_metatree(sdp->md.journal[i],
+			error = check_metatree(cx, sdp->md.journal[i],
 					       &rangecheck_journal);
 			if (error)
 				/* Don't use fsck_inode_put here because it's a
 				   system file and we need to dismantle it. */
-				inode_put(&sdp->md.journal[i]);
+				lgfs2_inode_put(&sdp->md.journal[i]);
 			error = 0; /* bad journal is non-fatal */
 		}
 		if (!sdp->md.journal[i]) {
@@ -720,19 +715,16 @@ int replay_journals(struct gfs2_sbd *sdp, int preen, int force_check,
 			continue;
 		}
 		if (!error) {
-			uint64_t jsize = sdp->md.journal[i]->i_di.di_size /
-				(1024 * 1024);
+			uint64_t jsize = sdp->md.journal[i]->i_size / (1024 * 1024);
 
-			if (sdp->jsize == GFS2_DEFAULT_JSIZE && jsize &&
+			if (sdp->jsize == LGFS2_DEFAULT_JSIZE && jsize &&
 			    jsize != sdp->jsize)
 				sdp->jsize = jsize;
-			error = gfs2_recover_journal(sdp->md.journal[i], i,
-						     preen, force_check,
-						     &clean);
+			error = recover_journal(sdp->md.journal[i], i, cx, &clean);
 			if (!clean)
 				dirty_journals++;
-			if (!gave_msg && dirty_journals == 1 && !opts.no &&
-			    preen_is_safe(sdp, preen, force_check)) {
+			if (!gave_msg && dirty_journals == 1 && !cx->opts->no &&
+			    preen_is_safe(sdp, cx->opts)) {
 				gave_msg = 1;
 				log_notice( _("Recovering journals (this may "
 					      "take a while)\n"));
@@ -754,13 +746,12 @@ int replay_journals(struct gfs2_sbd *sdp, int preen, int force_check,
  *
  * Returns: 0 on success, -1 on failure
  */
-int ji_update(struct gfs2_sbd *sdp)
+int ji_update(struct lgfs2_sbd *sdp)
 {
-	struct gfs2_inode *jip, *ip = sdp->md.jiinode;
+	struct lgfs2_inode *jip, *ip = sdp->md.jiinode;
 	char journal_name[JOURNAL_NAME_SIZE];
 	int i, error;
 	char buf[sizeof(struct gfs_jindex)];
-	struct gfs_jindex ji;
 
 	if (!ip) {
 		log_crit(_("Journal index inode not found.\n"));
@@ -772,21 +763,23 @@ int ji_update(struct gfs2_sbd *sdp)
 	   If per_node is missing or damaged, we have to trust jindex has
 	   the correct number of entries. */
 	if (sdp->gfs1)
-		sdp->md.journals = ip->i_di.di_size / sizeof(struct gfs_jindex);
+		sdp->md.journals = ip->i_size / sizeof(struct gfs_jindex);
 	else if (sdp->md.pinode) /* if per_node was read in properly */
-		sdp->md.journals = (sdp->md.pinode->i_di.di_entries - 2) / 3;
+		sdp->md.journals = (sdp->md.pinode->i_entries - 2) / 3;
 	else
-		sdp->md.journals = ip->i_di.di_entries - 2;
+		sdp->md.journals = ip->i_entries - 2;
 
 	if (!(sdp->md.journal = calloc(sdp->md.journals,
-				       sizeof(struct gfs2_inode *)))) {
+				       sizeof(struct lgfs2_inode *)))) {
 		log_err(_("Unable to allocate journal index\n"));
 		return -1;
 	}
 	memset(journal_name, 0, sizeof(*journal_name));
 	for (i = 0; i < sdp->md.journals; i++) {
 		if (sdp->gfs1) {
-			error = gfs2_readi(ip,
+			struct gfs_jindex *ji;
+
+			error = lgfs2_readi(ip,
 					   buf, i * sizeof(struct gfs_jindex),
 					   sizeof(struct gfs_jindex));
 			if (!error)
@@ -796,16 +789,15 @@ int ji_update(struct gfs2_sbd *sdp)
 					" journal index file.\n"));
 				return -1;
 			}
-			gfs_jindex_in(&ji, buf);
-			sdp->md.journal[i] = lgfs2_inode_read(sdp, ji.ji_addr);
+			ji = (struct gfs_jindex *)buf;
+			sdp->md.journal[i] = lgfs2_inode_read(sdp, be64_to_cpu(ji->ji_addr));
 			if (sdp->md.journal[i] == NULL)
 				return -1;
 		} else {
 			/* FIXME check snprintf return code */
-			snprintf(journal_name, JOURNAL_NAME_SIZE,
-				 "journal%u", i);
-			gfs2_lookupi(sdp->md.jiinode, journal_name,
-				     strlen(journal_name), &jip);
+			int len;
+			len = snprintf(journal_name, JOURNAL_NAME_SIZE, "journal%u", i);
+			jip = lgfs2_lookupi(sdp->md.jiinode, journal_name, len);
 			sdp->md.journal[i] = jip;
 		}
 	}
@@ -826,32 +818,30 @@ static void bad_journalname(const char *filename, int len)
  * This function makes sure the directory entries of the jindex are valid.
  * If they're not '.' or '..' they better have the form journalXXX.
  */
-static int check_jindex_dent(struct gfs2_inode *ip, struct gfs2_dirent *dent,
+static int check_jindex_dent(struct fsck_cx *cx, struct lgfs2_inode *ip, struct gfs2_dirent *dent,
 			     struct gfs2_dirent *prev_de,
-			     struct gfs2_buffer_head *bh, char *filename,
+			     struct lgfs2_buffer_head *bh, char *filename,
 			     uint32_t *count, int *lindex, void *priv)
 {
-	struct gfs2_dirent dentry, *de;
+	struct lgfs2_dirent d;
 	int i;
 
-	memset(&dentry, 0, sizeof(struct gfs2_dirent));
-	gfs2_dirent_in(&dentry, (char *)dent);
-	de = &dentry;
+	lgfs2_dirent_in(&d, dent);
 
-	if (de->de_name_len == 1 && filename[0] == '.')
+	if (d.dr_name_len == 1 && filename[0] == '.')
 		goto dirent_good;
-	if (de->de_name_len == 2 && filename[0] == '.' && filename[1] == '.')
+	if (d.dr_name_len == 2 && filename[0] == '.' && filename[1] == '.')
 		goto dirent_good;
 
-	if ((de->de_name_len >= 11) || /* "journal9999" */
-	    (de->de_name_len <= 7) ||
+	if ((d.dr_name_len >= 11) || /* "journal9999" */
+	    (d.dr_name_len <= 7) ||
 	    (strncmp(filename, "journal", 7))) {
-		bad_journalname(filename, de->de_name_len);
+		bad_journalname(filename, d.dr_name_len);
 		return -1;
 	}
-	for (i = 7; i < de->de_name_len; i++) {
+	for (i = 7; i < d.dr_name_len; i++) {
 		if (filename[i] < '0' || filename[i] > '9') {
-			bad_journalname(filename, de->de_name_len);
+			bad_journalname(filename, d.dr_name_len);
 			return -2;
 		}
 	}
@@ -863,27 +853,48 @@ dirent_good:
 	return 0;
 }
 
-struct metawalk_fxns jindex_check_fxns = {
+static struct metawalk_fxns jindex_check_fxns = {
 	.private = NULL,
 	.check_dentry = check_jindex_dent,
 };
 
-/**
- * init_jindex - read in the rindex file
- */
-int init_jindex(struct gfs2_sbd *sdp, int allow_ji_rebuild)
+int build_jindex(struct lgfs2_sbd *sdp)
 {
-	/*******************************************************************
-	 ******************  Fill in journal information  ******************
-	 *******************************************************************/
+	struct lgfs2_inode *jindex;
+
+	jindex = lgfs2_createi(sdp->master_dir, "jindex", S_IFDIR | 0700,
+			 GFS2_DIF_SYSTEM);
+	if (jindex == NULL) {
+		return errno;
+	}
+	sdp->md.journal = calloc(sdp->md.journals, sizeof(struct lgfs2_inode *));
+	if (sdp->md.journal == NULL) {
+		lgfs2_inode_put(&jindex);
+		return ENOSPC;
+	}
+	for (unsigned j = 0; j < sdp->md.journals; j++) {
+		int ret = lgfs2_build_journal(sdp, j, jindex);
+		if (ret)
+			return ret;
+		lgfs2_inode_put(&sdp->md.journal[j]);
+	}
+	free(sdp->md.journal);
+	lgfs2_inode_put(&jindex);
+	return 0;
+}
+
+int init_jindex(struct fsck_cx *cx, int allow_ji_rebuild)
+{
+	struct lgfs2_sbd *sdp = cx->sdp;
 
 	log_debug(_("Validating the journal index.\n"));
 	/* rgrepair requires the journals be read in in order to distinguish
 	   "real" rgrps from rgrps that are just copies left in journals. */
 	if (sdp->gfs1)
-		sdp->md.jiinode = lgfs2_inode_read(sdp, sbd1->sb_jindex_di.no_addr);
+		sdp->md.jiinode = lgfs2_inode_read(sdp, sdp->sd_jindex_di.in_addr);
 	else
-		gfs2_lookupi(sdp->master_dir, "jindex", 6, &sdp->md.jiinode);
+		/* coverity[identity_transfer:SUPPRESS] */
+		sdp->md.jiinode = lgfs2_lookupi(sdp->master_dir, "jindex", 6);
 
 	if (!sdp->md.jiinode) {
 		int err;
@@ -893,7 +904,7 @@ int init_jindex(struct gfs2_sbd *sdp, int allow_ji_rebuild)
 				   "corrupt.\n"));
 			return -1;
 		}
-		if (!query( _("The gfs2 system jindex inode is missing. "
+		if (!query(cx, _("The gfs2 system jindex inode is missing. "
 			      "Okay to rebuild it? (y/n) "))) {
 			log_crit(_("Error: cannot proceed without a valid "
 				   "jindex file.\n"));
@@ -905,7 +916,8 @@ int init_jindex(struct gfs2_sbd *sdp, int allow_ji_rebuild)
 			log_crit(_("Error %d rebuilding jindex\n"), err);
 			return err;
 		}
-		gfs2_lookupi(sdp->master_dir, "jindex", 6, &sdp->md.jiinode);
+		/* coverity[pass_freed_arg:SUPPRESS] */
+		sdp->md.jiinode = lgfs2_lookupi(sdp->master_dir, "jindex", 6);
 	}
 
 	/* check for irrelevant entries in jindex. Can't use check_dir because
@@ -914,22 +926,22 @@ int init_jindex(struct gfs2_sbd *sdp, int allow_ji_rebuild)
 		int error;
 
 		log_debug(_("Checking the integrity of the journal index.\n"));
-		if (sdp->md.jiinode->i_di.di_flags & GFS2_DIF_EXHASH)
-			error = check_leaf_blks(sdp->md.jiinode,
+		if (sdp->md.jiinode->i_flags & GFS2_DIF_EXHASH)
+			error = check_leaf_blks(cx, sdp->md.jiinode,
 						&jindex_check_fxns);
 		else
-			error = check_linear_dir(sdp->md.jiinode,
+			error = check_linear_dir(cx, sdp->md.jiinode,
 						 sdp->md.jiinode->i_bh,
 						 &jindex_check_fxns);
 		if (error) {
 			log_err(_("The system journal index is damaged.\n"));
-			if (!query( _("Okay to rebuild it? (y/n) "))) {
+			if (!query(cx, _("Okay to rebuild it? (y/n) "))) {
 				log_crit(_("Error: cannot proceed without a "
 					   "valid jindex file.\n"));
 				return -1;
 			}
-			inode_put(&sdp->md.jiinode);
-			gfs2_dirent_del(sdp->master_dir, "jindex", 6);
+			lgfs2_inode_put(&sdp->md.jiinode);
+			lgfs2_dirent_del(sdp->master_dir, "jindex", 6);
 			log_err(_("Corrupt journal index was removed.\n"));
 			error = build_jindex(sdp);
 			if (error) {
@@ -937,8 +949,8 @@ int init_jindex(struct gfs2_sbd *sdp, int allow_ji_rebuild)
 					  "index: Cannot continue.\n"));
 				return error;
 			}
-			gfs2_lookupi(sdp->master_dir, "jindex", 6,
-				     &sdp->md.jiinode);
+			/* coverity[pass_freed_arg:SUPPRESS] */
+			sdp->md.jiinode = lgfs2_lookupi(sdp->master_dir, "jindex", 6);
 		}
 	}
 
