@@ -1,12 +1,9 @@
 #include <stdint.h>
 #include <string.h>
+#include <uuid.h>
 #include "libgfs2.h"
 #include "clusterautoconfig.h"
 
-#ifdef GFS2_HAS_UUID
-#include <uuid.h>
-#endif
-
 #define ARRAY_SIZE(x) (sizeof(x)/sizeof(x[0]))
 #define SYM(x) { x, #x, 0 },
 #define PREFIX_SYM(p,x) { p##x, #p #x, sizeof #p - 1 },
@@ -72,7 +69,6 @@ const unsigned lgfs2_di_flag_size = ARRAY_SIZE(lgfs2_di_flags);
 
 const struct lgfs2_symbolic lgfs2_lh_flags[] = {
 PREFIX_SYM(GFS2_LOG_HEAD_, UNMOUNT)
-#ifdef GFS2_HAS_LH_V2
 PREFIX_SYM(GFS2_LOG_HEAD_FLUSH_, NORMAL)
 PREFIX_SYM(GFS2_LOG_HEAD_FLUSH_, SYNC)
 PREFIX_SYM(GFS2_LOG_HEAD_FLUSH_, SHUTDOWN)
@@ -100,7 +96,6 @@ PREFIX_SYM(GFS2_LFC_, EVICT_INODE)
 PREFIX_SYM(GFS2_LFC_, TRANS_END)
 PREFIX_SYM(GFS2_LFC_, LOGD_JFLUSH_REQD)
 PREFIX_SYM(GFS2_LFC_, LOGD_AIL_FLUSH_REQD)
-#endif
 };
 
 const unsigned int lgfs2_lh_flag_size = ARRAY_SIZE(lgfs2_lh_flags);
@@ -183,9 +178,7 @@ F(sb_lockproto, .flags = LGFS2_MFF_STRING)
 F(sb_locktable, .flags = LGFS2_MFF_STRING)
 INR(__pad3, .points_to = (1 << LGFS2_MT_GFS2_DINODE))
 INR(__pad4, .points_to = (1 << LGFS2_MT_GFS2_DINODE))
-#ifdef GFS2_HAS_UUID
 F(sb_uuid, .flags = LGFS2_MFF_UUID)
-#endif
 };
 
 #undef STRUCT
@@ -230,18 +223,12 @@ MH(rg_header)
 F(rg_flags)
 F(rg_free, .flags = LGFS2_MFF_FSBLOCKS)
 F(rg_dinodes, .flags = LGFS2_MFF_FSBLOCKS)
-#ifdef GFS2_HAS_RG_SKIP
 FP(rg_skip, .points_to = (1 << LGFS2_MT_GFS2_RGRP))
-#else
-RF(__pad)
-#endif
 F(rg_igeneration)
-#ifdef GFS2_HAS_RG_RI_FIELDS
 FP(rg_data0, .points_to = ANY_GFS2_BLOCK|(1 << LGFS2_MT_FREE))
 F(rg_data, .flags = LGFS2_MFF_FSBLOCKS)
 F(rg_bitbytes, .flags = LGFS2_MFF_BYTES)
 F(rg_crc, .flags = LGFS2_MFF_CHECK)
-#endif
 RF(rg_reserved)
 };
 
@@ -392,7 +379,6 @@ F(lh_flags, .flags = LGFS2_MFF_MASK, .symtab = lgfs2_lh_flags, .nsyms = ARRAY_SI
 F(lh_tail)
 F(lh_blkno)
 F(lh_hash, .flags = LGFS2_MFF_CHECK)
-#ifdef GFS2_HAS_LH_V2
 F(lh_crc, .flags = LGFS2_MFF_CHECK)
 F(lh_nsec, .flags = LGFS2_MFF_NSECS)
 F(lh_sec, .flags = LGFS2_MFF_SECS)
@@ -403,7 +389,6 @@ FP(lh_quota_addr, .points_to = (1 << LGFS2_MT_GFS2_DINODE))
 F(lh_local_total, .flags = LGFS2_MFF_FSBLOCKS)
 F(lh_local_free, .flags = LGFS2_MFF_FSBLOCKS)
 F(lh_local_dinodes, .flags = LGFS2_MFF_FSBLOCKS)
-#endif
 };
 
 #undef STRUCT
@@ -486,17 +471,9 @@ F(de_hash, .flags = LGFS2_MFF_CHECK)
 F(de_rec_len, .flags = LGFS2_MFF_BYTES)
 F(de_name_len, .flags = LGFS2_MFF_BYTES)
 F(de_type)
-#ifdef GFS2_HAS_DE_RAHEAD
 F(de_rahead)
-#ifdef GFS2_HAS_DE_COOKIE
 F(de_cookie)
 RF(pad3)
-#else
-RF(pad2)
-#endif /* GFS2_HAS_DE_COOKIE */
-#else
-RF(__pad)
-#endif /* GFS2_HAS_DE_RAHEAD */
 };
 
 #undef STRUCT
@@ -843,89 +820,6 @@ const struct lgfs2_metafield *lgfs2_find_mfield_name(const char *name, const str
 	return NULL;
 }
 
-static int check_metadata_sizes(void)
-{
-	unsigned offset;
-	int i, j;
-	int ret = 0;
-
-	for (i = 0; i < lgfs2_metadata_size; i++) {
-		const struct lgfs2_metadata *m = &lgfs2_metadata[i];
-		offset = 0;
-		for (j = 0; j < m->nfields; j++) {
-			const struct lgfs2_metafield *f = &m->fields[j];
-			if (f->offset != offset) {
-				fprintf(stderr, "%s: %s: offset is %u, expected %u\n", m->name, f->name, f->offset, offset);
-				ret = -1;
-			}
-			offset += f->length;
-		}
-		if (offset != m->size) {
-			fprintf(stderr, "%s: size mismatch between struct %u and fields %u\n", m->name, m->size, offset);
-			ret = -1;
-		}
-	}
-
-	return ret;
-}
-
-static int check_symtab(void)
-{
-	int i, j;
-	int ret = 0;
-
-	for (i = 0; i < lgfs2_metadata_size; i++) {
-		const struct lgfs2_metadata *m = &lgfs2_metadata[i];
-		for (j = 0; j < m->nfields; j++) {
-			const struct lgfs2_metafield *f = &m->fields[j];
-			if (f->flags & (LGFS2_MFF_MASK|LGFS2_MFF_ENUM)) {
-				if (f->symtab == NULL) {
-					fprintf(stderr, "%s: Missing symtab for %s\n", m->name, f->name);
-					ret = -1;
-				}
-			}
-			if (f->symtab) {
-				if (!(f->flags & (LGFS2_MFF_MASK|LGFS2_MFF_ENUM))) {
-					fprintf(stderr, "%s: Symtab for non-enum and non-mask field %s\n", m->name, f->name);
-					ret = -1;
-				}
-			}
-		}
-	}
-
-	return ret;
-}
-
-static int check_ptrs(void)
-{
-	int i, j;
-	int ret = 0;
-
-	for (i = 0; i < lgfs2_metadata_size; i++) {
-		const struct lgfs2_metadata *m = &lgfs2_metadata[i];
-		for (j = 0; j < m->nfields; j++) {
-			const struct lgfs2_metafield *f = &m->fields[j];
-			if ((f->flags & LGFS2_MFF_POINTER) && !f->points_to) {
-				fprintf(stderr, "%s: Pointer entry %s has no destination\n", m->name, f->name);
-				ret = -1;
-			}
-		}
-	}
-
-	return ret;
-}
-
-int lgfs2_selfcheck(void)
-{
-	int ret = 0;
-
-	ret |= check_metadata_sizes();
-	ret |= check_symtab();
-	ret |= check_ptrs();
-
-	return ret;
-}
-
 const struct lgfs2_metadata *lgfs2_find_mtype(uint32_t mh_type, const unsigned versions)
 {
 	const struct lgfs2_metadata *m = lgfs2_metadata;
@@ -963,14 +857,12 @@ int lgfs2_field_str(char *str, const size_t size, const char *blk, const struct
 		return 1;
 
 	if (field->flags & LGFS2_MFF_UUID) {
-#ifdef GFS2_HAS_UUID
 		char readable_uuid[36+1];
 		uuid_t uuid;
 
 		memcpy(uuid, fieldp, sizeof(uuid_t));
 		uuid_unparse(uuid, readable_uuid);
 		snprintf(str, size, "%s", readable_uuid);
-#endif
 	} else if (field->flags & LGFS2_MFF_STRING) {
 		snprintf(str, size, "%s", fieldp);
 	} else {
@@ -981,13 +873,13 @@ int lgfs2_field_str(char *str, const size_t size, const char *blk, const struct
 			value = *(uint8_t *)fieldp;
 			break;
 		case sizeof(uint16_t):
-			value = be16_to_cpu(*(uint16_t *)fieldp);
+			value = be16_to_cpu(*(__be16 *)fieldp);
 			break;
 		case sizeof(uint32_t):
-			value = be32_to_cpu(*(uint32_t *)fieldp);
+			value = be32_to_cpu(*(__be32 *)fieldp);
 			break;
 		case sizeof(uint64_t):
-			value = be64_to_cpu(*(uint64_t *)fieldp);
+			value = be64_to_cpu(*(__be64 *)fieldp);
 			break;
 		default:
 			*str = 0;
@@ -1056,13 +948,13 @@ int lgfs2_field_assign(char *blk, const struct lgfs2_metafield *field, const voi
 		*fieldp = (uint8_t)num;
 		return 0;
 	case sizeof(uint16_t):
-		*(uint16_t *)fieldp = cpu_to_be16((uint16_t)num);
+		*(__be16 *)fieldp = cpu_to_be16((uint16_t)num);
 		return 0;
 	case sizeof(uint32_t):
-		*(uint32_t *)fieldp = cpu_to_be32((uint32_t)num);
+		*(__be32 *)fieldp = cpu_to_be32((uint32_t)num);
 		return 0;
 	case sizeof(uint64_t):
-		*(uint64_t *)fieldp = cpu_to_be64((uint64_t)num);
+		*(__be64 *)fieldp = cpu_to_be64((uint64_t)num);
 		return 0;
 	default:
 		/* Will never happen */
