@@ -14,28 +14,44 @@
 #include <string.h>
 #include "libgfs2.h"
 
-void gfs2_replay_incr_blk(struct gfs2_inode *ip, unsigned int *blk)
+void lgfs2_replay_incr_blk(struct lgfs2_inode *ip, unsigned int *blk)
 {
-	uint32_t jd_blocks = ip->i_di.di_size / ip->i_sbd->sd_sb.sb_bsize;
+	uint32_t jd_blocks = ip->i_size / ip->i_sbd->sd_bsize;
 
         if (++*blk == jd_blocks)
                 *blk = 0;
 }
 
-int gfs2_replay_read_block(struct gfs2_inode *ip, unsigned int blk,
-			   struct gfs2_buffer_head **bh)
+int lgfs2_replay_read_block(struct lgfs2_inode *ip, unsigned int blk,
+			   struct lgfs2_buffer_head **bh)
 {
 	int new = 0;
 	uint64_t dblock;
 
-	block_map(ip, blk, &new, &dblock, NULL, 0);
+	if (lgfs2_block_map(ip, blk, &new, &dblock, NULL, 0))
+		return -EINVAL;
 	if (!dblock)
 		return -EIO;
 
-	*bh = bread(ip->i_sbd, dblock);
+	*bh = lgfs2_bread(ip->i_sbd, dblock);
 	return 0;
 }
 
+static void log_header_in(struct lgfs2_log_header *lh, char *buf)
+{
+	struct gfs2_log_header *lhd = (struct gfs2_log_header *)buf;
+
+	lh->lh_sequence = be64_to_cpu(lhd->lh_sequence);
+	lh->lh_flags = be32_to_cpu(lhd->lh_flags);
+	lh->lh_tail = be32_to_cpu(lhd->lh_tail);
+	lh->lh_blkno = be32_to_cpu(lhd->lh_blkno);
+	lh->lh_hash = be32_to_cpu(lhd->lh_hash);
+	lh->lh_crc = be32_to_cpu(lhd->lh_crc);
+	lh->lh_local_total = be64_to_cpu(lhd->lh_local_total);
+	lh->lh_local_free = be64_to_cpu(lhd->lh_local_free);
+	lh->lh_local_dinodes = be64_to_cpu(lhd->lh_local_dinodes);
+}
+
 /**
  * get_log_header - read the log header for a given segment
  * @ip: the journal incore inode
@@ -50,17 +66,19 @@ int gfs2_replay_read_block(struct gfs2_inode *ip, unsigned int blk,
  *          errno on error
  */
 
-int get_log_header(struct gfs2_inode *ip, unsigned int blk,
-		   struct gfs2_log_header *head)
+int lgfs2_get_log_header(struct lgfs2_inode *ip, unsigned int blk,
+                         struct lgfs2_log_header *head)
 {
-	struct gfs2_buffer_head *bh;
-	struct gfs2_log_header lh, *tmp;
-	uint32_t hash, saved_hash;
+	struct lgfs2_buffer_head *bh;
+	struct lgfs2_log_header lh;
+	struct gfs2_log_header *tmp;
+	__be32 saved_hash;
+	uint32_t hash;
 	uint32_t lh_crc = 0;
 	uint32_t crc;
 	int error;
 
-	error = gfs2_replay_read_block(ip, blk, &bh);
+	error = lgfs2_replay_read_block(ip, blk, &bh);
 	if (error)
 		return error;
 
@@ -69,12 +87,10 @@ int get_log_header(struct gfs2_inode *ip, unsigned int blk,
 	tmp->lh_hash = 0;
 	hash = lgfs2_log_header_hash(bh->b_data);
 	tmp->lh_hash = saved_hash;
-	crc = lgfs2_log_header_crc(bh->b_data, ip->i_sbd->bsize);
-	gfs2_log_header_in(&lh, bh->b_data);
-	brelse(bh);
-#ifdef GFS2_HAS_LH_V2
+	crc = lgfs2_log_header_crc(bh->b_data, ip->i_sbd->sd_bsize);
+	log_header_in(&lh, bh->b_data);
+	lgfs2_brelse(bh);
 	lh_crc = lh.lh_crc;
-#endif
 	if (error || lh.lh_blkno != blk || lh.lh_hash != hash)
 		return 1;
 	/* Don't check the crc if it's zero, as it is in pre-v2 log headers */
@@ -98,14 +114,14 @@ int get_log_header(struct gfs2_inode *ip, unsigned int blk,
  *
  * Returns: errno
  */
-static int find_good_lh(struct gfs2_inode *ip, unsigned int *blk, struct gfs2_log_header *head)
+static int find_good_lh(struct lgfs2_inode *ip, unsigned int *blk, struct lgfs2_log_header *head)
 {
 	unsigned int orig_blk = *blk;
 	int error;
-	uint32_t jd_blocks = ip->i_di.di_size / ip->i_sbd->sd_sb.sb_bsize;
+	uint32_t jd_blocks = ip->i_size / ip->i_sbd->sd_bsize;
 
 	for (;;) {
-		error = get_log_header(ip, *blk, head);
+		error = lgfs2_get_log_header(ip, *blk, head);
 		if (error <= 0)
 			return error;
 
@@ -128,18 +144,18 @@ static int find_good_lh(struct gfs2_inode *ip, unsigned int *blk, struct gfs2_lo
  * Returns: errno
  */
 
-static int jhead_scan(struct gfs2_inode *ip, struct gfs2_log_header *head)
+static int jhead_scan(struct lgfs2_inode *ip, struct lgfs2_log_header *head)
 {
 	unsigned int blk = head->lh_blkno;
-	uint32_t jd_blocks = ip->i_di.di_size / ip->i_sbd->sd_sb.sb_bsize;
-	struct gfs2_log_header lh;
+	uint32_t jd_blocks = ip->i_size / ip->i_sbd->sd_bsize;
+	struct lgfs2_log_header lh;
 	int error;
 
 	for (;;) {
 		if (++blk == jd_blocks)
 			blk = 0;
 
-		error = get_log_header(ip, blk, &lh);
+		error = lgfs2_get_log_header(ip, blk, &lh);
 		if (error < 0)
 			return error;
 		if (error == 1)
@@ -167,11 +183,11 @@ static int jhead_scan(struct gfs2_inode *ip, struct gfs2_log_header *head)
  * Returns: errno
  */
 
-int gfs2_find_jhead(struct gfs2_inode *ip, struct gfs2_log_header *head)
+int lgfs2_find_jhead(struct lgfs2_inode *ip, struct lgfs2_log_header *head)
 {
-	struct gfs2_log_header lh_1, lh_m;
+	struct lgfs2_log_header lh_1, lh_m;
 	uint32_t blk_1, blk_2, blk_m;
-	uint32_t jd_blocks = ip->i_di.di_size / ip->i_sbd->sd_sb.sb_bsize;
+	uint32_t jd_blocks = ip->i_size / ip->i_sbd->sd_bsize;
 	int error;
 
 	blk_1 = 0;
@@ -215,23 +231,24 @@ int gfs2_find_jhead(struct gfs2_inode *ip, struct gfs2_log_header *head)
  * Returns: errno
  */
 
-int clean_journal(struct gfs2_inode *ip, struct gfs2_log_header *head)
+int lgfs2_clean_journal(struct lgfs2_inode *ip, struct lgfs2_log_header *head)
 {
 	unsigned int lblock;
 	struct gfs2_log_header *lh;
 	uint32_t hash;
-	struct gfs2_buffer_head *bh;
+	struct lgfs2_buffer_head *bh;
 	int new = 0;
 	uint64_t dblock;
 
 	lblock = head->lh_blkno;
-	gfs2_replay_incr_blk(ip, &lblock);
-	block_map(ip, lblock, &new, &dblock, NULL, 0);
+	lgfs2_replay_incr_blk(ip, &lblock);
+	if (lgfs2_block_map(ip, lblock, &new, &dblock, NULL, 0))
+		return -EINVAL;
 	if (!dblock)
 		return -EIO;
 
-	bh = bread(ip->i_sbd, dblock);
-	memset(bh->b_data, 0, ip->i_sbd->bsize);
+	bh = lgfs2_bread(ip->i_sbd, dblock);
+	memset(bh->b_data, 0, ip->i_sbd->sd_bsize);
 
 	lh = (struct gfs2_log_header *)bh->b_data;
 	memset(lh, 0, sizeof(struct gfs2_log_header));
@@ -243,8 +260,8 @@ int clean_journal(struct gfs2_inode *ip, struct gfs2_log_header *head)
 	lh->lh_blkno = cpu_to_be32(lblock);
 	hash = lgfs2_log_header_hash(bh->b_data);
 	lh->lh_hash = cpu_to_be32(hash);
-	bmodified(bh);
-	brelse(bh);
+	lgfs2_bmodified(bh);
+	lgfs2_brelse(bh);
 
 	return 0;
 }
