@@ -16,28 +16,28 @@ static void compute_bitmaps(lgfs2_rgrp_t rg, const unsigned bsize)
 {
 	int x;
 
-	rg->bits[0].bi_offset = sizeof(struct gfs2_rgrp);
-	rg->bits[0].bi_start = 0;
-	rg->bits[0].bi_len = bsize - sizeof(struct gfs2_rgrp);
+	rg->rt_bits[0].bi_offset = sizeof(struct gfs2_rgrp);
+	rg->rt_bits[0].bi_start = 0;
+	rg->rt_bits[0].bi_len = bsize - sizeof(struct gfs2_rgrp);
 
-	for (x = 1; x < rg->ri.ri_length; x++) {
-		rg->bits[x].bi_offset = sizeof(struct gfs2_meta_header);
-		rg->bits[x].bi_start = rg->bits[x - 1].bi_start + rg->bits[x - 1].bi_len;
-		rg->bits[x].bi_len = bsize - sizeof(struct gfs2_meta_header);
+	for (x = 1; x < rg->rt_length; x++) {
+		rg->rt_bits[x].bi_offset = sizeof(struct gfs2_meta_header);
+		rg->rt_bits[x].bi_start = rg->rt_bits[x - 1].bi_start + rg->rt_bits[x - 1].bi_len;
+		rg->rt_bits[x].bi_len = bsize - sizeof(struct gfs2_meta_header);
 	}
 	x--;
-	rg->bits[x].bi_len = rg->ri.ri_bitbytes - rg->bits[x].bi_start;
+	rg->rt_bits[x].bi_len = rg->rt_bitbytes - rg->rt_bits[x].bi_start;
 }
 
 /**
- * gfs2_compute_bitstructs - Compute the bitmap sizes
+ * lgfs2_compute_bitstructs - Compute the bitmap sizes
  * bsize: Block size
  * rgd: The resource group descriptor
  * Returns: 0 on success, -1 on error
  */
-int gfs2_compute_bitstructs(const uint32_t bsize, struct rgrp_tree *rgd)
+int lgfs2_compute_bitstructs(const uint32_t bsize, struct lgfs2_rgrp_tree *rgd)
 {
-	uint32_t length = rgd->ri.ri_length;
+	uint32_t length = rgd->rt_length;
 	uint32_t bytes_left;
 	int ownbits = 0;
 
@@ -51,29 +51,29 @@ int gfs2_compute_bitstructs(const uint32_t bsize, struct rgrp_tree *rgd)
 	if (length > 2149 || length == 0)
 		return -1;
 
-	if(rgd->bits == NULL) {
-		rgd->bits = calloc(length, sizeof(struct gfs2_bitmap));
-		if(rgd->bits == NULL)
+	if(rgd->rt_bits == NULL) {
+		rgd->rt_bits = calloc(length, sizeof(struct lgfs2_bitmap));
+		if(rgd->rt_bits == NULL)
 			return -1;
 		ownbits = 1;
 	}
 
 	compute_bitmaps(rgd, bsize);
-	bytes_left = rgd->ri.ri_bitbytes - (rgd->bits[rgd->ri.ri_length - 1].bi_start +
-	                                    rgd->bits[rgd->ri.ri_length - 1].bi_len);
+	bytes_left = rgd->rt_bitbytes - (rgd->rt_bits[rgd->rt_length - 1].bi_start +
+	                                    rgd->rt_bits[rgd->rt_length - 1].bi_len);
 	errno = EINVAL;
 	if(bytes_left)
 		goto errbits;
 
-	if((rgd->bits[length - 1].bi_start +
-	    rgd->bits[length - 1].bi_len) * GFS2_NBBY != rgd->ri.ri_data)
+	if((rgd->rt_bits[length - 1].bi_start +
+	    rgd->rt_bits[length - 1].bi_len) * GFS2_NBBY != rgd->rt_data)
 		goto errbits;
 
 	return 0;
 errbits:
 	if (ownbits) {
-		free(rgd->bits);
-		rgd->bits = NULL;
+		free(rgd->rt_bits);
+		rgd->rt_bits = NULL;
 	}
 	return -1;
 }
@@ -86,14 +86,14 @@ errbits:
  *
  * Returns: Ths resource group, or NULL if not found
  */
-struct rgrp_tree *gfs2_blk2rgrpd(struct gfs2_sbd *sdp, uint64_t blk)
+struct lgfs2_rgrp_tree *lgfs2_blk2rgrpd(struct lgfs2_sbd *sdp, uint64_t blk)
 {
-	struct rgrp_tree *rgd = (struct rgrp_tree *)sdp->rgtree.osi_node;
+	struct lgfs2_rgrp_tree *rgd = (struct lgfs2_rgrp_tree *)sdp->rgtree.osi_node;
 	while (rgd) {
-		if (blk < rgd->ri.ri_addr)
-			rgd = (struct rgrp_tree *)rgd->node.osi_left;
-		else if (blk >= rgd->ri.ri_data0 + rgd->ri.ri_data)
-			rgd = (struct rgrp_tree *)rgd->node.osi_right;
+		if (blk < rgd->rt_addr)
+			rgd = (struct lgfs2_rgrp_tree *) rgd->rt_node.osi_left;
+		else if (blk >= rgd->rt_data0 + rgd->rt_data)
+			rgd = (struct lgfs2_rgrp_tree *) rgd->rt_node.osi_right;
 		else
 			return rgd;
 	}
@@ -108,15 +108,15 @@ struct rgrp_tree *gfs2_blk2rgrpd(struct gfs2_sbd *sdp, uint64_t blk)
  */
 int lgfs2_rgrp_bitbuf_alloc(lgfs2_rgrp_t rg)
 {
-	struct gfs2_sbd *sdp = rg->rgrps->sdp;
-	size_t len = rg->ri.ri_length * sdp->bsize;
-	unsigned long io_align = sdp->bsize;
+	struct lgfs2_sbd *sdp = rg->rt_rgrps->rgs_sdp;
+	size_t len = rg->rt_length * sdp->sd_bsize;
+	unsigned long io_align = sdp->sd_bsize;
 	unsigned i;
 	char *bufs;
 
-	if (rg->rgrps->align > 0) {
-		len = ROUND_UP(len, rg->rgrps->align * sdp->bsize);
-		io_align = rg->rgrps->align_off * sdp->bsize;
+	if (rg->rt_rgrps->rgs_align > 0) {
+		len = ROUND_UP(len, rg->rt_rgrps->rgs_align * sdp->sd_bsize);
+		io_align = rg->rt_rgrps->rgs_align_off * sdp->sd_bsize;
 	}
 	if (posix_memalign((void **)&bufs, io_align, len) != 0) {
 		errno = ENOMEM;
@@ -124,10 +124,11 @@ int lgfs2_rgrp_bitbuf_alloc(lgfs2_rgrp_t rg)
 	}
 	memset(bufs, 0, len);
 
-	for (i = 0; i < rg->ri.ri_length; i++) {
-		rg->bits[i].bi_data = bufs + (i * sdp->bsize);
-		rg->bits[i].bi_modified = 0;
+	for (i = 0; i < rg->rt_length; i++) {
+		rg->rt_bits[i].bi_data = bufs + (i * sdp->sd_bsize);
+		rg->rt_bits[i].bi_modified = 0;
 	}
+	/* coverity[leaked_storage:SUPPRESS] */
 	return 0;
 }
 
@@ -142,10 +143,10 @@ void lgfs2_rgrp_bitbuf_free(lgfs2_rgrp_t rg)
 {
 	unsigned i;
 
-	free(rg->bits[0].bi_data);
-	for (i = 0; i < rg->ri.ri_length; i++) {
-		rg->bits[i].bi_data = NULL;
-		rg->bits[i].bi_modified = 0;
+	free(rg->rt_bits[0].bi_data);
+	for (i = 0; i < rg->rt_length; i++) {
+		rg->rt_bits[i].bi_data = NULL;
+		rg->rt_bits[i].bi_modified = 0;
 	}
 }
 
@@ -156,18 +157,16 @@ void lgfs2_rgrp_bitbuf_free(lgfs2_rgrp_t rg)
 int lgfs2_rgrp_crc_check(char *buf)
 {
 	int ret = 0;
-#ifdef GFS2_HAS_RG_RI_FIELDS
 	struct gfs2_rgrp *rg = (struct gfs2_rgrp *)buf;
-	uint32_t crc = rg->rg_crc;
+	__be32 crc = rg->rg_crc;
 
 	if (crc == 0)
 		return 0;
 
 	rg->rg_crc = 0;
-	if (be32_to_cpu(crc) != gfs2_disk_hash(buf, sizeof(struct gfs2_rgrp)))
+	if (be32_to_cpu(crc) != lgfs2_disk_hash(buf, sizeof(struct gfs2_rgrp)))
 		ret = 1;
 	rg->rg_crc = crc;
-#endif
 	return ret;
 }
 
@@ -176,28 +175,26 @@ int lgfs2_rgrp_crc_check(char *buf)
  */
 void lgfs2_rgrp_crc_set(char *buf)
 {
-#ifdef GFS2_HAS_RG_RI_FIELDS
 	struct gfs2_rgrp *rg = (struct gfs2_rgrp *)buf;
 	uint32_t crc;
 
 	rg->rg_crc = 0;
-	crc = gfs2_disk_hash(buf, sizeof(struct gfs2_rgrp));
+	crc = lgfs2_disk_hash(buf, sizeof(struct gfs2_rgrp));
 	rg->rg_crc = cpu_to_be32(crc);
-#endif
 }
 
 /**
- * gfs2_rgrp_read - read in the resource group information from disk.
+ * lgfs2_rgrp_read - read in the resource group information from disk.
  * @rgd - resource group structure
  * returns: 0 if no error, otherwise the block number that failed
  */
-uint64_t gfs2_rgrp_read(struct gfs2_sbd *sdp, struct rgrp_tree *rgd)
+uint64_t lgfs2_rgrp_read(struct lgfs2_sbd *sdp, struct lgfs2_rgrp_tree *rgd)
 {
-	unsigned length = rgd->ri.ri_length * sdp->bsize;
-	off_t offset = rgd->ri.ri_addr * sdp->bsize;
+	unsigned length = rgd->rt_length * sdp->sd_bsize;
+	off_t offset = rgd->rt_addr * sdp->sd_bsize;
 	char *buf;
 
-	if (length == 0 || gfs2_check_range(sdp, rgd->ri.ri_addr))
+	if (length == 0 || lgfs2_check_range(sdp, rgd->rt_addr))
 		return -1;
 
 	buf = calloc(1, length);
@@ -209,93 +206,96 @@ uint64_t gfs2_rgrp_read(struct gfs2_sbd *sdp, struct rgrp_tree *rgd)
 		return -1;
 	}
 
-	for (unsigned i = 0; i < rgd->ri.ri_length; i++) {
+	for (unsigned i = 0; i < rgd->rt_length; i++) {
 		int mtype = (i ? GFS2_METATYPE_RB : GFS2_METATYPE_RG);
 
-		rgd->bits[i].bi_data = buf + (i * sdp->bsize);
-		if (gfs2_check_meta(rgd->bits[i].bi_data, mtype)) {
+		rgd->rt_bits[i].bi_data = buf + (i * sdp->sd_bsize);
+		if (lgfs2_check_meta(rgd->rt_bits[i].bi_data, mtype)) {
 			free(buf);
-			return rgd->ri.ri_addr + i;
+			rgd->rt_bits[0].bi_data = NULL;
+			return rgd->rt_addr + i;
 		}
 	}
 	if (sdp->gfs1)
-		gfs_rgrp_in((struct gfs_rgrp *)&rgd->rg, buf);
+		lgfs2_gfs_rgrp_in(rgd, buf);
 	else {
 		if (lgfs2_rgrp_crc_check(buf)) {
 			free(buf);
-			return rgd->ri.ri_addr;
+			return rgd->rt_addr;
 		}
-		gfs2_rgrp_in(&rgd->rg, buf);
+		lgfs2_rgrp_in(rgd, buf);
 	}
+	/* coverity[leaked_storage:SUPPRESS] */
 	return 0;
 }
 
-void gfs2_rgrp_relse(struct gfs2_sbd *sdp, struct rgrp_tree *rgd)
+void lgfs2_rgrp_relse(struct lgfs2_sbd *sdp, struct lgfs2_rgrp_tree *rgd)
 {
-	if (rgd->bits == NULL)
+	if (rgd->rt_bits == NULL)
 		return;
-	for (unsigned i = 0; i < rgd->ri.ri_length; i++) {
-		off_t offset = sdp->bsize * (rgd->ri.ri_addr + i);
+	for (unsigned i = 0; i < rgd->rt_length; i++) {
+		off_t offset = sdp->sd_bsize * (rgd->rt_addr + i);
 		ssize_t ret;
 
-		if (rgd->bits[i].bi_data == NULL || !rgd->bits[i].bi_modified)
+		if (rgd->rt_bits[i].bi_data == NULL || !rgd->rt_bits[i].bi_modified)
 			continue;
 
-		ret = pwrite(sdp->device_fd, rgd->bits[i].bi_data, sdp->bsize, offset);
-		if (ret != sdp->bsize) {
+		ret = pwrite(sdp->device_fd, rgd->rt_bits[i].bi_data,
+		             sdp->sd_bsize, offset);
+		if (ret != sdp->sd_bsize) {
 			fprintf(stderr, "Failed to write modified resource group at block %"PRIu64": %s\n",
-			        (uint64_t)rgd->ri.ri_addr, strerror(errno));
+			        rgd->rt_addr, strerror(errno));
 		}
-		rgd->bits[i].bi_modified = 0;
+		rgd->rt_bits[i].bi_modified = 0;
 	}
-	free(rgd->bits[0].bi_data);
-	for (unsigned i = 0; i < rgd->ri.ri_length; i++)
-		rgd->bits[i].bi_data = NULL;
+	free(rgd->rt_bits[0].bi_data);
+	for (unsigned i = 0; i < rgd->rt_length; i++)
+		rgd->rt_bits[i].bi_data = NULL;
 }
 
-struct rgrp_tree *rgrp_insert(struct osi_root *rgtree, uint64_t rgblock)
+struct lgfs2_rgrp_tree *lgfs2_rgrp_insert(struct osi_root *rgtree, uint64_t rgblock)
 {
 	struct osi_node **newn = &rgtree->osi_node, *parent = NULL;
-	struct rgrp_tree *data;
+	struct lgfs2_rgrp_tree *data;
 
 	/* Figure out where to put new node */
 	while (*newn) {
-		struct rgrp_tree *cur = (struct rgrp_tree *)*newn;
+		struct lgfs2_rgrp_tree *cur = (struct lgfs2_rgrp_tree *)*newn;
 
 		parent = *newn;
-		if (rgblock < cur->ri.ri_addr)
+		if (rgblock < cur->rt_addr)
 			newn = &((*newn)->osi_left);
-		else if (rgblock > cur->ri.ri_addr)
+		else if (rgblock > cur->rt_addr)
 			newn = &((*newn)->osi_right);
 		else
 			return cur;
 	}
 
-	data = calloc(1, sizeof(struct rgrp_tree));
+	data = calloc(1, sizeof(struct lgfs2_rgrp_tree));
 	if (!data)
 		return NULL;
 	/* Add new node and rebalance tree. */
-	data->ri.ri_addr = rgblock;
-	osi_link_node(&data->node, parent, newn);
-	osi_insert_color(&data->node, rgtree);
+	data->rt_addr = rgblock;
+	osi_link_node(&data->rt_node, parent, newn);
+	osi_insert_color(&data->rt_node, rgtree);
 
 	return data;
 }
 
-void gfs2_rgrp_free(struct gfs2_sbd *sdp, struct osi_root *rgrp_tree)
+void lgfs2_rgrp_free(struct lgfs2_sbd *sdp, struct osi_root *rgrp_tree)
 {
-	struct rgrp_tree *rgd;
+	struct lgfs2_rgrp_tree *rgd;
 	struct osi_node *n;
 
 	if (OSI_EMPTY_ROOT(rgrp_tree))
 		return;
 	while ((n = osi_first(rgrp_tree))) {
-		rgd = (struct rgrp_tree *)n;
+		rgd = (struct lgfs2_rgrp_tree *)n;
 
-		gfs2_rgrp_relse(sdp, rgd);
-		free(rgd->bits);
-		rgd->bits = NULL;
-		osi_erase(&rgd->node, rgrp_tree);
+		lgfs2_rgrp_relse(sdp, rgd);
+		free(rgd->rt_bits);
+		rgd->rt_bits = NULL;
+		osi_erase(&rgd->rt_node, rgrp_tree);
 		free(rgd);
 	}
 }
@@ -315,7 +315,7 @@ static uint64_t align_block(const uint64_t base, const uint64_t align)
  */
 uint64_t lgfs2_rgrp_align_addr(const lgfs2_rgrps_t rgs, uint64_t addr)
 {
-	return align_block(addr, rgs->align_off);
+	return align_block(addr, rgs->rgs_align_off);
 }
 
 /**
@@ -328,7 +328,7 @@ uint64_t lgfs2_rgrp_align_addr(const lgfs2_rgrps_t rgs, uint64_t addr)
  */
 uint32_t lgfs2_rgrp_align_len(const lgfs2_rgrps_t rgs, uint32_t len)
 {
-	return align_block(len, rgs->align) + rgs->align_off;
+	return align_block(len, rgs->rgs_align) + rgs->rgs_align_off;
 }
 
 /**
@@ -347,22 +347,22 @@ uint32_t lgfs2_rgrp_align_len(const lgfs2_rgrps_t rgs, uint32_t len)
  */
 uint32_t lgfs2_rgrps_plan(const lgfs2_rgrps_t rgs, uint64_t space, uint32_t tgtsize)
 {
-	uint32_t maxlen = (GFS2_MAX_RGSIZE << 20) / rgs->sdp->bsize;
-	uint32_t minlen = (GFS2_MIN_RGSIZE << 20) / rgs->sdp->bsize;
-	struct rg_spec *spec = rgs->plan->rg_specs;
+	uint32_t maxlen = (LGFS2_MAX_RGSIZE << 20) / rgs->rgs_sdp->sd_bsize;
+	uint32_t minlen = (LGFS2_MIN_RGSIZE << 20) / rgs->rgs_sdp->sd_bsize;
+	struct rg_spec *spec = rgs->rgs_plan->rg_specs;
 
 	/* Apps should already have checked that the rg size is <=
 	   GFS2_MAX_RGSIZE but just in case alignment pushes it over we clamp
 	   it back down while calculating the initial rgrp length.  */
 	do {
 		spec[0].len = lgfs2_rgrp_align_len(rgs, tgtsize);
-		tgtsize -= (rgs->align + 1);
+		tgtsize -= (rgs->rgs_align + 1);
 	} while (spec[0].len > maxlen);
 
 	spec[0].num = space / spec[0].len;
 
-	if ((space - (spec[0].num * spec[0].len)) > rgs->align) {
-		unsigned adj = (rgs->align > 0) ? rgs->align : 1;
+	if ((space - (spec[0].num * spec[0].len)) > rgs->rgs_align) {
+		unsigned adj = (rgs->rgs_align > 0) ? rgs->rgs_align : 1;
 
 		/* Spread the adjustment required to fit a new rgrp at the end
 		   over all of the rgrps so that we don't end with a single
@@ -381,8 +381,8 @@ uint32_t lgfs2_rgrps_plan(const lgfs2_rgrps_t rgs, uint64_t space, uint32_t tgts
 		spec[1].len = spec[0].len - adj;
 		spec[1].num = 0;
 
-		while (((spec[0].len * spec[0].num) +
-		        (spec[1].len * spec[1].num)) >= space) {
+		while ((((uint64_t)spec[0].len * spec[0].num) +
+		        ((uint64_t)spec[1].len * spec[1].num)) >= space) {
 			/* Total number of rgrps stays constant now. We just
 			   need to shift some weight around */
 			spec[0].num--;
@@ -407,7 +407,7 @@ uint32_t lgfs2_rgrps_plan(const lgfs2_rgrps_t rgs, uint64_t space, uint32_t tgts
  * offset: The required stripe offset of the resource groups
  * Returns an initialised lgfs2_rgrps_t or NULL if unsuccessful with errno set
  */
-lgfs2_rgrps_t lgfs2_rgrps_init(struct gfs2_sbd *sdp, uint64_t align, uint64_t offset)
+lgfs2_rgrps_t lgfs2_rgrps_init(struct lgfs2_sbd *sdp, uint64_t align, uint64_t offset)
 {
 	lgfs2_rgrps_t rgs;
 
@@ -419,17 +419,17 @@ lgfs2_rgrps_t lgfs2_rgrps_init(struct gfs2_sbd *sdp, uint64_t align, uint64_t of
 	if (rgs == NULL)
 		return NULL;
 
-	rgs->plan = calloc(1, sizeof(struct rgs_plan) + (5 * sizeof(struct rg_spec)));
-	if (rgs->plan == NULL) {
+	rgs->rgs_plan = calloc(1, sizeof(struct rgs_plan) + (5 * sizeof(struct rg_spec)));
+	if (rgs->rgs_plan == NULL) {
 		free(rgs);
 		return NULL;
 	}
-	rgs->plan->length = 0;
-	rgs->plan->capacity = 5;
-	rgs->sdp = sdp;
-	rgs->align = align;
-	rgs->align_off = offset;
-	memset(&rgs->root, 0, sizeof(rgs->root));
+	rgs->rgs_plan->length = 0;
+	rgs->rgs_plan->capacity = 5;
+	rgs->rgs_sdp = sdp;
+	rgs->rgs_align = align;
+	rgs->rgs_align_off = offset;
+	memset(&rgs->rgs_root, 0, sizeof(rgs->rgs_root));
 
 	return rgs;
 }
@@ -444,7 +444,6 @@ lgfs2_rgrps_t lgfs2_rgrps_init(struct gfs2_sbd *sdp, uint64_t align, uint64_t of
 unsigned lgfs2_rindex_read_fd(int fd, lgfs2_rgrps_t rgs)
 {
 	unsigned count = 0;
-	char buf[sizeof(struct gfs2_rindex)];
 
 	errno = EINVAL;
 	if (fd < 0 || rgs == NULL)
@@ -453,14 +452,13 @@ unsigned lgfs2_rindex_read_fd(int fd, lgfs2_rgrps_t rgs)
 	while (1) {
 		lgfs2_rgrp_t rg;
 		struct gfs2_rindex ri;
-		ssize_t ret = read(fd, buf, sizeof(struct gfs2_rindex));
+		ssize_t ret = read(fd, &ri, sizeof(ri));
 		if (ret == 0)
 			break;
 
-		if (ret != sizeof(struct gfs2_rindex))
+		if (ret != sizeof(ri))
 			return 0;
 
-		gfs2_rindex_in(&ri, buf);
 		rg = lgfs2_rgrps_append(rgs, &ri, 0);
 		if (rg == NULL)
 			return 0;
@@ -477,10 +475,9 @@ unsigned lgfs2_rindex_read_fd(int fd, lgfs2_rgrps_t rgs)
  * Returns the new rindex entry added to the set or NULL on error with errno
  * set.
  */
-const struct gfs2_rindex *lgfs2_rindex_read_one(struct gfs2_inode *rip, lgfs2_rgrps_t rgs, unsigned i)
+lgfs2_rgrp_t lgfs2_rindex_read_one(struct lgfs2_inode *rip, lgfs2_rgrps_t rgs, unsigned i)
 {
 	uint64_t off = i * sizeof(struct gfs2_rindex);
-	char buf[sizeof(struct gfs2_rindex)];
 	struct gfs2_rindex ri;
 	lgfs2_rgrp_t rg;
 	int ret;
@@ -489,16 +486,15 @@ const struct gfs2_rindex *lgfs2_rindex_read_one(struct gfs2_inode *rip, lgfs2_rg
 	if (rip == NULL || rgs == NULL)
 		return NULL;
 
-	ret = gfs2_readi(rip, buf, off, sizeof(struct gfs2_rindex));
+	ret = lgfs2_readi(rip, &ri, off, sizeof(struct gfs2_rindex));
 	if (ret != sizeof(struct gfs2_rindex))
 		return NULL;
 
-	gfs2_rindex_in(&ri, buf);
 	rg = lgfs2_rgrps_append(rgs, &ri, 0);
 	if (rg == NULL)
 		return NULL;
 
-	return &rg->ri;
+	return rg;
 }
 
 /**
@@ -509,22 +505,48 @@ const struct gfs2_rindex *lgfs2_rindex_read_one(struct gfs2_inode *rip, lgfs2_rg
 void lgfs2_rgrps_free(lgfs2_rgrps_t *rgs)
 {
 	lgfs2_rgrp_t rg;
-	struct osi_root *tree = &(*rgs)->root;
+	struct osi_root *tree = &(*rgs)->rgs_root;
 
-	while ((rg = (struct rgrp_tree *)osi_first(tree))) {
+	while ((rg = (struct lgfs2_rgrp_tree *)osi_first(tree))) {
 		int i;
-		free(rg->bits[0].bi_data);
-		for (i = 0; i < rg->ri.ri_length; i++) {
-			rg->bits[i].bi_data = NULL;
+		free(rg->rt_bits[0].bi_data);
+		for (i = 0; i < rg->rt_length; i++) {
+			rg->rt_bits[i].bi_data = NULL;
 		}
-		osi_erase(&rg->node, tree);
+		osi_erase(&rg->rt_node, tree);
 		free(rg);
 	}
-	free((*rgs)->plan);
+	free((*rgs)->rgs_plan);
 	free(*rgs);
 	*rgs = NULL;
 }
 
+/**
+ * Given a number of blocks in a resource group, return the number of blocks
+ * needed for bitmaps. Also calculate the adjusted number of free data blocks
+ * in the resource group and store it in *ri_data.
+ */
+uint32_t lgfs2_rgblocks2bitblocks(const unsigned int bsize, const uint32_t rgblocks, uint32_t *ri_data)
+{
+	uint32_t mappable = 0;
+	uint32_t bitblocks = 0;
+	/* Number of blocks mappable by bitmap blocks with these header types */
+	const uint32_t blks_rgrp = GFS2_NBBY * (bsize - sizeof(struct gfs2_rgrp));
+	const uint32_t blks_meta = GFS2_NBBY * (bsize - sizeof(struct gfs2_meta_header));
+
+	while (blks_rgrp + (blks_meta * bitblocks) < ((rgblocks - bitblocks) & ~(uint32_t)3))
+		bitblocks++;
+
+	if (bitblocks > 0)
+		mappable = blks_rgrp + (blks_meta * (bitblocks - 1));
+
+	*ri_data = (rgblocks - (bitblocks + 1)) & ~(uint32_t)3;
+	if (mappable < *ri_data)
+		bitblocks++;
+
+	return bitblocks;
+}
+
 /**
  * Calculate the fields for a new entry in the resource group index.
  * ri: A pointer to the resource group index entry to be calculated.
@@ -537,7 +559,8 @@ void lgfs2_rgrps_free(lgfs2_rgrps_t *rgs)
  */
 uint64_t lgfs2_rindex_entry_new(lgfs2_rgrps_t rgs, struct gfs2_rindex *ri, uint64_t addr, uint32_t len)
 {
-	struct rg_spec *spec = rgs->plan->rg_specs;
+	struct rg_spec *spec = rgs->rgs_plan->rg_specs;
+	uint32_t ri_length, ri_data;
 	int plan = -1;
 	errno = EINVAL;
 	if (!ri)
@@ -556,37 +579,19 @@ uint64_t lgfs2_rindex_entry_new(lgfs2_rgrps_t rgs, struct gfs2_rindex *ri, uint6
 		spec[plan].num--;
 	}
 
-	if (addr + len > rgs->sdp->device.length)
+	if (addr + len > rgs->rgs_sdp->device.length)
 		return 0;
 
-	ri->ri_addr = addr;
-	ri->ri_length = rgblocks2bitblocks(rgs->sdp->bsize, len, &ri->ri_data);
+	ri_length = lgfs2_rgblocks2bitblocks(rgs->rgs_sdp->sd_bsize, len, &ri_data);
+	ri->ri_addr = cpu_to_be64(addr);
+	ri->ri_length = cpu_to_be32(ri_length);
+	ri->ri_data = cpu_to_be32(ri_data);
 	ri->__pad = 0;
-	ri->ri_data0 = ri->ri_addr + ri->ri_length;
-	ri->ri_bitbytes = ri->ri_data / GFS2_NBBY;
+	ri->ri_data0 = cpu_to_be64(addr + ri_length);
+	ri->ri_bitbytes = cpu_to_be32(ri_data / GFS2_NBBY);
 	memset(&ri->ri_reserved, 0, sizeof(ri->ri_reserved));
 
-	return ri->ri_addr + len;
-}
-
-/**
- * Return the rindex structure relating to a resource group.
- * The return type is const to advise callers that making changes to this
- * structure directly isn't wise. libgfs2 functions should be used instead.
- */
-const struct gfs2_rindex *lgfs2_rgrp_index(lgfs2_rgrp_t rg)
-{
-	return &rg->ri;
-}
-
-/**
- * Return the rgrp structure relating to a resource group.
- * The return type is const to advise callers that making changes to this
- * structure directly isn't wise. libgfs2 functions should be used instead.
- */
-const struct gfs2_rgrp *lgfs2_rgrp_rgrp(lgfs2_rgrp_t rg)
-{
-	return &rg->rg;
+	return addr + len;
 }
 
 /**
@@ -604,9 +609,9 @@ unsigned lgfs2_rgsize_for_data(uint64_t blksreq, unsigned bsize)
 }
 
 // Temporary function to aid in API migration
-struct osi_node *lgfs2_rgrps_root(lgfs2_rgrps_t rgs)
+void lgfs2_attach_rgrps(struct lgfs2_sbd *sdp, lgfs2_rgrps_t rgs)
 {
-	return rgs->root.osi_node;
+	sdp->rgtree.osi_node = rgs->rgs_root.osi_node;
 }
 
 /**
@@ -619,8 +624,8 @@ struct osi_node *lgfs2_rgrps_root(lgfs2_rgrps_t rgs)
 lgfs2_rgrp_t lgfs2_rgrps_append(lgfs2_rgrps_t rgs, struct gfs2_rindex *entry, uint32_t rg_skip)
 {
 	lgfs2_rgrp_t rg;
-	struct osi_node **link = &rgs->root.osi_node;
-	struct osi_node *parent = osi_last(&rgs->root);
+	struct osi_node **link = &rgs->rgs_root.osi_node;
+	struct osi_node *parent = osi_last(&rgs->rgs_root);
 	lgfs2_rgrp_t lastrg = (lgfs2_rgrp_t)parent;
 
 	errno = EINVAL;
@@ -628,36 +633,32 @@ lgfs2_rgrp_t lgfs2_rgrps_append(lgfs2_rgrps_t rgs, struct gfs2_rindex *entry, ui
 		return NULL;
 
 	if (lastrg != NULL) { /* Tree is not empty */
-		if (entry->ri_addr <= lastrg->ri.ri_addr)
+		if (be64_to_cpu(entry->ri_addr) <= lastrg->rt_addr)
 			return NULL; /* Appending with a lower address doesn't make sense */
-		link = &lastrg->node.osi_right;
+		link = &lastrg->rt_node.osi_right;
 	}
 
-	rg = calloc(1, sizeof(*rg) + (entry->ri_length * sizeof(struct gfs2_bitmap)));
+	rg = calloc(1, sizeof(*rg) + (be32_to_cpu(entry->ri_length) * sizeof(struct lgfs2_bitmap)));
 	if (rg == NULL)
 		return NULL;
 
-	rg->bits = (struct gfs2_bitmap *)(rg + 1);
+	rg->rt_bits = (struct lgfs2_bitmap *)(rg + 1);
 
-	osi_link_node(&rg->node, parent, link);
-	osi_insert_color(&rg->node, &rgs->root);
+	osi_link_node(&rg->rt_node, parent, link);
+	osi_insert_color(&rg->rt_node, &rgs->rgs_root);
 
-	memcpy(&rg->ri, entry, sizeof(struct gfs2_rindex));
-	rg->rg.rg_header.mh_magic = GFS2_MAGIC;
-	rg->rg.rg_header.mh_type = GFS2_METATYPE_RG;
-	rg->rg.rg_header.mh_format = GFS2_FORMAT_RG;
-	rg->rg.rg_free = rg->ri.ri_data;
-#ifdef GFS2_HAS_RG_SKIP
-	rg->rg.rg_skip = rg_skip;
-#endif
-#ifdef GFS2_HAS_RG_RI_FIELDS
-	rg->rg.rg_data0 = rg->ri.ri_data0;
-	rg->rg.rg_data = rg->ri.ri_data;
-	rg->rg.rg_bitbytes = rg->ri.ri_bitbytes;
-	rg->rg.rg_crc = 0;
-#endif
-	compute_bitmaps(rg, rgs->sdp->bsize);
-	rg->rgrps = rgs;
+	rg->rt_addr = be64_to_cpu(entry->ri_addr);
+	rg->rt_length = be32_to_cpu(entry->ri_length);
+	rg->rt_data0 = rg->rt_rg_data0 = be64_to_cpu(entry->ri_data0);
+	rg->rt_data = rg->rt_rg_data = be32_to_cpu(entry->ri_data);
+	rg->rt_bitbytes = rg->rt_rg_bitbytes = be32_to_cpu(entry->ri_bitbytes);
+	rg->rt_flags = 0;
+	rg->rt_free = be32_to_cpu(entry->ri_data);
+	rg->rt_dinodes = 0;
+	rg->rt_skip = rg_skip;
+	rg->rt_igeneration = 0;
+	compute_bitmaps(rg, rgs->rgs_sdp->sd_bsize);
+	rg->rt_rgrps = rgs;
 	return rg;
 }
 
@@ -667,32 +668,32 @@ lgfs2_rgrp_t lgfs2_rgrps_append(lgfs2_rgrps_t rgs, struct gfs2_rindex *entry, ui
  */
 int lgfs2_rgrp_write(int fd, const lgfs2_rgrp_t rg)
 {
-	struct gfs2_sbd *sdp = rg->rgrps->sdp;
+	struct lgfs2_sbd *sdp = rg->rt_rgrps->rgs_sdp;
 	unsigned int i;
-	const struct gfs2_meta_header bmh = {
-		.mh_magic = GFS2_MAGIC,
-		.mh_type = GFS2_METATYPE_RB,
-		.mh_format = GFS2_FORMAT_RB,
-	};
 	int freebufs = 0;
 	ssize_t ret;
 	size_t len;
 
-	if (rg->bits[0].bi_data == NULL) {
+	if (rg->rt_bits[0].bi_data == NULL) {
 		freebufs = 1;
 		if (lgfs2_rgrp_bitbuf_alloc(rg) != 0)
 			return -1;
 	}
-	gfs2_rgrp_out(&rg->rg, rg->bits[0].bi_data);
-	for (i = 1; i < rg->ri.ri_length; i++) {
-		gfs2_meta_header_out(&bmh, rg->bits[i].bi_data);
+	lgfs2_rgrp_out(rg, rg->rt_bits[0].bi_data);
+	for (i = 1; i < rg->rt_length; i++) {
+		struct gfs2_meta_header *mh = (void *) rg->rt_bits[i].bi_data;
+
+		mh->mh_magic = cpu_to_be32(GFS2_MAGIC);
+		mh->mh_type = cpu_to_be32(GFS2_METATYPE_RB);
+		mh->mh_format = cpu_to_be32(GFS2_FORMAT_RB);
 	}
 
-	len = sdp->bsize * rg->ri.ri_length;
-	if (rg->rgrps->align > 0)
-		len = ROUND_UP(len, rg->rgrps->align * sdp->bsize);
+	len = sdp->sd_bsize * rg->rt_length;
+	if (rg->rt_rgrps->rgs_align > 0)
+		len = ROUND_UP(len, rg->rt_rgrps->rgs_align * sdp->sd_bsize);
 
-	ret = pwrite(fd, rg->bits[0].bi_data, len, rg->ri.ri_addr * sdp->bsize);
+	ret = pwrite(fd, rg->rt_bits[0].bi_data, len,
+		     rg->rt_addr * sdp->sd_bsize);
 
 	if (freebufs)
 		lgfs2_rgrp_bitbuf_free(rg);
@@ -711,7 +712,7 @@ int lgfs2_rgrps_write_final(int fd, lgfs2_rgrps_t rgs)
 {
 	lgfs2_rgrp_t rg = lgfs2_rgrp_last(rgs);
 
-	rg->rg.rg_skip = 0;
+	rg->rt_skip = 0;
 	if (lgfs2_rgrp_write(fd, rg) != 0)
 		return -1;
 	return 0;
@@ -719,22 +720,22 @@ int lgfs2_rgrps_write_final(int fd, lgfs2_rgrps_t rgs)
 
 lgfs2_rgrp_t lgfs2_rgrp_first(lgfs2_rgrps_t rgs)
 {
-	return (lgfs2_rgrp_t)osi_first(&rgs->root);
+	return (lgfs2_rgrp_t)osi_first(&rgs->rgs_root);
 }
 
 lgfs2_rgrp_t lgfs2_rgrp_next(lgfs2_rgrp_t rg)
 {
-	return (lgfs2_rgrp_t)osi_next(&rg->node);
+	return (lgfs2_rgrp_t)osi_next(&rg->rt_node);
 }
 
 lgfs2_rgrp_t lgfs2_rgrp_prev(lgfs2_rgrp_t rg)
 {
-	return (lgfs2_rgrp_t)osi_prev(&rg->node);
+	return (lgfs2_rgrp_t)osi_prev(&rg->rt_node);
 }
 
 lgfs2_rgrp_t lgfs2_rgrp_last(lgfs2_rgrps_t rgs)
 {
-	return (lgfs2_rgrp_t)osi_last(&rgs->root);
+	return (lgfs2_rgrp_t)osi_last(&rgs->rgs_root);
 }
 
 /**
@@ -751,14 +752,14 @@ lgfs2_rgrp_t lgfs2_rgrp_last(lgfs2_rgrps_t rgs)
  */
 int lgfs2_rbm_from_block(struct lgfs2_rbm *rbm, uint64_t block)
 {
-	uint64_t rblock = block - rbm->rgd->ri.ri_data0;
-	struct gfs2_sbd *sdp = rbm->rgd->rgrps->sdp;
+	uint64_t rblock = block - rbm->rgd->rt_data0;
+	struct lgfs2_sbd *sdp = rbm->rgd->rt_rgrps->rgs_sdp;
 
 	if (rblock > UINT_MAX) {
 		errno = EINVAL;
 		return 1;
 	}
-	if (block >= rbm->rgd->ri.ri_data0 + rbm->rgd->ri.ri_data) {
+	if (block >= rbm->rgd->rt_data0 + rbm->rgd->rt_data) {
 		errno = E2BIG;
 		return 1;
 	}
@@ -794,7 +795,7 @@ static int lgfs2_rbm_incr(struct lgfs2_rbm *rbm)
 		rbm->offset++;
 		return 0;
 	}
-	if (rbm->bii == rbm->rgd->ri.ri_length - 1) /* at the last bitmap */
+	if (rbm->bii == rbm->rgd->rt_length - 1) /* at the last bitmap */
 		return 1;
 
 	rbm->offset = 0;
@@ -810,7 +811,7 @@ static int lgfs2_rbm_incr(struct lgfs2_rbm *rbm)
  */
 static inline uint8_t lgfs2_testbit(const struct lgfs2_rbm *rbm)
 {
-	struct gfs2_bitmap *bi = rbm_bi(rbm);
+	struct lgfs2_bitmap *bi = rbm_bi(rbm);
 	const uint8_t *buffer = (uint8_t *)bi->bi_data + bi->bi_offset;
 	const uint8_t *byte;
 	unsigned int bit;
@@ -882,8 +883,8 @@ static uint32_t lgfs2_free_extlen(const struct lgfs2_rbm *rrbm, uint32_t len)
 	uint32_t chunk_size;
 	uint8_t *ptr, *start, *end;
 	uint64_t block;
-	struct gfs2_bitmap *bi;
-	struct gfs2_sbd *sdp = rbm.rgd->rgrps->sdp;
+	struct lgfs2_bitmap *bi;
+	struct lgfs2_sbd *sdp = rbm.rgd->rt_rgrps->rgs_sdp;
 
 	if (n_unaligned &&
 	    lgfs2_unaligned_extlen(&rbm, 4 - n_unaligned, &len))
@@ -894,7 +895,7 @@ static uint32_t lgfs2_free_extlen(const struct lgfs2_rbm *rrbm, uint32_t len)
 	while (len > 3) {
 		bi = rbm_bi(&rbm);
 		start = (uint8_t *)bi->bi_data;
-		end = start + sdp->bsize;
+		end = start + sdp->sd_bsize;
 		start += bi->bi_offset;
 		start += (rbm.offset / GFS2_NBBY);
 		bytes = (len / GFS2_NBBY) < (end - start) ? (len / GFS2_NBBY):(end - start);
@@ -935,7 +936,7 @@ int lgfs2_rbm_find(struct lgfs2_rbm *rbm, uint8_t state, uint32_t *minext)
 	int initial_bii;
 	uint32_t offset;
 	int n = 0;
-	int iters = rbm->rgd->ri.ri_length;
+	int iters = rbm->rgd->rt_length;
 	uint32_t extlen;
 
 	/* If we are not starting at the beginning of a bitmap, then we
@@ -946,16 +947,16 @@ int lgfs2_rbm_find(struct lgfs2_rbm *rbm, uint8_t state, uint32_t *minext)
 		iters++;
 
 	for (n = 0; n < iters; n++) {
-		struct gfs2_bitmap *bi = rbm_bi(rbm);
+		struct lgfs2_bitmap *bi = rbm_bi(rbm);
 		uint8_t *buf = (uint8_t *)bi->bi_data + bi->bi_offset;
 		uint64_t block;
 		int ret;
 
-		if ((rbm->rgd->rg.rg_free < *minext) && (state == GFS2_BLKST_FREE))
+		if ((rbm->rgd->rt_free < *minext) && (state == GFS2_BLKST_FREE))
 			goto next_bitmap;
 
-		offset = gfs2_bitfit(buf, bi->bi_len, rbm->offset, state);
-		if (offset == BFITNOENT)
+		offset = lgfs2_bitfit(buf, bi->bi_len, rbm->offset, state);
+		if (offset == LGFS2_BFITNOENT)
 			goto next_bitmap;
 
 		rbm->offset = offset;
@@ -987,7 +988,7 @@ int lgfs2_rbm_find(struct lgfs2_rbm *rbm, uint8_t state, uint32_t *minext)
 next_bitmap:	/* Find next bitmap in the rgrp */
 		rbm->offset = 0;
 		rbm->bii++;
-		if (rbm->bii == rbm->rgd->ri.ri_length)
+		if (rbm->bii == rbm->rgd->rt_length)
 			rbm->bii = 0;
 
 res_covered_end_of_rgrp:
@@ -1012,13 +1013,13 @@ unsigned lgfs2_alloc_extent(const struct lgfs2_rbm *rbm, int state, const unsign
 	const uint64_t block = lgfs2_rbm_to_block(rbm);
 	unsigned len;
 
-	gfs2_set_bitmap(rbm->rgd, block, state);
+	lgfs2_set_bitmap(rbm->rgd, block, state);
 
 	for (len = 1; len < elen; len++) {
 		int ret = lgfs2_rbm_from_block(&pos, block + len);
 		if (ret || lgfs2_testbit(&pos) != GFS2_BLKST_FREE)
 			break;
-		gfs2_set_bitmap(pos.rgd, block + len, GFS2_BLKST_USED);
+		lgfs2_set_bitmap(pos.rgd, block + len, GFS2_BLKST_USED);
 	}
 	return len;
 }
