@@ -22,17 +22,14 @@
 #include <limits.h>
 #include <blkid.h>
 #include <locale.h>
+#include <uuid.h>
 
 #define _(String) gettext(String)
 
-#include <linux/types.h>
 #include "libgfs2.h"
 #include "gfs2_mkfs.h"
 #include "progress.h"
-
-#ifdef GFS2_HAS_UUID
-#include <uuid.h>
-#endif
+#include "struct_print.h"
 
 static void print_usage(const char *prog_name)
 {
@@ -55,6 +52,7 @@ static void print_usage(const char *prog_name)
 	    "-q", NULL,          _("Don't print anything"),
 	    "-r", _("<size>"),   _("Size of resource groups, in megabytes"),
 	    "-t", _("<name>"),   _("Name of the lock table"),
+	    "-U", _("<UUID>"),   _("The UUID of the file system"),
 	    "-V", NULL,          _("Display program version information, then exit"),
 	    NULL, NULL, NULL /* Must be kept at the end */
 	};
@@ -112,6 +110,9 @@ struct mkfs_dev {
 	unsigned int got_topol:1;
 };
 
+/* Number of resource groups */
+uint64_t nrgrp = 0;
+
 struct mkfs_opts {
 	unsigned bsize;
 	unsigned qcsize;
@@ -124,8 +125,10 @@ struct mkfs_opts {
 	int journals;
 	const char *lockproto;
 	const char *locktable;
+	const char *uuid;
 	struct mkfs_dev dev;
 	unsigned discard:1;
+	unsigned root_inherit_jd:1;
 
 	unsigned got_bsize:1;
 	unsigned got_qcsize:1;
@@ -140,6 +143,7 @@ struct mkfs_opts {
 	unsigned got_device:1;
 	unsigned got_topol:1;
 	unsigned got_format:1;
+	unsigned got_uuid:1;
 
 	unsigned override:1;
 	unsigned quiet:1;
@@ -153,10 +157,10 @@ static void opts_init(struct mkfs_opts *opts)
 	memset(opts, 0, sizeof(*opts));
 	opts->discard = 1;
 	opts->journals = 1;
-	opts->bsize = GFS2_DEFAULT_BSIZE;
-	opts->jsize = GFS2_DEFAULT_JSIZE;
-	opts->qcsize = GFS2_DEFAULT_QCSIZE;
-	opts->rgsize = GFS2_DEFAULT_RGSIZE;
+	opts->bsize = LGFS2_DEFAULT_BSIZE;
+	opts->jsize = LGFS2_DEFAULT_JSIZE;
+	opts->qcsize = LGFS2_DEFAULT_QCSIZE;
+	opts->rgsize = LGFS2_DEFAULT_RGSIZE;
 	opts->lockproto = "lock_dlm";
 	opts->locktable = "";
 	opts->confirm = 1;
@@ -164,7 +168,7 @@ static void opts_init(struct mkfs_opts *opts)
 	opts->format = GFS2_FORMAT_FS;
 }
 
-struct gfs2_inum *mkfs_journals = NULL;
+static struct lgfs2_inum *mkfs_journals = NULL;
 
 #ifndef BLKDISCARD
 #define BLKDISCARD      _IO(0x12,119)
@@ -172,16 +176,15 @@ struct gfs2_inum *mkfs_journals = NULL;
 
 static int discard_blocks(int fd, uint64_t len, int debug)
 {
-        __uint64_t range[2];
+        uint64_t range[2];
 
 	range[0] = 0;
 	range[1] = len;
 	if (debug)
 		/* Translators: "discard" is a request sent to a storage device to
 		 * discard a range of blocks. */
-		printf(_("Issuing discard request: range: %llu - %llu..."),
-		       (unsigned long long)range[0],
-		       (unsigned long long)range[1]);
+		printf(_("Issuing discard request: range: %"PRIu64" - %"PRIu64"..."),
+		       range[0], range[1]);
 	if (ioctl(fd, BLKDISCARD, &range) < 0) {
 		if (debug)
 			printf("%s = %d\n", _("error"), errno);
@@ -234,7 +237,8 @@ static long long cvtnum(unsigned int blocksize, unsigned int sectorsize, const c
         return -1LL;
 }
 
-static int parse_ulong(struct mkfs_opts *opts, const char *key, const char *val, unsigned long *pn)
+static int parse_ulong(struct mkfs_opts *opts, const char *key, const char *val, unsigned long *pn,
+                       unsigned long max)
 {
 	long long l;
 	if (val == NULL || *val == '\0') {
@@ -242,7 +246,7 @@ static int parse_ulong(struct mkfs_opts *opts, const char *key, const char *val,
 		return -1;
 	}
 	l = cvtnum(opts->bsize, 0, val);
-	if (l > ULONG_MAX || l < 0) {
+	if ((max > 0 && l > max) || l < 0) {
 		fprintf(stderr, _("Value of '%s' is invalid\n"), key);
 		return -1;
 	}
@@ -283,7 +287,7 @@ static int parse_topology(struct mkfs_opts *opts, char *str)
 			fprintf(stderr, "Too many topology values.\n");
 			return 1;
 		}
-		if (parse_ulong(opts, "test_topology", opt, topol[i]))
+		if (parse_ulong(opts, "test_topology", opt, topol[i], 0))
 			return 1;
 		i++;
 	}
@@ -298,10 +302,10 @@ static int parse_format(struct mkfs_opts *opts, char *str)
 {
 	unsigned long ln;
 
-	if (parse_ulong(opts, "format", str, &ln) != 0)
+	if (parse_ulong(opts, "format", str, &ln, LGFS2_FS_FORMAT_MAX) != 0)
 		return -1;
 
-	if (ln < LGFS2_FS_FORMAT_MIN || ln > LGFS2_FS_FORMAT_MAX) {
+	if (ln < LGFS2_FS_FORMAT_MIN) {
 		fprintf(stderr, _("Invalid filesystem format: %s\n"), str);
 		return -1;
 	}
@@ -310,6 +314,21 @@ static int parse_format(struct mkfs_opts *opts, char *str)
 	return 0;
 }
 
+static int parse_root_inherit_jd(struct mkfs_opts *opts, const char *str)
+{
+	unsigned long n = 0;
+
+	if (str == NULL) { /* -o root_inherit_jdata */
+		opts->root_inherit_jd = 1;
+		return 0;
+	}
+	/* -o root_inherit_jdata=N */
+	if (parse_ulong(opts, "root_inherit_jdata", str, &n, 1) != 0)
+		return -1;
+	opts->root_inherit_jd = (unsigned)n;
+	return 0;
+}
+
 static int opt_parse_extended(char *str, struct mkfs_opts *opts)
 {
 	char *opt;
@@ -321,11 +340,11 @@ static int opt_parse_extended(char *str, struct mkfs_opts *opts)
 			return -1;
 		}
 		if (strcmp("sunit", key) == 0) {
-			if (parse_ulong(opts, "sunit", val, &opts->sunit) != 0)
+			if (parse_ulong(opts, "sunit", val, &opts->sunit, 0) != 0)
 				return -1;
 			opts->got_sunit = 1;
 		} else if (strcmp("swidth", key) == 0) {
-			if (parse_ulong(opts, "swidth", val, &opts->swidth) != 0)
+			if (parse_ulong(opts, "swidth", val, &opts->swidth, 0) != 0)
 				return -1;
 			opts->got_swidth = 1;
 		} else if (strcmp("align", key) == 0) {
@@ -339,6 +358,9 @@ static int opt_parse_extended(char *str, struct mkfs_opts *opts)
 		} else if (strcmp("format", key) == 0) {
 			if (parse_format(opts, val) != 0)
 				return -1;
+		} else if (strcmp("root_inherit_jdata", key) == 0) {
+			if (parse_root_inherit_jd(opts, val) != 0)
+				return -1;
 		} else if (strcmp("help", key) == 0) {
 			print_ext_opts();
 			return 1;
@@ -356,7 +378,7 @@ static int opts_get(int argc, char *argv[], struct mkfs_opts *opts)
 	int ret;
 	int c;
 	while (1) {
-		c = getopt(argc, argv, "-b:c:DhJ:j:KOo:p:qr:t:V");
+		c = getopt(argc, argv, "-b:c:DhJ:j:KOo:p:qr:t:U:V");
 		if (c == -1)
 			break;
 
@@ -371,7 +393,6 @@ static int opts_get(int argc, char *argv[], struct mkfs_opts *opts)
 			break;
 		case 'D':
 			opts->debug = 1;
-			lgfs2_set_debug(1);
 			break;
 		case 'h':
 			print_usage(argv[0]);
@@ -410,6 +431,10 @@ static int opts_get(int argc, char *argv[], struct mkfs_opts *opts)
 			if (ret != 0)
 				return ret;
 			break;
+		case 'U':
+			opts->uuid = optarg;
+			opts->got_uuid = 1;
+			break;
 		case 'V':
 			printf("mkfs.gfs2 %s (built %s %s)\n", VERSION,
 			       __DATE__, __TIME__);
@@ -566,10 +591,10 @@ static int choose_blocksize(struct mkfs_opts *opts, unsigned *pbsize)
 	}
 	if (!opts->got_bsize && got_topol) {
 		if (dev->optimal_io_size <= getpagesize() &&
-		    dev->optimal_io_size >= GFS2_DEFAULT_BSIZE)
+		    dev->optimal_io_size >= LGFS2_DEFAULT_BSIZE)
 			bsize = dev->optimal_io_size;
 		else if (dev->physical_sector_size <= getpagesize() &&
-		         dev->physical_sector_size >= GFS2_DEFAULT_BSIZE)
+		         dev->physical_sector_size >= LGFS2_DEFAULT_BSIZE)
 			bsize = dev->physical_sector_size;
 	}
 	/* Block sizes must be a power of two from 512 to 65536 */
@@ -607,7 +632,7 @@ static int opts_check(struct mkfs_opts *opts)
 	if (test_locking(opts) != 0)
 		return -1;
 
-	if (GFS2_MIN_RGSIZE > opts->rgsize || opts->rgsize > GFS2_MAX_RGSIZE) {
+	if (LGFS2_MIN_RGSIZE > opts->rgsize || opts->rgsize > LGFS2_MAX_RGSIZE) {
 		/* Translators: gfs2 file systems are split into equal sized chunks called
 		   resource groups. We're checking that the user gave a valid size for them. */
 		fprintf(stderr, _("Bad resource group size\n"));
@@ -622,7 +647,7 @@ static int opts_check(struct mkfs_opts *opts)
 		fprintf(stderr, _("Number of journals cannot be negative: %d\n"), opts->journals);
 		return -1;
 	}
-	if (opts->jsize < GFS2_MIN_JSIZE || opts->jsize > GFS2_MAX_JSIZE) {
+	if (opts->jsize < LGFS2_MIN_JSIZE || opts->jsize > LGFS2_MAX_JSIZE) {
 		fprintf(stderr, _("Bad journal size\n"));
 		return -1;
 	}
@@ -639,30 +664,27 @@ static int opts_check(struct mkfs_opts *opts)
 	return 0;
 }
 
-static void print_results(struct gfs2_sb *sb, struct mkfs_opts *opts, uint64_t rgrps, uint64_t fssize)
+static void print_results(struct lgfs2_sbd *sb, struct mkfs_opts *opts)
 {
+	char readable_uuid[36+1];
+
+	uuid_unparse(sb->sd_uuid, readable_uuid);
+
 	printf("%-27s%s\n", _("Device:"), opts->dev.path);
-	printf("%-27s%u\n", _("Block size:"), sb->sb_bsize);
+	printf("%-27s%u\n", _("Block size:"), sb->sd_bsize);
 	printf("%-27s%.2f %s (%"PRIu64" %s)\n", _("Device size:"),
 	       /* Translators: "GB" here means "gigabytes" */
 	       (opts->dev.size / ((float)(1 << 30))), _("GB"),
-	       (opts->dev.size / sb->sb_bsize), _("blocks"));
+	       (opts->dev.size / sb->sd_bsize), _("blocks"));
 	printf("%-27s%.2f %s (%"PRIu64" %s)\n", _("Filesystem size:"),
-	       (fssize / ((float)(1 << 30)) * sb->sb_bsize), _("GB"), fssize, _("blocks"));
+	       (sb->fssize / ((float)(1 << 30)) * sb->sd_bsize), _("GB"), sb->fssize, _("blocks"));
 	printf("%-27s%u\n", _("Journals:"), opts->journals);
 	printf("%-27s%uMB\n", _("Journal size:"), opts->jsize);
-	printf("%-27s%"PRIu64"\n", _("Resource groups:"), rgrps);
+	printf("%-27s%"PRIu64"\n", _("Resource groups:"), nrgrp);
 	printf("%-27s\"%s\"\n", _("Locking protocol:"), opts->lockproto);
 	printf("%-27s\"%s\"\n", _("Lock table:"), opts->locktable);
-#ifdef GFS2_HAS_UUID
-	{
-	char readable_uuid[36+1];
-
-	uuid_unparse(sb->sb_uuid, readable_uuid);
 	/* Translators: "UUID" = universally unique identifier. */
 	printf("%-27s%s\n", _("UUID:"), readable_uuid);
-	}
-#endif
 }
 
 static int warn_of_destruction(const char *path)
@@ -675,7 +697,8 @@ static int warn_of_destruction(const char *path)
 		return -1;
 	}
 	if (S_ISLNK(lnkstat.st_mode)) {
-		abspath = canonicalize_file_name(path);
+		/* coverity[toctou:SUPPRESS] */
+		abspath = realpath(path, NULL);
 		if (abspath == NULL) {
 			perror(_("Could not find the absolute path of the device"));
 			return -1;
@@ -689,7 +712,73 @@ static int warn_of_destruction(const char *path)
 	return 0;
 }
 
-static int zero_gap(struct gfs2_sbd *sdp, uint64_t addr, size_t blocks)
+static int build_per_node(struct lgfs2_sbd *sdp, struct mkfs_opts *opts)
+{
+	struct lgfs2_inode *per_node;
+	unsigned int j;
+
+	/* coverity[identity_transfer:SUPPRESS] False positive */
+	per_node = lgfs2_createi(sdp->master_dir, "per_node", S_IFDIR | 0700,
+			   GFS2_DIF_SYSTEM);
+	if (per_node == NULL) {
+		fprintf(stderr, _("Error building '%s': %s\n"), "per_node", strerror(errno));
+		return -1;
+	}
+	for (j = 0; j < sdp->md.journals; j++) {
+		struct lgfs2_inode *ip;
+
+		/* coverity[identity_transfer:SUPPRESS] False positive */
+		ip = lgfs2_build_inum_range(per_node, j);
+		if (ip == NULL) {
+			fprintf(stderr, _("Error building '%s': %s\n"), "inum_range",
+			        strerror(errno));
+			lgfs2_inode_put(&per_node);
+			return 1;
+		}
+		if (opts->debug) {
+			printf("\nInum Range %u:\n", j);
+			dinode_print(ip->i_bh->b_data);
+		}
+		lgfs2_inode_put(&ip);
+
+		/* coverity[identity_transfer:SUPPRESS] False positive */
+		ip = lgfs2_build_statfs_change(per_node, j);
+		if (ip == NULL) {
+			fprintf(stderr, _("Error building '%s': %s\n"), "statfs_change",
+			        strerror(errno));
+			lgfs2_inode_put(&per_node);
+			return 1;
+		}
+		if (opts->debug) {
+			printf("\nStatFS Change %u:\n", j);
+			dinode_print(ip->i_bh->b_data);
+		}
+		lgfs2_inode_put(&ip);
+
+		/* coverity[identity_transfer:SUPPRESS] False positive */
+		ip = lgfs2_build_quota_change(per_node, j);
+		if (ip == NULL) {
+			fprintf(stderr, _("Error building '%s': %s\n"), "quota_change",
+			        strerror(errno));
+			/* coverity[deref_arg:SUPPRESS] */
+			lgfs2_inode_put(&per_node);
+			return 1;
+		}
+		if (opts->debug) {
+			printf("\nQuota Change %u:\n", j);
+			dinode_print(ip->i_bh->b_data);
+		}
+		lgfs2_inode_put(&ip);
+	}
+	if (opts->debug) {
+		printf("\nper_node:\n");
+		dinode_print(per_node->i_bh->b_data);
+	}
+	lgfs2_inode_put(&per_node);
+	return 0;
+}
+
+static int zero_gap(struct lgfs2_sbd *sdp, uint64_t addr, size_t blocks)
 {
 	struct iovec *iov;
 	char *zerobuf;
@@ -703,7 +792,7 @@ static int zero_gap(struct gfs2_sbd *sdp, uint64_t addr, size_t blocks)
 		perror(_("Failed to zero blocks\n"));
 		return 1;
 	}
-	zerobuf = calloc(1, sdp->bsize);
+	zerobuf = calloc(1, sdp->sd_bsize);
 	if (zerobuf == NULL) {
 		perror(_("Failed to zero blocks\n"));
 		free(iov);
@@ -711,10 +800,10 @@ static int zero_gap(struct gfs2_sbd *sdp, uint64_t addr, size_t blocks)
 	}
 	for (i = 0; i < blocks; i++) {
 		iov[i].iov_base = zerobuf;
-		iov[i].iov_len = sdp->bsize;
+		iov[i].iov_len = sdp->sd_bsize;
 	}
-	wrote = pwritev(sdp->device_fd, iov, blocks, addr * sdp->bsize);
-	if (wrote != blocks * sdp->bsize) {
+	wrote = pwritev(sdp->device_fd, iov, blocks, addr * sdp->sd_bsize);
+	if (wrote != blocks * sdp->sd_bsize) {
 		fprintf(stderr, _("Zeroing write failed at block %"PRIu64"\n"), addr);
 		free(zerobuf);
 		free(iov);
@@ -725,24 +814,24 @@ static int zero_gap(struct gfs2_sbd *sdp, uint64_t addr, size_t blocks)
 	return 0;
 }
 
-static lgfs2_rgrps_t rgs_init(struct mkfs_opts *opts, struct gfs2_sbd *sdp)
+static lgfs2_rgrps_t rgs_init(struct mkfs_opts *opts, struct lgfs2_sbd *sdp)
 {
 	lgfs2_rgrps_t rgs;
 	uint64_t al_base = 0;
 	uint64_t al_off = 0;
 
 	if (opts->align && opts->got_sunit) {
-		if ((opts->sunit % sdp->bsize) != 0) {
+		if ((opts->sunit % sdp->sd_bsize) != 0) {
 			fprintf(stderr, _("Stripe unit (%lu) must be a multiple of block size (%u)\n"),
-			        opts->sunit, sdp->bsize);
+			        opts->sunit, sdp->sd_bsize);
 			return NULL;
 		} else if ((opts->swidth % opts->sunit) != 0) {
 			fprintf(stderr, _("Stripe width (%lu) must be a multiple of stripe unit (%lu)\n"),
 			        opts->swidth, opts->sunit);
 			return NULL;
 		} else {
-			al_base = opts->swidth / sdp->bsize;
-			al_off = opts->sunit / sdp->bsize;
+			al_base = opts->swidth / sdp->sd_bsize;
+			al_off = opts->sunit / sdp->sd_bsize;
 		}
 	} else if (opts->align) {
 		if (opts->dev.optimal_io_size <= opts->dev.physical_sector_size ||
@@ -753,8 +842,8 @@ static lgfs2_rgrps_t rgs_init(struct mkfs_opts *opts, struct gfs2_sbd *sdp)
 			   rgrp alignment */
 			opts->align = 0;
 		} else {
-			al_base = opts->dev.optimal_io_size / sdp->bsize;
-			al_off = opts->dev.minimum_io_size / sdp->bsize;
+			al_base = opts->dev.optimal_io_size / sdp->sd_bsize;
+			al_off = opts->dev.minimum_io_size / sdp->sd_bsize;
 		}
 	}
 
@@ -775,19 +864,24 @@ static lgfs2_rgrps_t rgs_init(struct mkfs_opts *opts, struct gfs2_sbd *sdp)
 	return rgs;
 }
 
-static int place_rgrp(struct gfs2_sbd *sdp, lgfs2_rgrp_t rg, int debug)
+static int place_rgrp(struct lgfs2_sbd *sdp, lgfs2_rgrp_t rg, int debug)
 {
-	uint64_t prev_end = (GFS2_SB_ADDR * GFS2_BASIC_BLOCK / sdp->bsize) + 1;
-	const struct gfs2_rindex *ri = lgfs2_rgrp_index(rg);
+	uint64_t prev_end = (GFS2_SB_ADDR * GFS2_BASIC_BLOCK / sdp->sd_bsize) + 1;
 	lgfs2_rgrp_t prev = lgfs2_rgrp_prev(rg);
+	struct gfs2_rindex ri;
+	uint64_t addr;
 	int err = 0;
 
 	if (prev != NULL) {
-		prev_end = lgfs2_rgrp_index(prev)->ri_data0 +
-		           lgfs2_rgrp_index(prev)->ri_data;
+		lgfs2_rindex_out(prev, &ri);
+		prev_end = be64_to_cpu(ri.ri_data0) + be32_to_cpu(ri.ri_data);
 	}
-	while (prev_end < ri->ri_addr) {
-		size_t gap_len = ri->ri_addr - prev_end;
+
+	lgfs2_rindex_out(rg, &ri);
+	addr = be64_to_cpu(ri.ri_addr);
+
+	while (prev_end < addr) {
+		size_t gap_len = addr - prev_end;
 
 		if (gap_len > IOV_MAX)
 			gap_len = IOV_MAX;
@@ -802,12 +896,12 @@ static int place_rgrp(struct gfs2_sbd *sdp, lgfs2_rgrp_t rg, int debug)
 		return -1;
 	}
 	if (debug) {
-		gfs2_rindex_print(ri);
+		rindex_print(&ri);
 		printf("\n");
 	}
-	sdp->blks_total += ri->ri_data;
-	sdp->fssize = ri->ri_data0 + ri->ri_data;
-	sdp->rgrps++;
+	sdp->blks_total += be32_to_cpu(ri.ri_data);
+	sdp->fssize = be64_to_cpu(ri.ri_data0) + be32_to_cpu(ri.ri_data);
+	nrgrp++;
 	return 0;
 }
 
@@ -836,11 +930,11 @@ static int add_rgrp(lgfs2_rgrps_t rgs, uint64_t *addr, uint32_t len, lgfs2_rgrp_
 	return 0;
 }
 
-static int place_journals(struct gfs2_sbd *sdp, lgfs2_rgrps_t rgs, struct mkfs_opts *opts, uint64_t *rgaddr)
+static int place_journals(struct lgfs2_sbd *sdp, lgfs2_rgrps_t rgs, struct mkfs_opts *opts, uint64_t *rgaddr)
 {
 	struct gfs2_progress_bar progress;
-	uint64_t jfsize = lgfs2_space_for_data(sdp, sdp->bsize, opts->jsize << 20);
-	uint32_t rgsize = lgfs2_rgsize_for_data(jfsize, sdp->bsize);
+	uint64_t jfsize = lgfs2_space_for_data(sdp, sdp->sd_bsize, opts->jsize << 20);
+	uint32_t rgsize = lgfs2_rgsize_for_data(jfsize, sdp->sd_bsize);
 	unsigned j;
 
 	gfs2_progress_init(&progress, opts->journals, _("Adding journals: "), opts->quiet);
@@ -855,7 +949,8 @@ static int place_journals(struct gfs2_sbd *sdp, lgfs2_rgrps_t rgs, struct mkfs_o
 	for (j = 0; j < opts->journals; j++) {
 		int result;
 		lgfs2_rgrp_t rg;
-		struct gfs2_inode in = {0};
+		struct lgfs2_inode in = {0};
+		struct gfs2_rindex ri;
 
 		gfs2_progress_update(&progress, (j + 1));
 
@@ -874,7 +969,8 @@ static int place_journals(struct gfs2_sbd *sdp, lgfs2_rgrps_t rgs, struct mkfs_o
 			return result;
 		}
 		/* Allocate at the beginning of the rgrp, bypassing extent search */
-		in.i_di.di_num.no_addr = lgfs2_rgrp_index(rg)->ri_data0;
+		lgfs2_rindex_out(rg, &ri);
+		in.i_num.in_addr = be64_to_cpu(ri.ri_data0);
 		/* In order to keep writes sequential here, we have to allocate
 		   the journal, then write the rgrp header (which is now in its
 		   final form) and then write the journal out */
@@ -884,9 +980,6 @@ static int place_journals(struct gfs2_sbd *sdp, lgfs2_rgrps_t rgs, struct mkfs_o
 			return result;
 		}
 
-		if (opts->debug)
-			gfs2_dinode_print(&in.i_di);
-
 		result = place_rgrp(sdp, rg, opts->debug);
 		if (result != 0)
 			return result;
@@ -895,26 +988,28 @@ static int place_journals(struct gfs2_sbd *sdp, lgfs2_rgrps_t rgs, struct mkfs_o
 
 		result = lgfs2_write_filemeta(&in);
 		if (result != 0) {
-			fprintf(stderr, _("Failed to write journal %u\n"), j);
+			fprintf(stderr, _("Failed to write journal %u: %s\n"),
+			        j, strerror(errno));
 			return result;
 		}
 
 		result = lgfs2_write_journal_data(&in);
 		if (result != 0) {
-			fprintf(stderr, _("Failed to write data blocks for journal %u\n"), j);
+			fprintf(stderr, _("Failed to write data blocks for journal %u: %s\n"),
+			        j, strerror(errno));
 			return result;
 		}
-		mkfs_journals[j] = in.i_di.di_num;
+		mkfs_journals[j] = in.i_num;
 	}
 	gfs2_progress_close(&progress, _("Done\n"));
 
 	return 0;
 }
 
-static int place_rgrps(struct gfs2_sbd *sdp, lgfs2_rgrps_t rgs, uint64_t *rgaddr, struct mkfs_opts *opts)
+static int place_rgrps(struct lgfs2_sbd *sdp, lgfs2_rgrps_t rgs, uint64_t *rgaddr, struct mkfs_opts *opts)
 {
 	struct gfs2_progress_bar progress;
-	uint32_t rgblks = ((opts->rgsize << 20) / sdp->bsize);
+	uint32_t rgblks = ((opts->rgsize << 20) / sdp->sd_bsize);
 	uint32_t rgnum;
 	int result;
 
@@ -934,7 +1029,7 @@ static int place_rgrps(struct gfs2_sbd *sdp, lgfs2_rgrps_t rgs, uint64_t *rgaddr
 			fprintf(stderr, _("Failed to build resource groups\n"));
 			return result;
 		}
-		gfs2_progress_update(&progress, (sdp->rgrps));
+		gfs2_progress_update(&progress, nrgrp);
 	}
 	if (lgfs2_rgrps_write_final(sdp->device_fd, rgs) != 0) {
 		perror(_("Failed to write final resource group"));
@@ -945,6 +1040,24 @@ static int place_rgrps(struct gfs2_sbd *sdp, lgfs2_rgrps_t rgs, uint64_t *rgaddr
 	return 0;
 }
 
+static int create_jindex(struct lgfs2_sbd *sdp, struct mkfs_opts *opts, struct lgfs2_inum *jnls)
+{
+	struct lgfs2_inode *jindex;
+
+	jindex = lgfs2_build_jindex(sdp->master_dir, jnls, opts->journals);
+	if (jindex == NULL) {
+		fprintf(stderr, _("Error building '%s': %s\n"), "jindex", strerror(errno));
+		return 1;
+	}
+	if (opts->debug) {
+		printf("Jindex:\n");
+		dinode_print(jindex->i_bh->b_data);
+	}
+	lgfs2_inode_put(&jindex);
+	return 0;
+}
+
+
 /*
  * Find a reasonable journal file size (in blocks) given the number of blocks
  * in the filesystem.  For very small filesystems, it is not reasonable to
@@ -956,7 +1069,7 @@ static int place_rgrps(struct gfs2_sbd *sdp, lgfs2_rgrps_t rgs, uint64_t *rgaddr
  */
 static int default_journal_size(unsigned bsize, uint64_t num_blocks)
 {
-	int min_blocks = (GFS2_MIN_JSIZE << 20) / bsize;
+	int min_blocks = (LGFS2_MIN_JSIZE << 20) / bsize;
 
 	if (num_blocks < 2 * min_blocks)
 		return -1;
@@ -975,29 +1088,42 @@ static int default_journal_size(unsigned bsize, uint64_t num_blocks)
 	return 262144;                          /*   1 GB */
 }
 
-static int sbd_init(struct gfs2_sbd *sdp, struct mkfs_opts *opts, unsigned bsize)
+static int sbd_init(struct lgfs2_sbd *sdp, struct mkfs_opts *opts, unsigned bsize)
 {
-	memset(sdp, 0, sizeof(struct gfs2_sbd));
-	sdp->time = time(NULL);
+	memset(sdp, 0, sizeof(struct lgfs2_sbd));
+	sdp->sd_time = time(NULL);
 	sdp->rgtree.osi_node = NULL;
 	sdp->rgsize = opts->rgsize;
 	sdp->qcsize = opts->qcsize;
 	sdp->jsize = opts->jsize;
 	sdp->md.journals = opts->journals;
 	sdp->device_fd = opts->dev.fd;
-	sdp->bsize = sdp->sd_sb.sb_bsize = bsize;
+	sdp->sd_bsize = bsize;
+	sdp->sd_fs_format = opts->format;
+	sdp->sd_multihost_format = GFS2_FORMAT_MULTI;
+	sdp->sd_bsize = bsize;
+	sdp->sd_bsize_shift = ffs(bsize) - 1;
 
-	if (compute_constants(sdp)) {
+	if (opts->got_uuid) {
+		int err = uuid_parse(opts->uuid, sdp->sd_uuid);
+		if (err != 0) {
+			fprintf(stderr, _("Failed to parse UUID option."));
+			return -1;
+		}
+	} else
+		uuid_generate(sdp->sd_uuid);
+
+	if (lgfs2_compute_constants(sdp)) {
 		perror(_("Failed to compute file system constants"));
 		return -1;
 	}
-	sdp->device.length = opts->dev.size / sdp->bsize;
+	sdp->device.length = opts->dev.size / sdp->sd_bsize;
 	if (opts->got_fssize) {
 		if (opts->fssize > sdp->device.length) {
 			fprintf(stderr, _("Specified size is bigger than the device."));
 			fprintf(stderr, "%s %.2f %s (%"PRIu64" %s)\n", _("Device size:"),
 			       opts->dev.size / ((float)(1 << 30)), _("GB"),
-			       opts->dev.size / sdp->bsize, _("blocks"));
+			       opts->dev.size / sdp->sd_bsize, _("blocks"));
 			return -1;
 		}
 		sdp->device.length = opts->fssize;
@@ -1006,23 +1132,23 @@ static int sbd_init(struct gfs2_sbd *sdp, struct mkfs_opts *opts, unsigned bsize
 	   makes sense for the device size, or set a sensible default, if one
 	   will fit. For user-provided journal sizes, limit it to half of the fs. */
 	if (!opts->got_jsize) {
-		int default_jsize = default_journal_size(sdp->bsize, sdp->device.length / opts->journals);
+		int default_jsize = default_journal_size(sdp->sd_bsize, sdp->device.length / opts->journals);
 		unsigned jsize_mb;
 
 		if (default_jsize < 0) {
 			fprintf(stderr, _("gfs2 will not fit on this device.\n"));
 			return -1;
 		}
-		jsize_mb = (default_jsize * sdp->bsize) >> 20;
-		if (jsize_mb < GFS2_MIN_JSIZE)
-			opts->jsize = GFS2_MIN_JSIZE;
+		jsize_mb = (default_jsize * sdp->sd_bsize) >> 20;
+		if (jsize_mb < LGFS2_MIN_JSIZE)
+			opts->jsize = LGFS2_MIN_JSIZE;
 		else
 			opts->jsize = jsize_mb;
-	} else if ((((opts->jsize * opts->journals) << 20) / sdp->bsize) > (sdp->device.length / 2)) {
-		unsigned max_jsize = (sdp->device.length / 2 * sdp->bsize / opts->journals) >> 20;
+	} else if ((((opts->jsize * opts->journals) << 20) / sdp->sd_bsize) > (sdp->device.length / 2)) {
+		unsigned max_jsize = (sdp->device.length / 2 * sdp->sd_bsize / opts->journals) >> 20;
 
 		fprintf(stderr, _("gfs2 will not fit on this device.\n"));
-		if (max_jsize >= GFS2_MIN_JSIZE)
+		if (max_jsize >= LGFS2_MIN_JSIZE)
 			fprintf(stderr, _("Maximum size for %u journals on this device is %uMB.\n"),
 			        opts->journals, max_jsize);
 		return -1;
@@ -1091,7 +1217,7 @@ static int open_dev(struct mkfs_dev *dev, int withprobe)
 	}
 
 	/* Freshen up the cache */
-	posix_fadvise(dev->fd, 0, 0, POSIX_FADV_DONTNEED);
+	(void)posix_fadvise(dev->fd, 0, 0, POSIX_FADV_DONTNEED);
 
 	error = fstat(dev->fd, &dev->stat);
 	if (error < 0) {
@@ -1119,9 +1245,10 @@ static int open_dev(struct mkfs_dev *dev, int withprobe)
 #ifndef UNITTESTS
 int main(int argc, char *argv[])
 {
-	struct gfs2_sbd sbd;
-	struct gfs2_sb sb;
+	struct gfs2_statfs_change sc;
+	struct lgfs2_sbd sbd;
 	struct mkfs_opts opts;
+	struct lgfs2_inode *ip;
 	lgfs2_rgrps_t rgs;
 	uint64_t rgaddr;
 	int error;
@@ -1151,13 +1278,11 @@ int main(int argc, char *argv[])
 	if (S_ISREG(opts.dev.stat.st_mode)) {
 		opts.got_bsize = 1; /* Use default block size for regular files */
 	}
-
 	if (sbd_init(&sbd, &opts, bsize) != 0)
 		exit(-1);
-	lgfs2_sb_init(&sb, bsize, opts.format);
 	if (opts.debug) {
 		printf(_("File system options:\n"));
-		printf("  bsize = %u\n", sbd.bsize);
+		printf("  bsize = %u\n", sbd.sd_bsize);
 		printf("  qcsize = %u\n", sbd.qcsize);
 		printf("  jsize = %u\n", sbd.jsize);
 		printf("  journals = %u\n", sbd.md.journals);
@@ -1200,70 +1325,102 @@ int main(int argc, char *argv[])
 		fprintf(stderr, _("Failed to build resource groups\n"));
 		exit(1);
 	}
-	sbd.rgtree.osi_node = lgfs2_rgrps_root(rgs); // Temporary
+	lgfs2_attach_rgrps(&sbd, rgs); // Temporary
 
-	error = build_master(&sbd);
+	error = lgfs2_build_master(&sbd);
 	if (error) {
 		fprintf(stderr, _("Error building '%s': %s\n"), "master", strerror(errno));
 		exit(EXIT_FAILURE);
 	}
-	sb.sb_master_dir = sbd.master_dir->i_di.di_num;
+	if (opts.debug) {
+		printf("Metafs inode:\n");
+		dinode_print(sbd.master_dir->i_bh->b_data);
+	}
+	sbd.sd_meta_dir = sbd.master_dir->i_num;
 
-	error = lgfs2_build_jindex(sbd.master_dir, mkfs_journals, opts.journals);
-	if (error) {
-		fprintf(stderr, _("Error building '%s': %s\n"), "jindex", strerror(errno));
-		exit(EXIT_FAILURE);
-	}
+	error = create_jindex(&sbd, &opts, mkfs_journals);
 	free(mkfs_journals);
-	error = build_per_node(&sbd);
-	if (error) {
-		fprintf(stderr, _("Error building '%s': %s\n"), "per_node", strerror(errno));
-		exit(EXIT_FAILURE);
-	}
-	error = build_inum(&sbd);
-	if (error) {
+	if (error != 0)
+		exit(1);
+
+	error = build_per_node(&sbd, &opts);
+	if (error != 0)
+		exit(1);
+
+	sbd.md.inum = lgfs2_build_inum(&sbd);
+	if (sbd.md.inum == NULL) {
 		fprintf(stderr, _("Error building '%s': %s\n"), "inum", strerror(errno));
 		exit(EXIT_FAILURE);
 	}
-	gfs2_lookupi(sbd.master_dir, "inum", 4, &sbd.md.inum);
-	error = build_statfs(&sbd);
-	if (error) {
+	if (opts.debug) {
+		printf("\nInum Inode:\n");
+		dinode_print(sbd.md.inum->i_bh->b_data);
+	}
+	sbd.md.statfs = lgfs2_build_statfs(&sbd);
+	if (sbd.md.statfs == NULL) {
 		fprintf(stderr, _("Error building '%s': %s\n"), "statfs", strerror(errno));
 		exit(EXIT_FAILURE);
 	}
-	gfs2_lookupi(sbd.master_dir, "statfs", 6, &sbd.md.statfs);
-	error = build_rindex(&sbd);
-	if (error) {
+	if (opts.debug) {
+		printf("\nStatFS Inode:\n");
+		dinode_print(sbd.md.statfs->i_bh->b_data);
+	}
+	ip = lgfs2_build_rindex(&sbd);
+	if (ip == NULL) {
 		fprintf(stderr, _("Error building '%s': %s\n"), "rindex", strerror(errno));
 		exit(EXIT_FAILURE);
 	}
+	if (opts.debug) {
+		printf("\nResource Index:\n");
+		dinode_print(ip->i_bh->b_data);
+	}
+	lgfs2_inode_put(&ip);
 	if (!opts.quiet) {
 		printf("%s", _("Creating quota file: "));
 		fflush(stdout);
 	}
-	error = build_quota(&sbd);
-	if (error) {
+	ip = lgfs2_build_quota(&sbd);
+	if (ip == NULL) {
 		fprintf(stderr, _("Error building '%s': %s\n"), "quota", strerror(errno));
 		exit(EXIT_FAILURE);
 	}
+	if (opts.debug) {
+		printf("\nQuota:\n");
+		dinode_print(ip->i_bh->b_data);
+	}
+	lgfs2_inode_put(&ip);
 	if (!opts.quiet)
 		printf("%s", _("Done\n"));
 
-	build_root(&sbd);
-	sb.sb_root_dir = sbd.md.rooti->i_di.di_num;
+	lgfs2_build_root(&sbd);
+	if (opts.root_inherit_jd) {
+		sbd.md.rooti->i_flags |= GFS2_DIF_INHERIT_JDATA;
+		lgfs2_dinode_out(sbd.md.rooti, sbd.md.rooti->i_bh->b_data);
+	}
+	if (opts.debug) {
+		printf("\nRoot directory:\n");
+		dinode_print(sbd.md.rooti->i_bh->b_data);
+	}
+	sbd.sd_root_dir = sbd.md.rooti->i_num;
 
-	strncpy(sb.sb_lockproto, opts.lockproto, GFS2_LOCKNAME_LEN - 1);
-	strncpy(sb.sb_locktable, opts.locktable, GFS2_LOCKNAME_LEN - 1);
-	sb.sb_lockproto[GFS2_LOCKNAME_LEN - 1] = '\0';
-	sb.sb_locktable[GFS2_LOCKNAME_LEN - 1] = '\0';
+	strncpy(sbd.sd_lockproto, opts.lockproto, GFS2_LOCKNAME_LEN - 1);
+	strncpy(sbd.sd_locktable, opts.locktable, GFS2_LOCKNAME_LEN - 1);
+	sbd.sd_lockproto[GFS2_LOCKNAME_LEN - 1] = '\0';
+	sbd.sd_locktable[GFS2_LOCKNAME_LEN - 1] = '\0';
 
-	do_init_inum(&sbd);
-	do_init_statfs(&sbd);
+	lgfs2_init_inum(&sbd);
+	if (opts.debug)
+		printf("\nNext Inum: %"PRIu64"\n", sbd.md.next_inum);
 
-	inode_put(&sbd.md.rooti);
-	inode_put(&sbd.master_dir);
-	inode_put(&sbd.md.inum);
-	inode_put(&sbd.md.statfs);
+	lgfs2_init_statfs(&sbd, &sc);
+	if (opts.debug) {
+		printf("\nStatfs:\n");
+		statfs_change_print(&sc);
+	}
+	lgfs2_inode_put(&sbd.md.rooti);
+	lgfs2_inode_put(&sbd.master_dir);
+	lgfs2_inode_put(&sbd.md.inum);
+	lgfs2_inode_put(&sbd.md.statfs);
 
 	lgfs2_rgrps_free(&rgs);
 
@@ -1272,7 +1429,7 @@ int main(int argc, char *argv[])
 		fflush(stdout);
 	}
 
-	error = lgfs2_sb_write(&sb, opts.dev.fd, sbd.bsize);
+	error = lgfs2_sb_write(&sbd, opts.dev.fd);
 	if (error) {
 		perror(_("Failed to write superblock\n"));
 		exit(EXIT_FAILURE);
@@ -1291,7 +1448,7 @@ int main(int argc, char *argv[])
 
 	if (!opts.quiet) {
 		printf("%s", _("Done\n"));
-		print_results(&sb, &opts, sbd.rgrps, sbd.fssize);
+		print_results(&sbd, &opts);
 	}
 	return 0;
 }
